/**
 * 盒子端文章内容展示组件
 * 可以分页或者滚动浏览文章，分页是利用CSS模拟分页
 * 依赖:/swatapps/ape/evm.js
 * 作者:wuqq@evmtv.com
 * 版本:v1.5.0
 * 创建时间:2016-03-23
 *
 * v1.1.0 2016-06-22
 * 1.新增ext配置属性
 * v1.2.0 2016-07-26
 * 1.增加对新的翻页模式支持pagingspec，外框和内部框顶部或者底部对齐
 * 2.文章组件初始化之后，可以调用appendContent追加内容
 * 3.新增属性和方法对组件的DOM隐藏和显示控制
 * 4.新增属性enabled，用于标识包含内容并且已经初始化
 * v1.3.0 2016-10-21
 * 1.新增onFlipover、setPageIndex方法
 * v1.3.1 2016-11-08
 * 上下翻页操作增加对type配置属性支持，如pagingspec 翻页，外框和内部框顶部或者底部对齐
 * v1.3.2 2016-12-09
 * 调整初始化enabled设置的条件
 * v1.4.1 2017-04-29
 * 1.增加控制强制刷新图片属性
 * 2.新增自定义事件，表示所有图片加载完成
 * 3.todo:pagingspec方式翻页，图片显示需要处理可能被截断的文字，移动step长度可以设置一定的安全长度
 * v1.5.0 2017-05-05
 * 1.增加goto操作，跳转到指定的页数
 */
(function (window) {
    //定义图文展示组件类
    function Article(options) {
        //类只能被实例化
        if (this === evm.ext.comp || this === window) {
            return new arguments.callee(options);
        }
        //默认配置参数
        var defaults = {
            /*基类配置参数*/
            prefix: 'Ape',
            compName: 'Article',
            parentComp: null,
            /*类本身配置参数*/
            //绝对定位位置属性
            baseLeft: 0,
            baseTop: 0,
            //容器元素属性
            boxId: '',
            boxHeight: 0,
            boxWidth: 0,
            //行高
            lineHeight: 36,
            //文章内容的html
            content: '',
            //文章分页方式
            //scroll 滚动-未实现
            //paging 翻页
            //pagingspec 翻页，外框和内部框顶部或者底部对齐
            type: 'paging',
            //分页单步移动距离
            step: 0,
            //自定义操作
            //@param {string} method 自定义事件名称
            //@param {object} args 返回一个对象
            userEvent: function (method, args) {
            },
            enabled: false,
            isVisible: true,
            isActive: false,
            //用于扩展方法和属性
            ext: {},
            //强制图片刷新
            //v1.4.1
            forceLoadImage: true
        };
        evm.extend(defaults, options || {});
        evm.extend(this, defaults);
        //继承基类
        evm.ext.comp.BaseComponent.call(this, defaults);
        //私有属性
        //容器元素
        this._boxElm = null;
        //容器子元素
        this._innerElm = null;
        this._innerHeight = 0;
        //标识所有图片加载完成
        //用于计算innerElm的正确高度
        this._allImagesLoadFinished = false;
        //分页属性
        this._pageIndex = 1;
        this._pageTotalNum = 1;
        //分页按照step高度计算
        this._pageIndexByStep = 1;
        this._pageTotalNumByStep = 1;
        //滚动配置属性
        this.dir;
        this.factor = 1;
        this.max = 0;
        this.min = 0;
    }

    evm.extend(Article.prototype, {
        _adjustStep: function () {
            if (this.type === 'paging' || this.type === 'pagingspec') {
                if (this.step <= 0 || this.step >= this.boxHeight) {
                    this.step = this.boxHeight;
                }
                if (this.step > 0 && this.step < this.boxHeight) {
                    //处理每次移动步长为行高的整数倍
                    this.step = parseInt(this.step / this.lineHeight) * this.lineHeight;
                }
            }
        },
        _allImagesLoad: function (cb) {
            var self = this;
            if (!this._innerElm) {
                return;
            }
            var imgs = this._innerElm.getElementsByTagName('img');
            var loadImgCount = 0;
            var me = this;
            if (imgs.length) {
                for (var i = 0; i < imgs.length; i++) {
                    var img = imgs[i];
                    img.onload = function () {
                        var img = this;
                        //包含类属性article-excluded排除
                        if (!evm.hasClass(img, 'article-excluded')) {
                            //处理图片
                            //获取图片的宽度
                            var imgW = parseFloat(evm.getStyle(img, 'width') || img.clientWidth);
                            var imgH = parseFloat(evm.getStyle(img, 'height') || img.clientHeight);
                            if (imgW > me.boxWidth) {
                                //设置图片padding-bottom使整体高度为行高的整数倍
                                imgH = (imgH / imgW) * me.boxHeight;

                                evm.css(img,{
                                    width:me.boxWidth + 'px',
                                    paddingBottom:(Math.ceil(imgH / me.lineHeight) * me.lineHeight - imgH) + 'px'
                                });
                            }
                            else {
                                evm.css(img, 'padding-bottom:' + (Math.ceil(imgH / me.lineHeight) * me.lineHeight - imgH) + 'px');
                            }
                        }
                        loadImgCount++;
                        callback();
                    };
                    img.onerror = function () {
                        loadImgCount++;
                        callback();
                    };
                    if (this.forceLoadImage) {
                        //给图片链接添加时间戳
                        img.src = img.src + '?' + (new Date()).getTime();
                    }
                }
            }
            else {
                callback();
            }
            function callback() {
                //evm.util.common.logger.debug('<ext.comp.Article> _allImagesLoad() imgs.length = ' + imgs.length);
                //evm.util.common.logger.debug('<ext.comp.Article> _allImagesLoad() 加载图片的数量' + loadImgCount);
                if (loadImgCount === imgs.length) {
                    //所有图片加载完成
                    self._trigger('onImagesLoadFinished');
                    //ipanel3.0上面动态获取高度需要延时处理
                    setTimeout(function () {
                        me._allImagesLoadFinished = true;
                        me._innerHeight = parseFloat(evm.getStyle(me._innerElm, 'height')) || me._innerElm.clientHeight;
                        me._pageTotalNum = Math.ceil(me._innerHeight / me.boxHeight);
                        me._pageTotalNumByStep = Math.ceil(me._innerHeight / me.step);
                        if (me.type === 'pagingspec') {
                            me._pageTotalNum = Math.ceil((me._innerHeight - me.boxHeight) / me.boxHeight) + 1;
                            me._pageTotalNumByStep = Math.ceil((me._innerHeight - me.boxHeight) / me.step) + 1;
                        }
                        cb && cb();
                        me._trigger('onFlipover', {
                            pageIndex: me._pageIndex,
                            pageTotalNum: me._pageTotalNum,
                            pageTotalNumByStep: me._pageTotalNumByStep,
                            pageIndexByStep: me._pageIndexByStep
                        });
                    }, 200);
                }
            }
        },
        _checkInInit: function () {
            try {
                if (!this.content) {
                    throw new Error('Article.init:对象content属性不能为空!');
                }
                if (!this.boxId) {
                    throw new Error('Article.init:对象配置参数boxId不能为空!');
                }
                if (!evm.isNumber(this.lineHeight)) {
                    throw new Error('Article.init:对象配置参数lineHeight必须为数字类型!');
                }
                if (!evm.isNumber(this.boxHeight)) {
                    throw new Error('Article.init:对象配置参数boxHeight必须为数字类型!');
                }
                if (!evm.isNumber(this.step)) {
                    throw new Error('Article.init:对象配置参数step必须为数字类型!');
                }
            }
            catch (e) {
                evm.log(e.message);
            }
        },
        _buildView: function () {
            var innerElmStyle = 'position:absolute;'
                + 'left:0;'
                + 'top:0;'
                + 'width:' + this.boxWidth + 'px;';
            this._innerElm = evm.createElem('div', {
                className: 'inner',
                style: innerElmStyle
            }, this._boxElm);
            evm.html(this._innerElm, this.content);
        },
        /**
         * @param method
         * @param {object} obj
         * @returns {*}
         * @private
         */
        _trigger: function (method, obj) {
            var args = {};
            evm.extend(args, obj);
            if (evm.isFunction(this['userEvent'])) {
                return this['userEvent'].call(this, method, args);
            }
        },
        action: function (keyAlias, myEvent) {
            if (!this._allImagesLoadFinished) {
                return;
            }
            if (this.type !== 'paging' && this.type !== 'pagingspec') {
                return;
            }
            switch (keyAlias) {
                case 'UP':
                    this.prev();
                    break;
                case 'RIGHT':
                    this.onWidgetBlur(keyAlias);
                    break;
                case 'DOWN':
                    this.next();
                    break;
                case 'LEFT':
                    this.onWidgetBlur(keyAlias);
                    break;
                case 'SELECT':
                    this.onSelect();
                    break;
            }
        },
        getPageIndex: function () {
            return this._pageIndex;
        },
        getPageTotalNum: function () {
            return this._pageTotalNum;
        },
        init: function () {
            var self = this;
            this._trigger('onPreInit');
            this._checkInInit();
            this._pageIndex = 1;
            this._pageIndexByStep = 1;
            //可视高度调整为行高的整数倍，避免文字横向截断
            this.boxHeight = parseInt(this.boxHeight / this.lineHeight) * this.lineHeight;
            this._boxElm = evm.$(this.boxId);
            var boxStyle = 'position:absolute;'
                + 'left:' + this.baseLeft + 'px;'
                + 'top:' + this.baseTop + 'px;'
                + 'width:' + this.boxWidth + 'px;'
                + 'height:' + this.boxHeight + 'px;'
                + 'line-height:' + this.lineHeight + 'px;'
                + 'display:' + (this.isVisible === true ? 'block' : 'none') + ';'
                + 'overflow:hidden;';
            if (!this._boxElm) {
                this._boxElm = evm.createElem('div', {id: this.boxId, style: boxStyle});
            }
            else {
                //清空容器
                this._boxElm.innerHTML = '';
                evm.css(this._boxElm, boxStyle);
            }
            this._buildView();
            this._adjustStep();
            this._allImagesLoad(function () {
                //判断有内容而且可翻页
                if (self._pageTotalNumByStep > 1) {
                    self.enabled = true;
                }
            });
            this._trigger('onInitComplete');
        },
        next: function () {
            var pi = this._pageIndexByStep;
            if (pi + 1 <= this._pageTotalNumByStep) {
                this._pageIndexByStep++;
                var innerTop = parseFloat(evm.getStyle(this._innerElm, 'top'));
                var step = this.step;
                if (pi + 1 === this._pageTotalNumByStep && pi - 1 >= 0 && this.type === 'pagingspec') {
                    step = this._innerHeight - this.boxHeight - (pi - 1) * this.step;
                }
                evm.css(this._innerElm, {
                    top: innerTop - step + 'px'
                });
                this._pageIndex = parseInt(Math.abs(innerTop - this.step) / this.boxHeight) + 1;
                this._trigger('onFlipover', {
                    pageIndex: this._pageIndex,
                    pageTotalNum: this._pageTotalNum,
                    pageTotalNumByStep: this._pageTotalNumByStep,
                    pageIndexByStep: this._pageIndexByStep,
                    dir: 'next'
                });
            }
            else {
                this.onWidgetBlur('DOWN');
            }
        },
        goto: function (pageIndexByStep) {
            if (+pageIndexByStep) {
                pageIndexByStep = Math.max(1, Math.min(this._pageTotalNumByStep, pageIndexByStep));
                var pi = pageIndexByStep;
                var innerTop = parseFloat(evm.getStyle(this._innerElm, 'top'));
                var step = this.step * (pi - this._pageIndexByStep);
                if (pi + 1 === this._pageTotalNumByStep && pi - 1 >= 0 && this.type === 'pagingspec') {
                    step = this._innerHeight - this.boxHeight - (pi - 1) * this.step;
                }

                evm.css(this._innerElm, {
                    top: innerTop - step + 'px'
                });

                this._pageIndex = parseInt(Math.abs(innerTop - step) / this.boxHeight) + 1;
                this._pageIndexByStep = pi;
                this._trigger('onFlipover', {
                    pageIndex: this._pageIndex,
                    pageTotalNum: this._pageTotalNum,
                    pageTotalNumByStep: this._pageTotalNumByStep,
                    pageIndexByStep: this._pageIndexByStep
                });
            }
        },
        prev: function () {
            var pi = this._pageIndexByStep;
            if (pi - 1 >= 1) {
                this._pageIndexByStep--;
                var innerTop = parseFloat(evm.getStyle(this._innerElm, 'top'));
                var step = this.step;
                if (pi === this._pageTotalNumByStep && pi - 2 >= 0 && this.type === 'pagingspec') {
                    step = this._innerHeight - this.boxHeight - (pi - 2) * this.step;
                }
                evm.css(this._innerElm, {
                    top: innerTop + step + 'px'
                });
                this._pageIndex = parseInt(Math.abs(innerTop + this.step) / this.boxHeight) + 1;
                this._trigger('onFlipover', {
                    pageIndex: this._pageIndex,
                    pageTotalNum: this._pageTotalNum,
                    pageTotalNumByStep: this._pageTotalNumByStep,
                    pageIndexByStep: this._pageIndexByStep,
                    dir: 'prev'
                });
            } else {
                this.onWidgetBlur('UP');
            }
        },
        onFocus: function () {
            this._trigger('onFocus');
        },
        onBlur: function () {
            this._trigger('onBlur');
        },
        /**
         * 组件获取焦点时候执行
         */
        onWidgetFocus: function () {
            this._trigger('onWidgetFocus');
            this.isActive = true;
            this.onFocus();
        },
        /**
         * 组件失去焦点时候执行操作
         * @param dir
         */
        onWidgetBlur: function (dir) {
            if (this._trigger('onWidgetBlur', {
                    alias: dir
                })) {
                this.isActive = false;
                this.onBlur();
            }
        },
        getPageTotalNumByStep: function () {
            return this._pageTotalNumByStep;
        },
        getPageIndexByStep: function () {
            return this._pageIndexByStep;
        },
        show: function () {
            this.isVisible = true;
            evm.util.common.show(this._boxElm);
        },
        hide: function () {
            this.isVisible = false;
            evm.util.common.hide(this._boxElm);
        },
        /**
         * 追加内容
         * @param html 内容为html形式的字符串
         * @param cb 追加内容完成执行回调函数
         */
        appendContent: function (html, cb) {
            var tempDom = evm.createElem('div', {}, this._boxElm),
                me = this;
            evm.html(tempDom, html);
            this._innerElm.appendChild(tempDom.firstChild);
            setTimeout(function () {
                //移除tempDom
                tempDom.parentNode.removeChild(tempDom);
                me._allImagesLoad(function () {
                    cb && cb();
                });
            }, 0);
        },
        /**
         * 按确定键操作
         */
        onSelect: function () {
            this._trigger('onSelect');
        },
        /**
         * 翻页操作后可用于修改页码等
         * @param dir
         */
        onFlipover: function (dir) {
            this._trigger('onFlipover', {
                pageIndex: this._pageIndex,
                pageTotalNum: this._pageTotalNum,
                pageTotalNumByStep: this._pageTotalNumByStep,
                pageIndexByStep: this._pageIndexByStep,
                dir: 'prev'
            });
        },
        /**
         * 修改当前页码数
         * @param pageIndex
         */
        setPageIndex: function (pageIndex) {
            pageIndex = +pageIndex;
            if (evm.isNumber(pageIndex)) {
                this._pageIndex = Math.min(this._pageTotalNum, Math.max(0, pageIndex));
            }
        },
        /**
         * 设置总页数
         * @param pageTotalNum
         */
        setPageTotalNum: function (pageTotalNum) {
            this._pageTotalNum = pageTotalNum;
        }
    });
    //外部调用
    evm.ext.comp.Article = Article;
})(this);
