/**
 * 盒子端图文展示组件
 * 可以分页或者滚动浏览文章，分页是利用CSS模拟分页
 * 依赖:/swatapps/ape/evm.js
 * 作者:wuqq@evmtv.com
 * 版本:v1.0
 * 创建时间:2016-03-23
 */
(function (window) {
	//定义图文展示组件类
	function Article(options) {
		//类只能被实例化
		if (this === evm.ext.comp || this === window) {
			return new arguments.callee(options);
		}

		//默认配置参数
		var defaults = {
			/*基类配置参数*/
			prefix: 'Ape',
			compName: 'Article',
			parentComp: null,
			/*类本身配置参数*/
			//绝对定位位置属性
			baseLeft: 0,
			baseTop: 0,
			//容器元素属性
			boxId: '',
			boxWidth: 0,
			boxHeight: 0,
			//行高
			lineHeight: 36,
			//文章内容的html
			content: '',
			//文章分页方式scroll或者paging
			type: 'paging',
			//自定义操作
			//@param {string} method 自定义事件名称
			//@param {object} args 返回一个对象
			userEvent: function (method, args) {
			}
		};

		evm.extend(defaults, options || {});
		evm.extend(this, defaults);
		//继承基类
		evm.ext.comp.BaseComponent.call(this, defaults);

		//私有属性
		//容器元素
		this._boxElm = null;
		//容器子元素
		this._innerElm = null;
		this._innerHeight = 0;
		//标识所有图片加载完成
		//用于计算innerElm的正确高度
		this._allImagesLoadFinished = false;
		//分页属性
		this._pageIndex = 1;
		this._pageTotalNum = 1;
		//滚动配置属性
		this.dir;
		this.step = 3;
		this.factor = 1;
		this.max = 0;
		this.min = 0;
	}

	evm.extend(Article.prototype, {
		/**
		 * @param method
		 * @param {object} obj
		 * @returns {*}
		 * @private
		 */
		_trigger: function (method, obj) {
			var args = {
				pageIndex:this._pageIndex
			}
			evm.extend(args, obj);

			if (evm.isFunction(this['userEvent'])) {
				return this['userEvent'].call(this, method, args);
			}
		},
		action: function (keyAlias,myEvent) {
			if (!this._allImagesLoadFinished) {
				return;
			}
			if(this.type === 'paging'){
				return;
			}
			switch(keyAlias){
				case 'UP':
					break;
				case 'RIGHT':
					break;
				case 'DOWN':
					break;
				case 'LEFT':
					break;
				case 'SELECT':
					break;
			}
		},
		init: function () {
			//没有内容直接返回
			if (!this.content) {
				throw new Error('组件对象content属性不能为空!');
			}

			if (!this.boxId) {
				throw new Error('配置参数boxId不能为空!');
			}
			this._boxElm = evm.$(this.boxId);
			var boxStyle = 'position:absolute;'
				+ 'left:' + this.baseLeft + 'px;'
				+ 'top:' + this.baseTop + 'px;'
				+ 'width:' + this.boxWidth + 'px;'
				+ 'height:' + this.boxHeight + 'px;'
				+ 'overflow:hidden;';
			if (!this._boxElm) {
				this._boxElm = evm.createElem('div', {id: this.boxId, style: boxStyle});
			}
			else {
				//清空容器
				evm.html(this._boxElm, '');
				evm.css(this._boxElm, boxStyle);
			}

			this._buildView();

			this._allImagesLoad();
		},
		_buildView: function () {
			var innerElmStyle = 'position:absolute;'
				+ 'left:0;'
				+ 'top:0;'
				+ 'width:' + this.boxWidth + 'px;';
			this._innerElm = evm.createElem('div', {
				className:'inner',
				style:innerElmStyle
			},this._boxElm);

			evm.html(this._innerElm,this.content);
		},
		_allImagesLoad:function(){
			if (!this._innerElm) {
				return;
			}
			var imgs = this._innerElm.getElementsByTagName('img');
			var loadImgCount = 1;
			var me = this;
			if(imgs.length){
				for (var i = 0; i < imgs.length; i++) {
					var img = imgs[i];
					img.onload = function () {
						if (loadImgCount === imgs.length) {
							//ipanel3.0上面动态获取高度需要延时处理
							setTimeout(function(){
								me._allImagesLoadFinished = true;
								me._innerHeight = parseFloat(evm.getStyle(me._innerElm, 'height')) || me._innerElm.clientHeight;
								me._pageTotalNum = Math.ceil(me._innerHeight / me._height);
								me._trigger('onFlipover', {
									pageTotalNum:me._pageTotalNum
								});

								//处理图片
								//获取图片的宽度
								var imgW = parseFloat(evm.getStyle(img,'width') || img.clientWidth);
								var imgH = parseFloat(evm.getStyle(img,'height') || img.clientHeight);
								if(imgW > me.boxWidth){
									evm.css(img,'width:'+me.boxWidth+'px;');

									//设置图片padding-bottom使整体高度为行高的整数倍
									imgH = (imgH/imgW)*me.boxHeight;

									evm.css(img,'padding-bottom:'+(Math.ceil(imgH/me.lineHeight)*me.lineHeight-imgH)+'px');
								}
								else{

									evm.css(img,'padding-bottom:'+(Math.ceil(imgH/me.lineHeight)*me.lineHeight-imgH)+'px');
								}
							},200);
						}
						loadImgCount++;
					};
					img.onerror = function () {
						loadImgCount++;
					}
				}
			}
			else{
				//ipanel3.0上面动态获取高度需要延时处理
				setTimeout(function(){
					me._allImagesLoadFinished = true;
					me._innerHeight = parseFloat(evm.getStyle(me._innerElm, 'height')) || me._innerElm.clientHeight;
					me._pageTotalNum = Math.ceil(me._innerHeight / me.boxHeight);
					me._trigger('onFlipover', {
						pageTotalNum:me._pageTotalNum
					});
				},200);
			}
		},
		prev:function(){
			var pi = this._pageIndex;
			if(pi - 1 >= 1){
				this._pageIndex--;
				this._trigger('onFlipover',{
					pageTotalNum:this._pageTotalNum,
					dir:'prev'
				});

				var innerTop = parseFloat(evm.getStyle(this._innerElm,'top'));
				evm.css(this._innerElm,{
					top : innerTop + this.boxHeight + 'px'
				});
			}
		},
		next:function(){
			var pi = this._pageIndex;
			if(pi + 1 <= this._pageTotalNum){
				this._pageIndex++;
				this._trigger('onFlipover',{
					pageTotalNum:this._pageTotalNum,
					dir:'next'
				});

				var innerTop = parseFloat(evm.getStyle(this._innerElm,'top'));
				evm.css(this._innerElm,{
					top : innerTop - this.boxHeight + 'px'
				});
			}
		}
	});

	//外部调用
	evm.ext.comp.Article = Article;
})(this);
