/**
 * 无规律磁贴块组件
 * @dependence:/swatapps/ape/evm.js
 * @author:wuqq@evmtv.com
 * @version:v1.0
 * @created:2016-03-17
 */

(function (window) {
	function NonRegularTiles(options) {
		//类只能被实例化
		if (this === evm.ext.comp || this === window) {
			return new arguments.callee(options);
		}

		//默认配置参数
		var defaults = {
			/*基类配置参数*/
			prefix: 'Ape',
			compName: 'NonRegularTiles',
			parentComp: null,
			/*类本身配置参数*/
			//无规律磁贴块数据
			data: [],
			fetchDataType: 'once',
			//配置获取数据接口
			fetchDataUrl: '',
			//数据结构可能不同，需要处理获取到的数据
			//默认给出一种栏目数据结构
			//注意：需要计算总页码数
			dataHandler: function (data) {
				if (evm.isString(data)) {
					data = eval('(' + data + ')');
				}

				this.setOriData(data);
				var node = data && data.node;
				var ret = null;
				if (node) {
					ret = data[node];
				}
				if (ret && ret instanceof Array) {
					//初始化分页
					if (this.fetchDataType === 'once') {
						var pageIndex = 0;
						var i = 0;//计数器，累加每页显示的数量
						while (i < ret.length) {
							var view = this.getView(pageIndex);
							pageIndex++;
							i += view.length;
						}

						this.pageTotalNum = pageIndex;
					}
					else if (this.fetchDataType === 'paging') {
						//总页码需要根据返回的总项数计算

					}
				}
				return ret;
			},
			//配置无规律磁贴块每页显示数量，位置
			//用二维数组view配置,二维数组每一项都是配置单页展示
			//默认配置view[0]
			/*
			 view[0] = [
			 {
			 left:0,
			 top:0,
			 width:0,
			 height:0
			 },
			 ...
			 ]
			 */
			view: [],
			//当前页中的焦点位置
			activeIndex: 0,
			/*分页属性*/
			pageIndex: 1,
			pageTotalNum: 1,
			//自动翻页标识
			switchAutoPaging: true,
			//自动翻页方向H，水平或者垂直V
			autoPagingDir: 'H',
			//容器元素属性
			boxId: '',
			boxLeft: 0,
			boxTop: 0,
			//列表数据处理
			model: [],
			//定义配置方法
			userEvent: function () {
			},
			//没有数据给出默认提示
			emptyMsg: '',
			emptyImage: '',
			//标识组件是否被选中状态
			enabled:false
		};
		evm.extend(defaults, options || {});
		evm.extend(this, defaults);
		//继承基类
		evm.ext.comp.BaseComponent.call(this, defaults);
		//私有属性
		//接口获取的原始数据对象
		this._oriData = null;
		//容器元素
		this._boxElm = null;
		//每页数据
		this._dataInPage = null;
		//记录上一次操作记录，用于互逆操作返回到刚才位置
		this._lastDir;
		this._lastActivePos;
	}

	evm.extend(NonRegularTiles.prototype, {
		/**
		 * @param method
		 * @param {object} obj
		 * @returns {*}
		 * @private
		 */
		_trigger: function (method, obj) {
			var args = {
				activeIndex: this.activeIndex,
				pageIndex: this.pageIndex
			}
			evm.extend(args, obj);

			if (!args.activeData) {
				var activeData = this._dataInPage && this._dataInPage[this.activeIndex];
				args.activeData = activeData;
			}
			if (!args.activeElm) {
				var activeElm = this.getActiveElm();
				args.activeElm = activeElm;
			}

			if (evm.isFunction(this['userEvent'])) {
				return this['userEvent'].call(this, method, args);
			}
		},
		//获取view数组成员
		getView: function (index) {
			if (!this.view || !(this.view instanceof Array) || this.view.length < 1) {
				throw new Error('未正确配置view');
			}
			var view = this.view[index];
			if (!view) {
				view = this.view[0];
			}

			return view;
		},
		/**
		 * 获取单页显示数据
		 * @private
		 * @return 未找到数据返回null
		 */
		_getDataByPage: function () {
			var data = null,
				start = 0,
				end = 0,
				i = 0;

			if (!this.data || !(this.data instanceof Array) || !this.data.length) {
				return data;
			}
			if (this.fetchDataType === 'once') {
				for (; i < this.pageIndex; i++) {
					var view = this.getView(i);

					if (i < this.pageIndex - 1) {
						start += view.length;
					}
					end += view.length;
				}
			}
			else if (this.fetchDataType === 'paging') {
				var view = this.getView(this.pageIndex - 1);

				end = view.length;
			}

			if (start !== end) {
				data = this.data.slice(start, end);
			}
			return data;
		},
		/**
		 * 构建展示视图
		 * @param {Array} dataInPage 必须
		 * @private
		 */
		_buildView: function () {
			var self = this;

			function fill(elm) {
				var view = self.getView(self.pageIndex - 1),
					model = self.model,
					i = 0,
					iLen = self._dataInPage.length,
					j = 0,
					jLen = model.length,
					html = '',
					prop;
				for (; i < iLen; i++) {
					var itemStyle = 'position:absolute;'
						+ 'left:' + view[i].left + 'px;'
						+ 'top:' + view[i].top + 'px;'
						+ 'width:' + view[i].width + 'px;'
						+ 'height:' + view[i].height + 'px;';
					if(view[i].picBlur){
						itemStyle += 'background:transparent url('+ view[i].picBlur +') no-repeat left top';
					}

					var itemClassName = self.class_name +'-item';
					html += '<div id="' + self.comp_name + '-p' + self.pageIndex +'-item-' + i + '" class="' + itemClassName + '" style="' + itemStyle + '">';
					j = 0;
					if (jLen > 0) {
						for (; j < jLen; j++) {
							if (model[j].text) {
								prop = self._dataInPage[i][model[j].text];
							}
							else {
								throw new Error('需要配置model参数!');
							}
							if (model[j].callback && evm.isFunction(model[j].callback)) {
								prop = model[j].callback.call(self, prop, i, self._dataInPage, model[j].text);
							}
							html += prop;
						}
					}
					else {
						throw new Error('需要配置model参数!');
					}
					html += '</div>';
				}
				evm.html(elm, html);
			}

			if (this._dataInPage && this._dataInPage instanceof Array) {
				var viewElm = evm.$(this.comp_name + '-view-p' + this.pageIndex);
				if (!viewElm) {
					viewElm = evm.createElem('div', {
						id: this.comp_name + '-view-p' + this.pageIndex,
						className: this.class_name + '-view',
						style: 'position:absolute;left:0;top:0;'
					}, this._boxElm);
					fill(viewElm);
				}
				else {
					if (this.fetchDataType === 'once') {
						this.showView(this.pageIndex);
					}
					else if (this.fetchDataType === 'paging') {
						evm.html(viewElm, '');
						fill(viewElm);
					}
				}
			}
		},
		_fetchData: function (callback) {
			this._trigger('onPreFetchData');
			var self = this;
			if (this.fetchDataUrl) {
				if (this.fetchDataType === 'paging' || (this.fetchDataType === 'once' && !this._oriData)) {
					evm.ajax(this.fetchDataUrl, function (data) {
						self.data = self.dataHandler(data);
						callback.call(self);
					});
				}
				else {
					callback.call(self);
				}
			}
		},
		_findAvailablePos: function (keyAlias) {
			var self = this;

			/**
			 * 返回true表示找到更好的合适块
			 * @param currentPos
			 * @param availablePos
			 * @returns {*}
			 */
			function compare(currentPos, availablePos) {
				if (availablePos === -1) {
					return true;
				}
				//比较当前块和找到的最优块中心点与当前焦点块中心点距离，取最小者
				//当前焦点块中心点坐标
				var cx = x + view[self.activeIndex].width / 2;
				var cy = y + view[self.activeIndex].height / 2;
				//当前块中心点坐标
				var cx1 = view[currentPos].left + view[currentPos].width / 2;
				var cy1 = view[currentPos].top + view[currentPos].height / 2;
				//最优块中心点坐标
				var cx2 = view[availablePos].left + view[availablePos].width / 2;
				var cy2 = view[availablePos].top + view[availablePos].height / 2;

				if (keyAlias === 'LEFT' || keyAlias === 'RIGHT') {
					if (Math.abs(cy1 - cy) < Math.abs(cy2 - cy) && Math.abs(cx1 - cx) <= Math.abs(cx2 - cx)) {
						return true;
					}
					else if (Math.abs(cy1 - cy) === Math.abs(cy2 - cy) && Math.abs(cx1 - cx) <= Math.abs(cx2 - cx)) {
						return true;
					}
				}
				else if (keyAlias === 'UP' || keyAlias === 'DOWN') {
					if (Math.abs(cx1 - cx) < Math.abs(cx2 - cx) && Math.abs(cy1 - cy) <= Math.abs(cy2 - cy)) {
						return true;
					}
					else if (Math.abs(cx1 - cx) <= Math.abs(cx2 - cx) && Math.abs(cy1 - cy) <= Math.abs(cy2 - cy)) {
						return true;
					}
				}
				return false;
			}

			var i = 0,
				view = this.getView(this.pageIndex - 1),
				x = view[this.activeIndex].left,
				y = view[this.activeIndex].top,
				dx = 0,
				dy = 0,
				available = null,
				availablePos = -1;
			for (; i < this._dataInPage.length; i++) {
				if (i === this.activeIndex) {
					continue;
				}
				var item = this._dataInPage[i];

				dx = view[i].left - x;
				dy = view[i].top - y;

				if (keyAlias === 'LEFT') {
					dx += view[i].width;
					if (dx <= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
				else if (keyAlias === 'RIGHT') {
					dx -= view[this.activeIndex].width;
					if (dx >= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
				else if (keyAlias === 'UP') {
					dy += view[i].height;
					if (dy <= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
				else if (keyAlias === 'DOWN') {
					dy -= view[this.activeIndex].height;
					if (dy >= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
			}
			return availablePos;
		},
		_move: function (keyAlias, myEvent) {
			var availablePos;
			if (this._lastDir === 'LEFT' && keyAlias === 'RIGHT' || this._lastDir === 'RIGHT' && keyAlias === 'LEFT') {
				availablePos = this._lastActivePos;
			}
			else if (this._lastDir === 'UP' && keyAlias === 'DOWN' || this._lastDir === 'DOWN' && keyAlias === 'UP') {
				availablePos = this._lastActivePos;
			}
			else {
				availablePos = this._findAvailablePos(keyAlias);
			}

			if (availablePos !== -1 && availablePos !== this.activeIndex) {
				this._lastDir = keyAlias;
				this._lastActivePos = this.activeIndex;

				this.onBlur();
				this.activeIndex = availablePos;
				this.onFocus();
			}
			else {
				if (this.switchAutoPaging) {
					if (this.autoPagingDir === 'H') {
						if (keyAlias === 'LEFT' && this.pageIndex > 1) {
							this.prev();
						}
						else if (keyAlias === 'RIGHT' && this.pageIndex < this.pageTotalNum) {
							this.next();
						}
						else {
							this.onWidgetBlur(keyAlias);
						}
					}
					else if (this.autoPagingDir === 'V') {
						if (keyAlias === 'UP' && this.pageIndex > 1) {
							this.prev();
						}
						else if (keyAlias === 'DOWN' && this.pageIndex < this.pageTotalNum) {
							this.next();
						}
						else {
							this.onWidgetBlur(keyAlias);
						}
					}
					else {
						this.onWidgetBlur(keyAlias);
					}
				}
				else {
					this.onWidgetBlur(keyAlias);
				}
			}

		},
		action: function (keyAlias, myEvent) {
			switch (keyAlias) {
				case 'UP':
				case 'RIGHT':
				case 'DOWN':
				case 'LEFT':
					this._move(keyAlias, myEvent);
					break;
				case 'SELECT':
					this.onSelect();
					break;
			}
		},
		init: function () {
			var self = this;
			this._fetchData(function () {
				//准备容器元素
				var parentElm = evm.$('container') || document.body;
				if (!evm.isString(self.boxId)) {
					self.boxId = self.comp_name;
				}
				self._boxElm = evm.$(self.boxId);
				var boxStyle = 'position:absolute;'
					+ 'left:' + self.boxLeft + 'px;'
					+ 'top:' + self.boxTop + 'px;';
				if (!self._boxElm) {
					self._boxElm = evm.createElem('div', {
						id: self.boxId,
						className: self.class_name,
						style: boxStyle
					});
				}
				else {
					evm.css(self._boxElm, boxStyle);
				}

				var pageData = self._dataInPage = self._getDataByPage();

				if(self.emptyMsg || self.emptyImage){
					if (!pageData) {
						pageData = [
							{
								text: self.emptyMsg,
								picBlur: self.emptyImage
							}
						];
					}
				}

				self._buildView();

				self._trigger('onInitComplete');

				if(pageData){
					self.enabled = true;
				}
			});
		},
		setOriData: function (data) {
			this._oriData = data;
		},
		getOriData: function () {
			return this._oriData;
		},
		prev: function () {
			var self = this;

			this.onBlur();
			this.hideView(this.pageIndex);
			this.pageIndex--;
			this._lastDir = '';
			this._fetchData(function () {
				var dataInPage = self._dataInPage = self._getDataByPage();
				if (dataInPage) {
					self._buildView();

					self.activeIndex = dataInPage.length - 1;
					this.showView(this.pageIndex);
					self.onFocus();
				}
			});
		},
		next: function () {
			var self = this;

			this.onBlur();
			this.hideView(this.pageIndex);
			this.pageIndex++;
			this._lastDir = '';
			this._fetchData(function () {
				var dataInPage = self._dataInPage = self._getDataByPage();
				if (dataInPage) {
					self._buildView();
					self.activeIndex = 0;
					this.showView(this.pageIndex);
					self.onFocus();
				}
			});
		},
		showView: function (pageIndex) {
			var id = this.comp_name+'-view-p'+pageIndex;
			var viewBoxElm = evm.$(id);
			if(viewBoxElm){
				evm.css(viewBoxElm,{
					display:'block'
				});
			}

		},
		hideView: function (pageIndex) {
			var id = this.comp_name+'-view-p'+pageIndex;
			var viewBoxElm = evm.$(id);
			if(viewBoxElm){
				evm.css(viewBoxElm,{
					display:'none'
				});
			}

		},
		destroy: function () {

		},
		getActiveElm: function () {
			var activeElm = evm.$(this.comp_name + '-p'+ this.pageIndex +'-item-' + this.activeIndex);
			return activeElm;
		},
		onFocus: function () {
			var activeElm = this.getActiveElm();
			if (activeElm) {
				evm.addClass(activeElm, 'focus');
				this._trigger('onFocus');
			}
		},
		onBlur: function () {
			var activeElm = this.getActiveElm();
			if (activeElm) {
				evm.removeClass(activeElm, 'focus');
				this._trigger('onBlur');
			}
		},
		onSelect: function () {
			this._trigger('onSelect');
		},
		onWidgetBlur:function(keyAlias){
			if (this._trigger('onWidgetBlur', {
						alias: keyAlias
					})) {
				this.onBlur();
			}
		},
		setFetchDataUrl: function (url) {
			this.fetchDataUrl = url;
		}
	});

	evm.ext.comp.NonRegularTiles = NonRegularTiles;
})(this);