/*
 * 组件
 */

// 磁贴类
evm.ext.comp.TileBox = function(_c/*config object*/) {
    // 强制使用new
    if(this === evm.ext.comp || this === window) {
        return new arguments.callee(_c);
    }
    // 必须传参
    if(evm.isUndefined(_c) && !(evm.isObject(_c))) {
        throw Error('not found config parameter');
    }
    
    function hander(_c/*event object*/){
        // 触发事件的默认方法
    }
  
    evm.ext.comp.BaseComponent.call(this, _c/*object*/);
    this.config      = _c; // 配置项
    this.rows        = _c.rows || 2; // 水平方向上tile总数最大值
    this.cols        = _c.cols || 6; // 垂直方向上tile总数最大值
    this.tile_width  = _c.tileWidth || 160; // tile最小宽度
    this.tile_height = _c.tileHeight || 160; // tile最小高度
    this.items       = []; // 存储所有cells中显示的tile
    this.tile_count  = 0; // 计数器
    this.gap_x       = _c.gapX || 10; // 水平间隔
    this.gap_y       = _c.gapY || 10; // 垂直间隔
    this.start_x     = _c.startX || 0; // tileBox水平初始值
    this.start_y     = _c.startY || 0; // tileBox垂直初始值
    this.row_step    = 2;
    this.offset_row  = _c.persistent?_c.offsetRow||0:0;
    this.active_row  = _c.persistent?_c.rowIndex||0:0;
    this.active_col  = _c.persistent?_c.colIndex||0:0;
    this.rowPriority = evm.isBoolean(_c.rowPriority)?_c.rowPriority:true;
    this.back_row    = 0; // 
    this.back_col    = 0; // 
    this.back_dir    = ''; // 
    this.filter      = 'image'; // 
    this.cells       = []; // 画布数组
    this.type        = ''; //
    this.caption     = ''; // 标题
    this.box_elem    = null; // 所有tile的容器
    this.box_id      = _c.boxID || this.comp_name;
    this.parent_node = _c.parentNode || null;
    this.highLight_switch = evm.isBoolean(_c.highLightSwitch)?_c.highLightSwitch:false;
    this.isBuilt     = false;
    //====自定义事件的回调方法,需要在创建对象时传入对应的回调方法;=========================
    this.onBlur = _c.onBlur || hander;
    this.onChange = _c.onChange || hander;
    this.onChangeOut = _c.onChangeOut || hander;
    this.onFocus  = _c.onFocus || hander;
    this.onSelect = _c.onSelect || hander;
    this.onEvent  = _c.onEvent || hander;
}

evm.extend(evm.ext.comp.TileBox.prototype,{
    init : function(_d/*array*/) {
        var me   = this
        ,i    = 0
        ,data = _d || me.config.data || []
        ,len  = data.length
        ,model = me.config.model;
        
        if(!model) throw Error('config.model not found');
        
        if(evm.isString(me.parent_node)) {
            me.parent_node = evm.$(me.parent_node);
        }
    
        me.box_elem = evm.$(me.box_id);
        
        var value = me.start_y+me.offset_row*(me.tile_height+me.gap_y);
        if(!me.box_elem) {
            //若DOM没有查询到此元素,创建该元素,最后添加到body
            me.box_elem = evm.createElem('div', {
                id : me.box_id,
                className : me.class_name,
                style : ('width:'+(me.cols*(me.tile_width+me.gap_x*2)-me.gap_x*2)+'px;'+
                    'height:'+(me.rows*(me.tile_height+me.gap_y*2)-me.gap_y)+'px;'+
                    'position:absolute;'+
                    'left:'+me.start_x+'px;'+
                    'top:'+value*-1+'px')
            }, me.parent_node || document.body);
        }
        
        if(me.createCells()) {
            for(;i<len;i++) {
                me.addTile(data[i]);
            }
        }
        return me;
    }
    ,createCells : function() {
        //创建二维数组,并传入初始对象
        var me=this, r, c;
        for(r=0;r<me.rows;r++) {
            me.cells[r]=[];
            for(c=0;c<me.cols;c++) {
                me.cells[r][c] = {
                    cellId : (me.cols * r + c)
                    ,status : 0
                    ,type : ''
                    ,selected : 0
                    ,orgRow : -1
                    ,orgCol : -1
                };
            }
        }
        
        return true;
    }
    ,addTile : function(p/*object*/) {
        var me = this
        ,tile = null
        ,tile_id = me.box_id+'－tile－'+ me.tile_count
        ,tile_cls = me.box_id+'_tile'
        ,model = me.config.model||null
        ,free_cell, prop, r, c;
        
        if(!p) return false;
        // 磁贴对象
        tile = {
            index : me.tile_count++
        };
        
        if(model && model.length > 0) {
            for(var m=0; m<model.length; ++m) {
                if(model[m].text) {
                    prop =p[model[m].text];
                }
                if(evm.isFunction(model[m].callback)) {
                    model[m].callback(prop, tile, p);
                }
            }
        }
        
        free_cell = me.findFreeCell(tile);
        
        if(!free_cell) {
            return false;
        }
        
        var el = evm.createElem('div', {
            id : tile_id,
            className : tile_cls
        }, me.box_elem);
        
        evm.css(el,{
            overflow:'hidden'
            ,position:'absolute'
            ,top:free_cell.top+"px"
            ,left:free_cell.left+"px"
            ,width:free_cell.width+"px"
            ,height:free_cell.height+"px"
        });
        
        evm.extend(tile, {
            hasIcon: false
            ,x : free_cell.left
            ,y : free_cell.top
            ,w : free_cell.width
            ,h : free_cell.height
            ,gapX: me.gap_x
            ,gapY: me.gap_y
            ,bindElem : el
        });
    
        me.items.push(tile);
        me.putCell(free_cell.row, free_cell.col, tile.width, tile.height, tile);
        //console.log(me.cells);
        return true;
    }
    ,findFreeCell : function(p/*object:tile*/) {
        // 查找空格
        var me = this
        ,iw ,ih ,x ,y ,width ,height ,cell;
        
        if(me.rowPriority) {
            for(var r=0; r<me.rows; r++) {
                for(var c=0; c<me.cols; c++) {
                    for(iw=0; iw<p.width; iw++) {
                        if(c+iw > me.cols-1) break;
                        if(me.cells[r][c+iw].status !== 0) break;
                    }
                    for(ih=0; ih<p.height; ih++) {
                        if(r+ih>me.rows-1) break;
                        if(me.cells[r+ih][c].status !== 0) break;
                    }
                    if(iw == p.width && ih === p.height) {
                        x = c * me.tile_width + me.gap_x*(c+1);
                        y = r * me.tile_height + me.gap_y*(r+1);
                        width = p.width * me.tile_width + (p.width>1?(p.width-1)*me.gap_x:0);
                        height = p.height * me.tile_height + (p.height>1?(p.height-1)*me.gap_y:0);
                        //console.log(r,c,x,y,width,height);
                        return {row:r, col:c, left:x, top:y, width:width, height:height};
                    }
                }
            }
        }
        else{
            for(var c=0; c<me.cols; c++) {
                for(var r=0; r<me.rows; r++) {
                    for(iw=0; iw<p.width; iw++) {
                        if(c+iw > me.cols-1) break;
                        if(me.cells[r][c+iw].status !== 0) break;
                    }
                    for(ih=0; ih<p.height; ih++) {
                        if(r+ih>me.rows-1) break;
                        if(me.cells[r+ih][c].status !== 0) break;
                    }
                    if(iw == p.width && ih === p.height) {
                        x = c * me.tile_width + me.gap_x*(c+1);
                        y = r * me.tile_height + me.gap_y*(r+1);
                        width = p.width * me.tile_width + (p.width>1?(p.width-1)*me.gap_x:0);
                        height = p.height * me.tile_height + (p.height>1?(p.height-1)*me.gap_y:0);
                        //console.log(r,c,x,y,width,height);
                        return {row:r, col:c, left:x, top:y, width:width, height:height};
                    }
                }
            }
        }        
        return false;
    }
    ,putCell : function(r,c,w,h,tile) {
        var me = this
        ,cell = null; 
        for(var ih=0; ih<h; ih++) {
            for(var iw=0; iw<w; iw++) {
                cell = me.cells[r+ih][c+iw];
                cell.status = 1;
                cell.type = 'part';
                cell.orgRow = r;
                cell.orgCol = c;
            }
        }
        
        cell = me.cells[r][c];
        cell.index = me.tile_count;
        cell.type = "origin";
        cell.orgRow = -1;
        cell.orgCol = -1;
        cell.rows = h*1;
        cell.cols = w*1;
        cell.filter = tile.filter;
        cell.tile = tile;
        return cell;
    }
    ,findSibling : function(_d) {
        var me = this
        ,tile = me.cells[me.active_row][me.active_col].tile
        ,R = {row:-1, col:-1};
        
        if(_d === 'LEFT') {
            //var siblingCell = me.cells[me.active_row][me.active_col - tile.width];
            var siblingCell = me.cells[me.active_row][me.active_col - 1];
            
            if(siblingCell && siblingCell.status === 1) {
                if(siblingCell.orgRow === -1 && siblingCell.orgCol === -1) {
                    me.active_col = me.active_col - 1;
                    return true;
                }
                else{
                    me.active_row = siblingCell.orgRow;
                    me.active_col = siblingCell.orgCol;
                    return true;
                }
                
            }
        }
        else if(_d === 'RIGHT') {
            var siblingCell = me.cells[me.active_row][me.active_col + tile.width];
            if(siblingCell && siblingCell.status == 1) {
                if(siblingCell.orgRow === -1 && siblingCell.orgCol === -1) {
                    // 当前cell位置也是tile的位置
                    me.active_col = me.active_col + tile.width;
                    return true;
                }
                else{
                    me.active_row = siblingCell.orgRow;
                    me.active_col = siblingCell.orgCol;
                    return true;
                }
            }
        }
        else if(_d === 'UP') {
            if(!me.cells[me.active_row - tile.height]) {
                return false;
            }
            var siblingCell = me.cells[me.active_row - tile.height][me.active_col];
            if(siblingCell && siblingCell.status === 1) {
                if(siblingCell.orgRow === -1 && siblingCell.orgCol === -1) {
                    me.active_row = me.active_row - tile.height;
                    return true;
                }
                else{
                    me.active_row = siblingCell.orgRow;
                    me.active_col = siblingCell.orgCol;
                    return true;
                }
            }
        }
        else if(_d === 'DOWN') {
            if(!me.cells[me.active_row + tile.height]) {
                return false;
            }
            var siblingCell = me.cells[me.active_row + tile.height][me.active_col];
            if(siblingCell && siblingCell.status === 1) {
                if(siblingCell.orgRow === -1 && siblingCell.orgCol === -1) {
                    me.active_row = me.active_row + tile.height;
                    return true;
                }
                else{
                    me.active_row = siblingCell.orgRow;
                    me.active_col = siblingCell.orgCol;
                    return true;
                }
            }
        }
        
        return false;
    }
    ,move: function(_d/*str:'LEFT,RIGHT,UP,DOWN'*/) {
        var me = this
        evt = {
            type: 'blur'
            ,dir: _d
            ,back_dir: me.back_dir
            ,back_row: me.back_row
            ,back_col: me.back_col
            ,row: me.active_row
            ,col: me.active_col
            ,offsetRow: me.offset_row
        };
        me.onEvent(evt);
        
        // 记录前一次数据
        me.back_row = me.active_row;
        me.back_col = me.active_col;
        var hasSibling = me.findSibling(_d);
        evt = {
            type: 'focus'
            ,dir: _d
            ,back_dir: me.back_dir
            ,back_row: me.back_row
            ,back_col: me.back_col
            ,row: me.active_row
            ,col: me.active_col
            ,offsetRow: me.offset_row
            ,hasSibling: hasSibling
        }
        me.back_dir = _d;
        me.onEvent(evt);
        return me;
    }
    ,isEmpty: function() {
        var me = this;
        return me.config.data.length === 0;
    }
    ,getData: function() {
        var me = this;
        return me.config.data;
    }
    ,getCurrentTile: function() {
        var me = this
        ,tile = me.cells[me.active_row][me.active_col] || null;
        return tile;
    }
    ,checkIconVisible: function() {
        var me = this
        ,beginRow = me.offset_row
        ,cols = me.cols;
        
        for(var r=0; r<me.row_step; ++r) {
            if(me.cells[r+beginRow]) {
                for(var c=0; c<cols; ++c) {
                    if(me.cells[r+beginRow][c]) {
                        var current_cell = me.cells[r+beginRow][c];
                        if(current_cell.type === 'origin') {
                            if(!current_cell.tile.hasIcon) {
                                // 还没有加载图片
                                if(current_cell.tile.picURL) {
                                    // 有图片路径
                                    current_cell.tile.hasIcon = true;
                                    /**
                                     * 方式一:设置背景图
                                    var val = 'url('+current_cell.tile.picURL+')';
                                    evm.css(current_cell.tile.bindElem, {
                                        'backgroundImage': val
                                    });
                                     */
                                    /**
                                     * 方式二:插入Img
                                     */
                                    var val = '<img src="'+current_cell.tile.picURL+'" width="100%" height="100%">';
                                    evm.html(current_cell.tile.bindElem, val);
                                }
                                else{
                                    // 没有图标,填入栏目名
                                    var val = current_cell.tile.bkgColor;
                                    
                                    var tplHTML = '<span '
                                        +'style="width:100%;text-align:center;'
                                        +'position:absolute;top:45%;color:#FFF;">'
                                        +(current_cell.tile.text || 'unknown')
                                        +'</span>';
                                    evm.html(current_cell.tile.bindElem, tplHTML);
                                    evm.css(current_cell.tile.bindElem, {
                                        'backgroundColor': val
                                    });
                                }
                                
                                
                            }
                        }
                    }
                    
                }
            }
            
        }
        return me;
    }
    ,toggleFocus: function(_tile/*element node*/, _d/*str:'focus,blur'*/) {
        var me = this;
        if(!_tile) return me;
        /**
         * 支持CSS3特性的浏览器下使用
        evm.css(_elem, {
            '-webkit-transform': (_d === 'focus'?'scale(1.07)':'scale(1)')
            ,'box-shadow':(_d==='focus'?'2px 2px 12px #000':'none')
            ,'zIndex': (_d==='focus'?10:0)
        })
         */
        var elem = _tile.bindElem;
        
        if(_d === 'focus') {
            value = {
                left: (_tile.x - _tile.gapX)+'px'
                ,top: (_tile.y - _tile.gapY)+'px'
                ,width: (_tile.w + _tile.gapX*2)+'px'
                ,height:(_tile.h + _tile.gapY*2)+'px'
                ,'zIndex': 10
            };
            
            if(me.highLight_switch) {
                value['border'] = '4px solid #FFFF00';
                value['boxShadow'] = '1px 1px 8px #000000';
            }
        }
        else if(_d === 'blur') {
            value = {
                left: _tile.x+'px'
                ,top: _tile.y+'px'
                ,width: _tile.w+'px'
                ,height: _tile.h+'px'
                ,'zIndex':1
            };
            
            if(me.highLight_switch) {
                value['border'] = 'none';
                value['boxShadow'] = 'none';
            }
        }
        evm.css(elem, value);
        
        return me;
    }

});
