(function (window) {
    /**
     * 磁贴块组件 创建于2016-08-28
     *
     * @version 1.0.0
     * @author wuqq@evmtv.com
     *
     * @requires /swatapps/ape/evm.js
     * @requires /swatapps/ape/util/common.js
     *
     * @alias evm.ext.comp.Tiles
     * @param {Object} options 配置对象，具体成员可以参考默认配置对象{@link Tiles.defaults}
     * @constructor
     */
    function Tiles(options) {
        //类只能被实例化
        if (this === evm.ext.comp || this === window) {
            return new arguments.callee(options);
        }

        var config = evm.util.common.extend(true, {}, Tiles.defaults, options || {});
        evm.util.common.extend(this, config);
        //继承基类
        evm.ext.comp.BaseComponent.call(this, config);
        this._config = config;
        //私有属性
        //接口获取的原始数据对象
        this._oriData = null;
        //容器元素
        this._boxElm = null;
        //每页数据
        this._dataInPage = null;
        //记录上一次操作记录，用于互逆操作返回到刚才位置
        this._lastDir;
        this._lastActivePos;
        //按键的别名
        this._keyAlias = 'N/A';
    }

    Tiles.prototype = {
        /**
         * 构建展示视图
         * @param {Array} dataInPage 必须
         * @private
         */
        _buildView: function () {
            var self = this;

            function fill(elm) {
                var view = self.getView(self.pageIndex - 1),
                    model = self.model,
                    i = 0,
                    iLen = self._dataInPage.length,
                    j = 0,
                    jLen = model.length,
                    html = '',
                    prop;
                for (; i < iLen; i++) {
                    var itemStyle = 'position:absolute;'
                        + 'z-index:' + (iLen - i) + ';'
                        + 'left:' + view[i].left + 'px;'
                        + 'top:' + view[i].top + 'px;'
                        + 'width:' + view[i].width + 'px;'
                        + 'height:' + view[i].height + 'px;';
                    if (view[i].picBlur) {
                        itemStyle += 'background:transparent url(' + view[i].picBlur + ') no-repeat left top';
                    }

                    var itemClassName = self.class_name + '-item';
                    html += '<div id="' + self.comp_name + '-p' + self.pageIndex + '-item-' + i + '" class="' + itemClassName + '" style="' + itemStyle + '">';
                    j = 0;
                    if (jLen > 0) {
                        for (; j < jLen; j++) {
                            if (model[j].text) {
                                prop = self._dataInPage[i][model[j].text];
                            }
                            else {
                                throw new Error('需要配置model参数!');
                            }
                            //如果是文字
                            if (model[j].text === this.marqueeTextProp && prop.length > this.textLimit) {
                                prop = prop.substr(0, this.textLimit) + this.moreReplacement;
                            }
                            if (model[j].callback && evm.isFunction(model[j].callback)) {
                                prop = model[j].callback.call(self, prop, i, self._dataInPage, model[j].text);
                            }
                            html += prop;
                        }
                    }
                    else {
                        throw new Error('需要配置model参数!');
                    }
                    html += '</div>';
                }
                evm.html(elm, html);
            }

            if (this._dataInPage && this._dataInPage instanceof Array) {
                var viewElm = evm.$(this.comp_name + '-view-p' + this.pageIndex);
                if (!viewElm) {
                    viewElm = evm.createElem('div', {
                        id: this.comp_name + '-view-p' + this.pageIndex,
                        className: this.class_name + '-view',
                        style: 'position:absolute;left:0;top:0;'
                    }, this._boxElm);
                    fill.call(this, viewElm);
                }
                else {
                    if (this.fetchDataType === 'once') {
                        this.showView(this.pageIndex);
                    }
                    else if (this.fetchDataType === 'paging') {
                        evm.html(viewElm, '');
                        fill.call(this, viewElm);
                    }
                }
            }
        },
        //处理内部焦点循环
        _dealLoop: function (dir) {
            if (this.switchLoop) {
                var view = evm.util.common.extend(true, {}, this.getView(this.pageIndex - 1)[this.activeIndex]);
                if (dir === 'UP') {
                    view.top = 1440;
                }
                else if (dir === 'DOWN') {
                    view.top = -720;
                }
                else if (dir === 'LEFT') {
                    view.left = 2560;
                }
                else if (dir === 'RIGHT') {
                    view.left = -720;
                }
                this._move(dir, null, view);
                this._lastDir = 'N/A';
            }
        },
        /**
         * 获取接口数据
         * @param callback
         * @private
         */
        _fetchData: function (callback) {
            var self = this;
            this._trigger('onPreFetchData');
            var self = this;
            if (this.fetchDataType === 'once' && this.data && this._oriData) {//一次加载数据并且已经设置了数据和原始数据
                self._trigger('onFetchDataComplete');
                callback.call(this);
                return;
            }
            else if (this.fetchDataType === 'once' && this.data && !this._oriData//一次加载数据，设置了数据，未设置原始数据
                || this.fetchDataType === 'paging' && this.data && !this._oriData) {//分页加载数据，设置了数据，未设置原始数据
                self._trigger('onFetchDataComplete');
                this.setOriData(this.data);
                this.data = this.dataHandler(this.data);
                callback.call(this);
                return;
            }
            if (this.fetchDataUrl) {
                evm.ajax(this.fetchDataUrl, function (data) {
                    self._trigger('onFetchDataComplete');
                    try {
                        if (evm.isString(data)) {
                            data = eval('(' + data + ')');
                        }
                    }
                    catch (e) {
                        //如果解析失败，默认为空数组
                        data = [];
                    }

                    self.setOriData(data);
                    self.data = self.dataHandler(data);

                    callback.call(self);
                });
            }
        },
        /**
         * 返回true表示找到更好的合适块
         * @param currentPos 当前块的索引值
         * @param availablePos 当前最优块索引值
         * @returns {*}
         * @private
         */
        _compare: function (currentPos, availablePos, keyAlias, virtualView) {
            //焦点块边界坐标
            var view = this.getView(this.pageIndex - 1),
                activeView = virtualView || view[this.activeIndex],
                al = activeView.left,
                ar = activeView.left + activeView.width,
                at = activeView.top,
                ab = activeView.top + activeView.height;
            //当前块边界坐标
            var currentView = view[currentPos],
                cl = currentView.left,
                cr = currentView.left + currentView.width,
                ct = currentView.top,
                cb = currentView.top + currentView.height;
            if (availablePos === -1) {
                //初始找到可以选中的项目
                if (currentView.disabled) {
                    return false;
                }
                else {
                    return true;
                }
            }
            //比较跳过不可选中的项目
            if (currentView.disabled) {
                return false;
            }
            //最优边界坐标
            var mostView = view[availablePos],
                ml = mostView.left,
                mr = mostView.left + mostView.width,
                mt = mostView.top,
                mb = mostView.top + mostView.height;

            //条件二：按照和焦点块交叉面积多为优先
            if (keyAlias === 'LEFT' || keyAlias === 'RIGHT') {
                var dx = al - (view[currentPos].left + view[currentPos].width);
                var dx1 = al - (view[availablePos].left + view[availablePos].width);
                if (keyAlias === 'RIGHT') {
                    dx = view[currentPos].left - ar;
                    dx1 = view[availablePos].left - ar;
                }
                var dy,
                    dy1;

                //判断是否有垂直交叉
                //当前块和最优块都在焦点块上下边界之外，没有垂直交叉
                if ((ct > ab || cb < at) && (mt > ab || mb < at)) {
                    if (ct > ab) {
                        dy = ct - ab;
                    }
                    else {
                        dy = at - cb;
                    }
                    if (mt > ab) {
                        dy1 = mt - ab;
                    }
                    else {
                        dy1 = at - mb;
                    }
                    //取水平方向距离近的
                    if (dx < dx1) {
                        return true;
                    }
                }
                //最优块没有垂直交叉，当前块有垂直交叉
                else if ((mt > ab || mb < at) && (ct >= at && ct <= ab || cb >= at && cb <= ab || ct <= at && cb >= ab)) {
                    return true;
                }
                //当前块和最优块都和焦点块有边界延长重叠
                //重叠长度大者优先
                else if ((ct >= at && ct <= ab || cb >= at && cb <= ab || ct <= at && cb >= ab) && (mt >= at && mt <= ab || mb >= at && mb <= ab || mt <= at && mb >= ab)) {
                    var dl//当前块和焦点块重叠长度
                        , dl1;//最优块和焦点块重叠长度

                    if (ct <= at && cb >= ab) {
                        dl = ab - at;
                    }
                    else if (ct <= at && cb <= ab) {
                        dl = cb - at;
                    }
                    else if (ct >= at && cb <= ab) {
                        dl = cb - ct;
                    }
                    else if (ct >= at && cb >= ab) {
                        dl = ab - ct;
                    }

                    if (mt <= at && mb >= ab) {
                        dl1 = ab - at;
                    }
                    else if (mt <= at && mb <= ab) {
                        dl1 = mb - at;
                    }
                    else if (mt >= at && mb <= ab) {
                        dl1 = mb - mt;
                    }
                    else if (mt >= at && mb >= ab) {
                        dl1 = ab - mt;
                    }

                    if (dl != null && dl1 != null) {
                        if (dl > dl1) {
                            //当前块必须不能超过最优块的右侧
                            if (dx + view[currentPos].width <= dx1) {
                                return true;
                            }
                        }
                        else if (dl === dl1) {
                            if (dx < dx1) {
                                return true;
                            }
                        }
                        else {
                            //最优块距离当前块远端，选择当前块
                            if (dx + view[currentPos].width <= dx1) {
                                return true;
                            }
                        }
                    }
                }
            }
            else if (keyAlias === 'UP' || keyAlias === 'DOWN') {
                var dy = at - (view[currentPos].top + view[currentPos].height);
                var dy1 = at - (view[availablePos].top + view[availablePos].height);
                if (keyAlias === 'DOWN') {
                    dy = view[currentPos].top - ab;
                    dy1 = view[availablePos].top - ab;
                }
                var dx,
                    dx1;
                if (dy < dy1) {
                    return true;
                }
                else if (dy === dy1) {
                    //判断是否有边界延长重叠
                    if ((cr < al || cl > ar) && (mr < al || ml > ar)) {
                        //当前块和最优块都在焦点块左右边界之外
                        if (al > cr) {
                            dx = al - cr;
                        }
                        else {
                            dx = cl - ar;
                        }
                        if (al > mr) {
                            dx1 = al - mr;
                        }
                        else {
                            dx1 = ml - ar;
                        }
                        if (dx < dx1) {
                            return true;
                        }
                    }
                    else if ((mr < al || ml > ar) && (cl >= al && cl <= ar || cr <= ar && cr >= al || cr >= ar && cl <= al)) {
                        //最优块左右边界之外,当前块和焦点块有边界延长重叠
                        return true;
                    }
                    else if ((cl >= al && cl <= ar || cr <= ar && cr >= al || cl <= al && cr >= ar) && (ml >= al && ml <= ar || mr <= ar && mr >= al || ml <= al && mr >= ar)) {
                        //当前块和最优块都和焦点块有边界延长重叠
                        //重叠长度大者优先
                        var dl,
                            dl1;

                        if (cl >= al && cr <= ar) {
                            dl = cr - cl;
                        }
                        else if (cl <= al && cr >= ar) {
                            dl = ar - al;
                        }
                        else if (cl <= al && cr <= ar) {
                            dl = cr - al;
                        }
                        else if (cl >= al && cr >= ar) {
                            dl = ar - cl;
                        }

                        if (ml >= al && mr <= ar) {
                            dl1 = mr - ml;
                        }
                        else if (ml <= al && mr >= ar) {
                            dl1 = ar - al;
                        }
                        else if (ml <= al && mr <= ar) {
                            dl1 = mr - al;
                        }
                        else if (ml >= al && mr >= ar) {
                            dl1 = ar - ml;
                        }

                        if (dl != null && dl1 != null) {
                            if (dl >= dl1) {
                                return true;
                            }
                        }
                    }
                }
            }
        },
        /**
         * 查找最优块
         * @param keyAlias
         * @param virtualView 可选，用于处理内部焦点循环
         * @returns {number}
         * @private
         */
        _findAvailablePos: function (keyAlias, virtualView) {
            var i = 0,
                j,
                view = this.getView(this.pageIndex - 1),
                x = view[this.activeIndex].left,
                y = view[this.activeIndex].top,
                dx = 0,
                dy = 0,
                available = null,
                availablePos = -1,
                posibbleIndexs = [];

            if (virtualView) {
                x = virtualView.left;
                y = virtualView.top;
            }

            //焦点块边界查找可能的位置索引，Push进filterIndexs
            for (; i < this._dataInPage.length; i++) {
                if (!virtualView && i === this.activeIndex) {
                    continue;
                }
                var item = this._dataInPage[i];

                dx = view[i].left - x;
                dy = view[i].top - y;

                if (keyAlias === 'LEFT') {
                    dx += view[i].width;
                    if (dx <= 0) {
                        posibbleIndexs.push(i);
                    }
                }
                else if (keyAlias === 'RIGHT') {
                    if (virtualView) {
                        dx -= virtualView.width;
                    }
                    else {
                        dx -= view[this.activeIndex].width;
                    }
                    if (dx >= 0) {
                        posibbleIndexs.push(i);
                    }
                }
                else if (keyAlias === 'UP') {
                    dy += view[i].height;
                    if (dy <= 0) {
                        posibbleIndexs.push(i);
                    }
                }
                else if (keyAlias === 'DOWN') {
                    if (virtualView) {
                        dx -= virtualView.height;
                    }
                    else {
                        dy -= view[this.activeIndex].height;
                    }

                    if (dy >= 0) {
                        posibbleIndexs.push(i);
                    }
                }
            }

            for (i = 0, j = posibbleIndexs.length; i < j; i++) {
                var index = posibbleIndexs[i];
                var item = this._dataInPage[index];

                if (this._compare(index, availablePos, keyAlias, virtualView)) {
                    available = item;
                    availablePos = index;
                }
            }
            return availablePos;
        },
        /**
         * 获取单页显示数据
         * @private
         * @return 未找到数据返回null
         */
        _getDataByPage: function () {
            var data = null,
                start = 0,
                end = 0,
                i = 0;

            if (!this.data || !(this.data instanceof Array) || !this.data.length) {
                return data;
            }
            if (this.fetchDataType === 'once') {
                for (; i < this.pageIndex; i++) {
                    var view = this.getView(i);

                    if (i < this.pageIndex - 1) {
                        start += view.length;
                    }
                    end += view.length;
                }
            }
            else if (this.fetchDataType === 'paging') {
                var view = this.getView(this.pageIndex - 1);

                end = view.length;
            }

            if (start !== end) {
                data = this.data.slice(start, end);
            }
            return data;
        },
        /**
         * 焦点移动方法
         * @param keyAlias
         * @param myEvent
         * @param virtualView，可选，用于处理内部焦点循环
         * @private
         */
        _move: function (keyAlias, myEvent, virtualView) {
            var availablePos;
            if (this._lastDir === 'LEFT' && keyAlias === 'RIGHT' || this._lastDir === 'RIGHT' && keyAlias === 'LEFT') {
                availablePos = this._lastActivePos;
            }
            else if (this._lastDir === 'UP' && keyAlias === 'DOWN' || this._lastDir === 'DOWN' && keyAlias === 'UP') {
                availablePos = this._lastActivePos;
            }
            else {
                availablePos = this._findAvailablePos(keyAlias, virtualView);
            }
            if (virtualView && (availablePos === -1 || availablePos === this.activeIndex)) {
                //没有找到循环的焦点位置
                return;
            }
            if (availablePos !== -1 && availablePos !== this.activeIndex) {
                this._lastDir = keyAlias;
                this._lastActivePos = this.activeIndex;

                this.onBlur();
                this.activeIndex = availablePos;
                this.onFocus();
                if (!this.selectForChange) {
                    this.onChange();
                }
            }
            else {
                if (this.switchAutoPaging) {
                    if (this.autoPagingDir === 'H') {
                        if (keyAlias === 'LEFT' && this.pageIndex > 1) {
                            this.prev();
                        }
                        else if (keyAlias === 'RIGHT' && this.pageIndex < this.pageTotalNum) {
                            this.next();
                        }
                        else {
                            this.onWidgetBlur(keyAlias);
                        }
                    }
                    else if (this.autoPagingDir === 'V') {
                        if (keyAlias === 'UP' && this.pageIndex > 1) {
                            this.prev();
                        }
                        else if (keyAlias === 'DOWN' && this.pageIndex < this.pageTotalNum) {
                            this.next();
                        }
                        else {
                            this.onWidgetBlur(keyAlias);
                        }
                    }
                    else {
                        this.onWidgetBlur(keyAlias);
                    }
                }
                else {
                    this.onWidgetBlur(keyAlias);
                }
            }
        },
        /**
         * @param method
         * @param {object} obj
         * @returns {*}
         * @private
         */
        _trigger: function (method, obj) {
            var args = {
                activeIndex: this.activeIndex,
                pageIndex: this.pageIndex
            }
            evm.extend(args, obj);

            if (!args.activeData) {
                var activeData = this.getActiveData();
                args.activeData = activeData;
            }
            if (!args.activeElm) {
                var activeElm = this.getActiveElm();
                args.activeElm = activeElm;
            }
            if (!args.alias) {
                args.alias = this._keyAlias;
            }

            if (evm.isFunction(this['userEvent'])) {
                return this['userEvent'].call(this, method, args);
            }
        },
        /**
         * 组件按键事件入口
         * @param keyAlias
         * @param myEvent
         * @returns {number}
         */
        action: function (keyAlias, myEvent) {
            this._keyAlias = keyAlias;
            var res = 0;
            switch (keyAlias) {
                case 'UP':
                case 'RIGHT':
                case 'DOWN':
                case 'LEFT':
                    res = this._move(keyAlias, myEvent);
                    break;
                case 'SELECT':
                    res = this.onSelect();
                    break;
                default:
                    var args = {
                        alias: keyAlias,
                        event: myEvent
                    };
                    res = this._trigger('onActionDefault', args);
                    break;
            }
            return res;
        },
        /**
         * 组件对象部分恢复为初始状态
         */
        destroy: function () {
            this.enabled = false;
            this.activeIndex = 0;
            this.pageIndex = 1;
            this._lastDir = null;
            this._lastActivePos = -1;

            this._menuIndex = 0;
            this._menuPageIndex = 1;

            //清空DOM
            evm.html(this._boxElm, '');

            //清空数据
            this.data = null;
            this._oriData = null;

            this._trigger('onDestroy');
        },
        /**
         * 获取焦点对应的数据对象
         * @returns {Object|null}
         */
        getActiveData: function () {
            return this._dataInPage && this._dataInPage[this.activeIndex];
        },
        /**
         * 获取焦点对应的DOM对象
         */
        getActiveElm: function () {
            var activeElm = evm.$(this.comp_name + '-p' + this.pageIndex + '-item-' + this.activeIndex);
            return activeElm;
        },
        /**
         * 获取容器元素对象
         * @returns {null|*}
         */
        getBoxElm: function () {
            return this._boxElm;
        },
        /**
         * 获取接口返回的原始数据对象
         * @returns {null|*}
         */
        getOriData: function () {
            return this._oriData;
        },
        /**
         * 获取单页数据
         * @returns {*|null}
         */
        getDataInPage: function () {
            return this._dataInPage;
        },
        /**
         * 获取view数组成员
         * @param {number} index
         * @returns {*}
         */
        getView: function (index) {
            if (!this.view || !(this.view instanceof Array) || this.view.length < 1) {
                throw new Error('未正确配置view');
            }
            var view = this.view[index];
            if (!view) {
                view = this.view[this.view.length - 1];
            }

            return view;
        },
        /**
         * 隐藏页码对应的磁贴块视图
         * @param {number} pageIndex
         */
        hideView: function (pageIndex) {
            var id = this.comp_name + '-view-p' + pageIndex;
            var viewBoxElm = evm.$(id);
            if (viewBoxElm) {
                evm.css(viewBoxElm, {
                    display: 'none'
                });
            }

        },
        /**
         * 组件初始化
         */
        init: function () {
            this._trigger('onPreInit');
            var self = this;
            this._fetchData(function () {
                //准备容器元素
                var parentElm = evm.$('container') || document.body;
                if (!evm.isString(self.boxId)) {
                    self.boxId = self.comp_name;
                }
                self._boxElm = evm.$(self.boxId);
                var boxStyle = 'position:absolute;'
                    + 'left:' + self.boxLeft + 'px;'
                    + 'top:' + self.boxTop + 'px;'
                    + 'display:' + (self.isVisible === true ? 'block' : 'none') + ';';
                if (!self._boxElm) {
                    self._boxElm = evm.createElem('div', {
                        id: self.boxId,
                        className: self.class_name,
                        style: boxStyle
                    });
                }
                else {
                    evm.css(self._boxElm, boxStyle);
                }

                var pageData = self._dataInPage = self._getDataByPage();

                if (self.emptyMsg || self.emptyImage) {
                    if (!pageData) {
                        pageData = [
                            {
                                text: self.emptyMsg,
                                picBlur: self.emptyImage
                            }
                        ];
                    }
                }
                //数据准备完成，加载页面视图之前触发
                self._trigger('onInitPreBuildView');

                self._buildView();

                if (pageData) {
                    self.enabled = true;
                }

                self._trigger('onFlipover', {
                    pageTotalNum: self.pageTotalNum
                });

                if (self.isActive) {
                    self.onWidgetFocus();
                }
                else if (self.isMenu) {
                    self.onMenuFocus();
                }

                self._trigger('onInitComplete');
            });
        },
        /**
         * 向后翻页
         */
        next: function () {
            var self = this;

            if (this.pageIndex >= this.pageTotalNum) {
                return;
            }
            this.onBlur();
            this.hideView(this.pageIndex);
            this.pageIndex++;
            this._lastDir = '';
            this._fetchData(function () {
                var dataInPage = self._dataInPage = self._getDataByPage();
                if (dataInPage) {
                    self._buildView();
                    self.activeIndex = 0;
                    self.showView(self.pageIndex);
                    if (self.switchAutoPaging) {
                        self.onFocus();
                    }

                    self._trigger('onFlipover', {
                        pageTotalNum: self.pageTotalNum
                    });

                    if (!self.selectForChange) {
                        self.onChange();
                    }

                    self._trigger('onNextComplete');
                }
            });
        },
        /**
         * 磁贴块项失去焦点
         */
        onBlur: function () {
            var activeElm = this.getActiveElm();
            if (activeElm) {
                evm.removeClass(activeElm, 'focus');
                if (this.marqueeInst) {
                    this.marqueeInst.stopScroll();
                }
                this._trigger('onBlur');
            }
        },
        /**
         * 磁贴块焦点切换
         */
        onChange: function () {
            if (this.selectForChange) {
                this.onMenuBlur();

                this.menuIndex = this.activeIndex;
                this.menuPageIndex = this.pageIndex;

                this.onMenuFocus();
            }
            else {
                this.onMenuBlur();

                this.menuIndex = this.activeIndex;
                this.menuPageIndex = this.pageIndex;
            }
            this._trigger('onChange');
        },
        /**
         * 磁贴块项获取焦点
         */
        onFocus: function () {
            var activeElm = this.getActiveElm();
            if (activeElm) {
                evm.addClass(activeElm, 'focus');
                if (this.marqueeInst) {

                    evm.util.common.logger.clear();
                    evm.util.common.logger.debug('<ext.comp.Tiles> onFocus() 滚动开始');
                    //设置文字超长的项获取焦点时滚动显示
                    var activeData = this.getActiveData();
                    var textWrapElm = evm.getElementsByClassName(this.marqueeInst.wrapClassName, activeElm, 'div')[0];
                    var hasDataMarquee = false;
                    if (textWrapElm) {
                        hasDataMarquee = !!textWrapElm.getAttribute('data-marquee');
                    }
                    evm.util.common.logger.debug('<ext.comp.Tiles> onFocus() textWrapElm = ' + textWrapElm.className);
                    evm.util.common.logger.debug('<ext.comp.Tiles> onFocus() hasDataMarquee = ' + hasDataMarquee);
                    evm.util.common.logger.debug('<ext.comp.Tiles> onFocus() 启动滚动条件2：' + (activeData[this.marqueeTextProp] && activeData[this.marqueeTextProp].length > this.textLimit));

                    if (hasDataMarquee || (activeData[this.marqueeTextProp] && activeData[this.marqueeTextProp].length > this.textLimit)) {
                        this.marqueeInst.startScroll(textWrapElm, activeData[this.marqueeTextProp].substr(0, this.textLimit) + this.moreReplacement, activeData[this.marqueeTextProp]);
                    }
                }
                this._trigger('onFocus');
            }
        },
        /**
         * 磁贴块菜单项失去焦点
         */
        onMenuBlur: function () {
            if (this.menuPageIndex === this.pageIndex) {
                var menuElm = evm.$(this.comp_name + '-p' + this.menuPageIndex + '-item-' + this.menuIndex);
                if (menuElm) {
                    evm.removeClass(menuElm, 'menu');

                    this._trigger('onMenuBlur', {
                        menuPageIndex: this.menuPageIndex,
                        menuIndex: this.menuIndex,
                        menuElm: menuElm
                    });
                }
            }
        },
        /**
         * 磁贴块菜单项获取焦点
         * 假设menuIndex和menuPageIndex更新完成
         */
        onMenuFocus: function () {
            if (this.menuPageIndex === this.pageIndex) {
                var menuElm = evm.$(this.comp_name + '-p' + this.menuPageIndex + '-item-' + this.menuIndex);
                if (menuElm) {
                    evm.addClass(menuElm, 'menu');

                    this._trigger('onMenuFocus', {
                        menuPageIndex: this.menuPageIndex,
                        menuIndex: this.menuIndex,
                        menuElm: menuElm
                    });
                }
            }
        },
        /**
         * 按确认键
         */
        onSelect: function () {
            if (this.selectForChange) {
                this.onChange();
                this.onFocus();
            }
            this._trigger('onSelect');
        },
        /**
         * 组件获取焦点
         * @returns {boolean}
         */
        onWidgetFocus: function () {
            this._trigger('onWidgetFocus');
            //初始焦点位置设置
            var view = this.getView(this.pageIndex - 1);
            var availablePos = this.activeIndex;
            if (view[this.activeIndex].disabled) {
                //如果初始焦点位置项目设置为不可选中，重新设置初始焦点位置
                availablePos = this._findAvailablePos('RIGHT');
                if (availablePos === -1) {
                    availablePos = this._findAvailablePos('LEFT');
                }
                if (availablePos === -1) {
                    availablePos = this._findAvailablePos('DOWN');
                }
                if (availablePos === -1) {
                    availablePos = this._findAvailablePos('UP');
                }

                if (availablePos !== -1 && availablePos !== this.activeIndex) {
                    this.activeIndex = availablePos;
                }
                else {
                    return false;
                }
            }

            this.isActive = true;
            this._lastDir = 'N/A';

            //列表项获取焦点
            this.onFocus();
        },
        /**
         * 组件失去焦点
         * @param keyAlias
         */
        onWidgetBlur: function (keyAlias) {
            if (this._trigger('onWidgetBlur', {
                    alias: keyAlias
                })) {
                this.isActive = false;

                this.onBlur();

                if (this.isMenu) {
                    this.onMenuFocus();
                }
            }
            else {
                this._dealLoop(keyAlias);
            }
        },
        /**
         * 页码和页码中的位置转换为数据中的位置
         * @param activeIndex 焦点在单页中的位置
         * @param pageIndex 焦点所在的页码
         * @returns {*}
         */
        page2PosInData: function (activeIndex, pageIndex) {
            var posInData = 0;
            var index = 1;
            while (index < pageIndex) {
                var view = this.getView(index - 1);
                posInData += view.length;

                index++;
            }
            if (posInData >= 0) {
                return posInData + activeIndex;
            }
            else {
                return -1;
            }
        },
        /**
         * 在数据中的位置转换为页码和当前页中的位置
         * @param pos 焦点对应的数据对象在数据中的位置
         * @returns {{}}
         */
        posInData2Page: function (pos) {
            var obj = {};
            var pageIndex = 1;
            while (pos >= 0) {
                obj.activeIndex = pos;
                obj.pageIndex = pageIndex;

                var view = this.getView(pageIndex - 1);
                pos -= view.length;
                pageIndex++;
            }
            return obj;
        },
        /**
         * 向前翻页
         */
        prev: function () {
            var self = this;

            if (this.pageIndex <= 1) {
                return;
            }

            this.onBlur();
            this.hideView(this.pageIndex);
            this.pageIndex--;
            this._lastDir = '';
            this._fetchData(function () {
                var dataInPage = self._dataInPage = self._getDataByPage();
                if (dataInPage) {
                    self._buildView();

                    self.activeIndex = dataInPage.length - 1;
                    self.showView(self.pageIndex);
                    if (self.switchAutoPaging) {
                        self.onFocus();
                    }

                    self._trigger('onFlipover', {
                        pageTotalNum: self.pageTotalNum
                    });

                    if (!self.selectForChange) {
                        self.onChange();
                    }

                    self._trigger('onPrevComplete');
                }
            });
        },
        /**
         * 设置焦点位置
         * @param {number} val
         */
        setActiveIndex: function (val) {
            if (this._dataInPage && this._dataInPage[val]) {
                this.activeIndex = val;
            }
            else {
                this.activeIndex = 0;
            }
        },
        /**
         * 设置数据接口地址
         * @param {string} url
         */
        setFetchDataUrl: function (url) {
            this.fetchDataUrl = url;
        },
        /**
         * 保存接口返回的原始数据对象
         * @param data
         */
        setOriData: function (data) {
            this._oriData = data;
        },
        /**
         * 根据页码显示磁贴块视图
         * @param pageIndex
         */
        showView: function (pageIndex) {
            var id = this.comp_name + '-view-p' + pageIndex;
            var viewBoxElm = evm.$(id);
            if (viewBoxElm) {
                evm.css(viewBoxElm, {
                    display: 'block'
                });
            }

        },
        /**
         * 显示组件容器DOM对象
         */
        show: function () {
            this.isVisible = true;

            evm.util.common.show(this._boxElm);
            this._trigger('show');
        },
        /**
         * 隐藏组件容器DOM对象
         */
        hide: function () {
            this.isVisible = false;

            evm.util.common.hide(this._boxElm);
            this._trigger('hide');
        }
    };
    Tiles.prototype.constructor = Tiles;

    /**
     * 默认配置对象
     */
    Tiles.defaults = {
        /*基类配置参数*/
        prefix: 'Ape',
        compName: 'NonRegularTiles',
        parentComp: null,
        /*类本身配置参数*/
        //磁贴块数据
        data: null,
        /**
         * 获取数据方式，once或者paging
         * once表示请求的一次加载数据接口
         * paging表示请求分页加载数据接口
         */
        fetchDataType: 'once',
        //配置获取数据接口地址
        fetchDataUrl: '',
        //用于标识作为菜单项
        isMenu: false,
        //菜单项在单页中的位置
        menuIndex: 0,
        //菜单项所在的页数，起始页为1
        menuPageIndex: 1,
        //数据结构可能不同，需要处理数据
        //默认给出一种栏目数据结构
        //注意：需要计算总页码数
        dataHandler: function (data) {
            var node = data && data.node;
            var ret = null;
            if (node) {
                ret = data[node];
            }

            if (!ret) {
                return null;
            }

            //处理ret，去掉空的对象
            for (var i = 0; i < ret.length; i++) {
                if (evm.util.common.isEmptyObject(ret[i])) {
                    ret.splice(i, 1);
                    i = i - 1;
                }
            }
            //计算总页数
            if (ret && ret instanceof Array) {
                //初始化分页
                if (this.fetchDataType === 'once') {
                    var pageIndex = 0;
                    var i = 0;//计数器，累加每页显示的数量
                    while (i < ret.length) {
                        var view = this.getView(pageIndex);
                        pageIndex++;
                        i += view.length;
                    }

                    this.pageTotalNum = pageIndex;
                }
                else if (this.fetchDataType === 'paging') {
                    //总页码需要根据返回的总项数计算

                }
            }
            return ret;
        },
        //配置磁贴块每页显示数量，位置
        //用二维数组view配置,二维数组每一项都是配置单页展示
        //默认配置view[0]
        /**
         * @example
         * view[0] = [
         {
         left:0,
         top:0,
         width:0,
         height:0
         },
         ...
         ]
         */
        view: [],
        //当前页中的焦点位置
        activeIndex: 0,
        //当前页的页数，起始页为1
        pageIndex: 1,
        //总页数
        pageTotalNum: 1,
        //按确定更新内容
        selectForChange: false,
        //自动翻页标识
        switchAutoPaging: true,
        //自动翻页，水平H或垂直V
        autoPagingDir: 'H',
        //容器元素属性
        boxId: '',
        boxLeft: 0,
        boxTop: 0,
        //磁贴块项的Html生成配置
        //@example
        /**
         [{
			text: 'text',
			callback: function (val) {
				return '<div class="inner">' + val + '</div>';
			}
		}]
         */
        model: [],
        //定义配置方法
        /**
         * userEvent类似自定义的事件触发时候执行
         * 可以配置执行自己的操作
         * @param {string} method 自定义事件名称 onInitComplete,onWidgetBlur,onWidgetFocus,onFocus,onBlur,onSelect
         *        ,onPreFetchData,fetchDataSuccess,onMenuBlur,onMenuFocus
         * @param {object} args 返回一个对象
         */
        userEvent: function (method, args) {
        },
        //显示文字的最大字数
        textLimit: 100,
        //超出最大限制的文字的替换字符
        //默认替换字符为空
        moreReplacement: '',
        //设置滚动对象，用于焦点文字超出时滚动显示
        marqueeInst: null,
        //滚动文字对应的数据对象中的属性名称
        //如果属性名称不是'text'，必须手工配置
        marqueeTextProp: 'text',
        //获取焦点项的文字
        getActiveItemText: function () {
            var activeData = this.getActiveData();
            return activeData.text;
        },
        //没有数据给出默认提示
        //todo:该功能没有实现
        emptyMsg: '',
        emptyImage: '',
        //标识组件是否可用
        enabled: false,
        //扩展属性
        ext: {},
        //组件处于焦点状态，默认非焦点状态
        isActive: false,
        //组件容器DOM是否可见，默认为可见
        isVisible: true,
        //失去焦点时候，在没有其他组件获取焦点
        //是否在组件内部改变捕获焦点
        switchLoop: false
    };

    evm.ext.comp.Tiles = Tiles;
})(this);