/**
 * 垂直列表组件
 * 用于生成垂直列表
 * 依赖：/swatapps/ape/evm.js /swatapps/ape/util/marquee/main.js /swatapps/ape/util/common.js
 * 作者：wuqq@evmtv.com
 * 版本：v1.7.0
 * todo：
 * 1.增加获取数据的本地缓存
 *
 * 2016-04-12 v1.1.1
 * 1.配置文件新增marqueeInst属性，用于焦点时候超长文字自动滚动展示
 * 2016-05-13 v1.2.1
 * 1.新增获取容器DOM对象的方法getBoxElm
 * 2016-05-15 v1.3.1
 * 1.新增配置方法getActiveItemText，用于可以配置获取焦点项的文字
 * 2016-05-17 v1.3.2
 * 1.动态生成DOM，默认父级元素次序为.container,body
 * 2016-05-20 v1.3.2
 * 1.新增响应按键操作的默认回调消息'default'，可以在myEvent配置方法里面判断以响应除了方向键和确认操作之外的操作
 * 2016-05-25 v1.3.3
 * 1.优化dataHandler，修复json字符串中因为结束逗号导致的空对象的问题
 * 2.优化_fetchData方法
 * 3.注意增加了对/swatapps/ape/util/common.js的依赖
 * 2016-05-30 v1.3.4
 * 1.优化组件中文字超长滚动显示，新增配置属性marqueeTextProp，表示滚动文字对应的数据对象中属性名称
 * 2016-06-13 v1.4.4
 * 1.新增属性isVisible,isActive,ext
 * 2.新增方法show,hide
 * 2016-06-14 v1.5.4
 * 1.新增方法setActiveIndex
 * 2016-07-13 v1.6.0
 * 1.新增方法posInData2Page，用于换算数据位置和分页中的位置
 * 2017-06-07 v1.7.0
 * 1.修改ajax获取数据方式，支持post提交数据，增加配置属性fetchDataParam和相应的方法
 */
(function (window) {
	//定义垂直列表类
	function VList(options) {
		//类只能被实例化
		if (this === evm.ext.comp || this === window) {
			return new arguments.callee(options);
		}

		//默认配置参数
		var defaults = {
			/*基类配置参数*/
			prefix: 'Ape',
			compName: 'VList',
			parentComp: null,
			/*类本身配置参数*/
			//获取数据方式 once/一次 paging/分页
			//默认为once
			fetchDataType: 'once',
			//配置数据接口地址
			fetchDataUrl: '',
			//ajax param
			fetchDataParam:null,
			data: null,
			//当前页中的焦点位置
			activeIndex: 0,
			//获取焦点项的文字
			getActiveItemText: function () {
				var activeData = this.getActiveData();
				return activeData.text;
			},
			//容器元素属性
			boxId: '',
			boxLeft: 0,
			boxTop: 0,
			//列表项属性
			itemWidth: 0,
			itemHeight: 0,
			itemGapV: 0,
			//文字字数限制
			textLimit: 100,
			//文字超出部分替代
			//默认为空
			moreReplacement: '',
			//配置滚动对象后自动开启文字滚动
			marqueeInst: null,
			//滚动文字对应的数据对象中的属性名称
			//如果属性名称不是'text'，必须手工配置
			marqueeTextProp: 'text',
			//分页属性
			pageIndex: 1,
			pageSize: 8,
			pageTotalNum: 1,
			switchAutoPaging: true,
			//默认关闭焦点切换收尾循环
			switchLoop: false,
			//列表作为菜单
			//用于标识列表的菜单项
			isMenu: false,
			menuIndex: 0,
			menuPageIndex: 1,
			//需要按确认键才更新菜单项对应的内容
			selectForChange: false,
			//列表数据处理
			model: [],
			//没有数据给出默认提示
			emptyMsg: '',
			emptyImage: '',
			//定义配置方法
			/**
			 * userEvent类似自定义的事件触发时候执行
			 * 可以配置执行自己的操作
			 * @param {string} method 自定义事件名称 onInitComplete,onWidgetBlur,onWidgetFocus,onFocus,onBlur,onSelect
			 *        ,onPreFetchData,fetchDataSuccess,onMenuBlur,onMenuFocus，onFlipover
			 * @param {object} args 返回一个对象
			 */
			userEvent: function () {
			},

			//数据结构可能不同，需要处理获取到的数据
			//默认给出一种栏目数据结构
			//注意：需要计算总页码数
			dataHandler: function (data) {
				var node = data && data.node;
				var ret = null;
				if (node) {
					ret = data[node];
				}
				if (!ret) {
					return null;
				}

				//处理ret，去掉空的对象
				for (var i = 0; i < ret.length; i++) {
					if (evm.util.common.isEmptyObject(ret[i])) {
						ret.splice(i, 1);
						i = i - 1;
					}
				}
				if (ret && ret instanceof Array) {
					//初始化分页
					if (this.fetchDataType === 'once') {
						this.pageTotalNum = Math.ceil(ret.length / this.pageSize);
					}
					else if (this.fetchDataType === 'paging') {
						//总页码需要根据返回的总项数计算

					}
				}
				return ret;
			},
			//用于表示组件是否可被选中状态
			enabled: false,
			//扩展属性
			ext: {},
			//组件处于焦点状态，默认非焦点状态
			isActive: false,
			//组件DOM是否可见，默认为可见
			isVisible: true
		};

		evm.extend(defaults, options);
		evm.extend(this, defaults);
		//继承基类
		evm.ext.comp.BaseComponent.call(this, defaults);

		/*私有属性*/
		//容器元素
		this._boxElm = null;
		//接口返回的原始数据
		this._oriData = null;
		//按键别名
		this._keyAlias = 'N/A';
	}

	evm.extend(VList.prototype, {
		/**
		 * 构建视图
		 */
		_buildView: function () {
			var dataInPage = this.getDataByPage(),
				model = this.model,
				i = 0,
				iLen = dataInPage.length,
				j = 0,
				jLen = model.length,
				html = '',
				prop;
			for (; i < iLen; i++) {
				var itemStyle = 'position:absolute;'
					// + 'z-index:' + (iLen - i) + ';'
					+ 'width:' + this.itemWidth + 'px;'
					+ 'height:' + this.itemHeight + 'px;'
					+ 'line-height:' + this.itemHeight + 'px;'
					+ 'top:' + ((this.itemHeight + this.itemGapV) * i) + 'px;';
				var itemClassName = this.class_name + '-item';
				html += '<div id="' + this.comp_name + '-item-' + i + '" class="' + itemClassName + '" style="' + itemStyle + '">';
				j = 0;
				if (jLen > 0) {
					for (; j < jLen; j++) {
						if (model[j].text) {
							prop = dataInPage[i][model[j].text];
						}
						else {
							throw new Error('需要配置model参数!');
						}
						//如果是文字
						if ((model[j].text === this.marqueeTextProp || model[j].type === 'text') && prop.length > this.textLimit) {
							prop = prop.substr(0, this.textLimit) + this.moreReplacement;
						}
						if (model[j].callback && evm.isFunction(model[j].callback)) {
							prop = model[j].callback.call(this, prop, i, dataInPage, model[j].text);
						}
						html += prop;
					}
				}
				else {
					throw new Error('需要配置model参数!');
				}
				html += '</div>';
			}
			evm.html(this._boxElm, html);

			//页码等分页相关视图
			this._trigger('onFlipover', {
				pageIndex: this.pageIndex,
				pageTotalNum: this.pageTotalNum
			});
		},
		/**
		 * @param method
		 * @param {object} obj
		 * @returns {*}
		 * @private
		 */
		_trigger: function (method, obj) {
			var args = {
				activeIndex: this.activeIndex,
				pageIndex: this.pageIndex
			}
			evm.extend(args, obj);

			if (!args.activeData) {
				var activeData = this.getDataByPage()[this.activeIndex];
				args.activeData = activeData;
			}
			if (!args.activeElm) {
				var activeElm = evm.$(this.comp_name + '-item-' + this.activeIndex);
				args.activeElm = activeElm;
			}
			if (!args.alias) {
				args.alias = this._keyAlias;
			}

			if (evm.isFunction(this['userEvent'])) {
				return this['userEvent'].call(this, method, args);
			}
		},
		_fetchData: function (callback) {
			// evm.util.common.logger.debug('<ext.comp.Tiles._fetchData()> start');
			this._trigger('onPreFetchData');
			var self = this;
			if (!evm.util.common.isEmptyValue(this.data)) {
				if ((this.fetchDataType === 'once'
					|| this.fetchDataType === 'paging') && evm.util.common.isEmptyValue(this.getOriData())
				) {
					self._trigger('onFetchDataComplete');
					this.setOriData(this.data);
					this.data = this.dataHandler(this.data);
					callback.call(this);
					return;
				}
				else if (this.fetchDataType === 'once') {
					self._trigger('onFetchDataComplete');
					callback.call(this);
					return;
				}
			}

			if (this.fetchDataUrl) {
				// evm.util.common.logger.debug('<ext.comp.Tiles._fetchData()> response url:'+this.fetchDataUrl);
				evm.ajax(this.fetchDataUrl,this.fetchDataParam, function (data) {
					// evm.util.common.logger.debug('<ext.comp.VList._fetchData()> response data:'+data);
					self._trigger('onFetchDataComplete');
					var jsonData = evm.util.common.str2json(data);
					if(jsonData != null){
						data = jsonData;
					}

					self.setOriData(data);
					self.data = self.dataHandler(data);

					callback.call(self);
				});
			}
		},
		_moveV: function (keyAlias, myEvent) {
			var activeIndex = this.activeIndex,
				pageInData = this.getDataByPage(),
				limit = pageInData.length - 1;
			if (keyAlias === 'UP') {
				activeIndex--;
			}
			else if (keyAlias === 'DOWN') {
				activeIndex++;
			}

			if (activeIndex < 0) {
				if (this.pageIndex <= 1 || !this.switchAutoPaging) {
					this.onWidgetBlur('UP');
				}
				else {
					this.prev();
				}
			}
			else if (activeIndex >= 0 && activeIndex <= limit) {
				this.onBlur();
				this.activeIndex = activeIndex;
				this.onFocus();
				if (!this.selectForChange) {
					this.onChange();
				}
			}
			else {
				if (this.pageIndex >= this.pageTotalNum || !this.switchAutoPaging) {
					this.onWidgetBlur('DOWN');
				}
				else {
					this.next();
				}
			}
		},
		onChange: function () {
			if (this.selectForChange) {
				this.onMenuBlur();

				this.menuIndex = this.activeIndex;
				this.menuPageIndex = this.pageIndex;

				this.onMenuFocus();
			}
			else {
				this.onMenuBlur();
				this.menuIndex = this.activeIndex;
				this.menuPageIndex = this.pageIndex;
			}

			this._trigger('onChange');
		},
		onFocus: function () {
			if (this.isMenu && this.selectForChange) {
				//设置列表项的菜单高亮
				this.onMenuFocus();
			}

			var activeElm = evm.$(this.comp_name + '-item-' + this.activeIndex);
			var activeData = this.getActiveData();
			if (activeElm) {
				evm.addClass(activeElm, 'focus');

				if (this.marqueeInst) {
					//设置文字超长的项获取焦点时滚动显示
					var textWrapElm = evm.getElementsByClassName(this.marqueeInst.wrapClassName, activeElm, 'div')[0];
					var hasDataMarquee = false;
					if (textWrapElm) {
						hasDataMarquee = !!textWrapElm.getAttribute('data-marquee');
					}

					if (hasDataMarquee || (activeData[this.marqueeTextProp] && activeData[this.marqueeTextProp].length > this.textLimit)) {
						this.marqueeInst.startScroll(textWrapElm, activeData[this.marqueeTextProp].substr(0, this.textLimit) + this.moreReplacement, activeData[this.marqueeTextProp]);
					}
				}

				this._trigger('onFocus', {
					activeData: activeData,
					activeElm: activeElm
				});
			}
		},
		onBlur: function (method) {
			var activeElm = evm.$(this.comp_name + '-item-' + this.activeIndex);
			var activeData = this.getActiveData();
			if (activeElm) {
				evm.removeClass(activeElm, 'focus');
				if (this.marqueeInst) {
					this.marqueeInst.stopScroll();
				}
				this._trigger('onBlur', {
					activeData: activeData,
					activeElm: activeElm
				});
			}
		},
		//组件按键事件入口
		action: function (keyAlias, myEvent) {
			this._keyAlias = keyAlias;
			var res = 0;
			switch (keyAlias) {
				case 'UP':
				case 'DOWN':
					res = this._moveV(keyAlias, myEvent);
					break;
				case 'LEFT':
					res = this.onWidgetBlur('LEFT');
					break;
				case 'RIGHT':
					res = this.onWidgetBlur('RIGHT');
					break;
				case 'SELECT':
					res = this.onSelect();
					break;
				default:
					var args = {
						alias: keyAlias,
						event: myEvent
					};
					res = this._trigger('default', args);//向后兼容保留
					res = this._trigger('onActionDefault', args);
					break;
			}
			return res;
		},
		/**
		 * 组件初始化
		 */
		init: function () {
			// evm.util.common.logger.debug('<ext.comp.Tiles.init()> start');
			this._trigger('onPreInit');
			var self = this;

			//处理列表数据
			this._fetchData(function () {
				if (!self.data || !self.data.length) {
					self._trigger('onFetchDataError', {
						oriData: self.getOriData()
					});
					return;
				}

				//容器元素
				if (!self.boxId) {
					throw new Error('配置参数boxId不能为空!');
				}
				self._boxElm = evm.$(self.boxId);

				var boxStyle = 'position:absolute;'
					+ 'left:' + this.boxLeft + 'px;'
					+ 'top:' + this.boxTop + 'px;'
					+ 'display:' + (self.isVisible ? 'block' : 'none') + ';';
				if (!self._boxElm) {
					var parentNode = evm.$('container') || document.body;
					self._boxElm = evm.createElem('div', {id: self.boxId, style: boxStyle}, parentNode);
					//清空容器
					evm.html(self._boxElm, '');
				}
				else {
					evm.css(self._boxElm, boxStyle);
				}

				self._buildView();

				var dataInPage = self.getDataByPage();
				if (dataInPage.length) {
					self.enabled = true;
				}

				if (self.isActive) {
					self.onWidgetFocus();
				}
				else if (self.isMenu) {
					self.onMenuFocus();
				}

				self._trigger('onInitComplete');
			});
		},
		prev: function () {
			if (this.pageIndex <= 1) {
				return;
			}

			var self = this;
			var currPageIndex = this.pageIndex;
			currPageIndex--;
			this.onBlur();
			this.pageIndex = currPageIndex;
			this._fetchData(function () {
				var dataInPage = this.getDataByPage();
				if (dataInPage) {
					this._buildView();
					if (this.switchAutoPaging) {
						this.activeIndex = dataInPage.length - 1;
						this.onFocus();
						if (!this.selectForChange) {
							this.onChange();
						}
					}
					else {
						this.activeIndex = dataInPage.length - 1;
						this.onMenuFocus();
					}
					this._trigger('onPrevComplete');
				}
			});
		},
		next: function () {
			if (this.pageIndex >= this.pageTotalNum) {
				return;
			}

			var self = this;
			var currPageIndex = this.pageIndex;
			if (currPageIndex >= this.pageTotalNum) {
				return;
			}
			currPageIndex++;

			this.onBlur();
			this.pageIndex = currPageIndex;
			this._fetchData(function () {
				var dataInPage = this.getDataByPage();
				if (dataInPage) {
					this._buildView();
					if (this.switchAutoPaging) {
						this.activeIndex = 0;
						this.onFocus();
						if (!this.selectForChange) {
							this.onChange();
						}
					}
					else {
						this.activeIndex = dataInPage.length - 1;
						this.onMenuFocus();
					}

					this._trigger('onNextComplete');
				}
			});
		},
		/**
		 * 获取单页显示数据
		 * @returns {Array}
		 */
		getDataByPage: function () {
			var dataInPage = [],
				start,
				end;
			if (this.data) {
				if (this.fetchDataType === 'once') {
					start = (this.pageIndex - 1) * this.pageSize;
					end = this.pageIndex * this.pageSize;
				}
				else if (this.fetchDataType === 'paging') {
					start = 0;
					end = this.pageSize;
				}
				else {
					throw new Error('配置属性fetchDataType的值不正确!');
				}
				dataInPage = this.data.slice(start, end);
			}
			return dataInPage;
		},
		/**
		 * 获取焦点位置对应的数据对象
		 * @returns {*}
		 */
		getActiveData: function () {
			var dataInPage = this.getDataByPage();
			var ret = null;
			if (dataInPage) {
				ret = dataInPage[this.activeIndex];
			}
			return ret;
		},
		/**
		 * 实例化对象恢复为初始状态
		 */
		destroy: function () {
			//初始化一些属性
			this._oriData = null;
			this.data = null;

			this.activeIndex = 0;
			this.pageIndex = 1;

			this.menuIndex = 0;
			this.menuPageIndex = 1;

			this.enabled = false;

			evm.html(this._boxElm, '');
			this._trigger('onDestroy');
		},
		hasListData: function () {
			if (this.data instanceof Array) {
				return !!this.data.length;
			}
			return false;
		},
		/**
		 * 设置ajax param
		 * @param param
		 */
		setFetchDataParam:function(param){
			this.fetchDataParam = param;
		},
		/**
		 * 设置异步获取数据的请求地址
		 * @param url
		 */
		setFetchDataUrl: function (url) {
			this.fetchDataUrl = url;
		},
		/**
		 * 去除项目的菜单状态
		 */
		onMenuBlur: function () {
			if (this.menuPageIndex !== this.pageIndex) {
				//记录的菜单页码如果不在当前页，直接退出
				return;
			}
			var menuElm = evm.$(this.comp_name + '-item-' + this.menuIndex);
			if (menuElm) {
				evm.removeClass(menuElm, 'menu');
				this._trigger('onMenuBlur', {
					menuPageIndex: this.menuPageIndex,
					menuIndex: this.menuIndex,
					menuElm: menuElm
				});
			}
		},
		/**
		 * 设置项目菜单状态
		 */
		onMenuFocus: function () {
			if (this.menuPageIndex !== this.pageIndex) {
				//记录的菜单页码如果不在当前页，直接退出
				return;
			}
			//菜单新位置
			var menuElm = evm.$(this.comp_name + '-item-' + this.menuIndex);
			if (menuElm) {
				evm.addClass(menuElm, 'menu');
				this._trigger('onMenuFocus', {
					menuPageIndex: this.menuPageIndex,
					menuIndex: this.menuIndex,
					menuElm: menuElm
				});
			}
		},
		/**
		 * 按确定键操作
		 */
		onSelect: function () {
			if (this.selectForChange) {
				this.onChange();
				this.onFocus();
			}

			this._trigger('onSelect');
		},
		/**
		 * 设置接口返回的原始对象
		 * @param data
		 */
		setOriData: function (data) {
			this._oriData = data;
		},
		/**
		 * 返回接口返回的原始对象
		 * @returns {null|*}
		 */
		getOriData: function () {
			return this._oriData;
		},
		/**
		 * 组件获取焦点时候执行
		 */
		onWidgetFocus: function () {
			this._trigger('onWidgetFocus');
			//列表项获取焦点
			this.onFocus();

			this.isActive = true;
		},
		/**
		 * 组件失去焦点时候执行操作
		 * @param dir
		 */
		onWidgetBlur: function (dir) {
			if (this._trigger('onWidgetBlur', {
					alias: dir
				})) {
				this.onBlur();
				if (this.isMenu) {
					this.onMenuFocus();
				}

				this.isActive = false;
			}
			else {
				if (this.switchLoop) {
					if (dir === 'DOWN') {
						this.onBlur();
						this.activeIndex = 0;
						this.pageIndex = 1;
						if (this.pageTotalNum > 1) {
							this._buildView();
						}
						if (!this.selectForChange) {
							this.onChange();
						}
						this.onFocus();
					}
					else if (dir === 'UP') {
						this.onBlur();
						this.pageIndex = this.pageTotalNum;
						var dataInPage = this.getDataByPage();
						this.activeIndex = dataInPage.length - 1;
						if (this.pageTotalNum > 1) {
							this._buildView();
						}
						if (!this.selectForChange) {
							this.onChange();
						}
						this.onFocus();
					}
				}
			}
		},
		/**
		 * 获取容器元素
		 */
		getBoxElm: function () {
			return this._boxElm;
		},
		show: function () {
			this.isVisible = true;

			evm.util.common.show(this._boxElm);
			this._trigger('show');
		},
		hide: function () {
			this.isVisible = false;

			evm.util.common.hide(this._boxElm);
			this._trigger('hide');
		},
		//设置焦点值
		setActiveIndex: function (val) {
			this.activeIndex = val;
		},
		//在数据中的位置转换为页码和当前页中的位置
		//pos以0开始的位置
		posInData2Page: function (pos) {
			var obj = {};
			obj.pageIndex = Math.ceil((pos + 1) / this.pageSize);
			obj.activeIndex = pos % this.pageSize;

			return obj;
		}
	});

	//外部调用
	evm.ext.comp.VList = VList;
})(this);