/**
 * @namespace evm.util
 */
evm.namespace('evm.util');
(function () {
    /**
     * @desc
     * <pre>
     * 常用工具函数集合
     * v1.3.0
     * 1.日志增加logMode为server模式，输入打印日志到服务器
     * 2.新增支持POST提交xml
     *
     * v1.3.1
     * 1.日志新增配置客户端设置，可以设定固定盒子做为日志输入端
     * </pre>
     * @see 创建时间:2016-04-13
     * @version 1.3.1
     * @author wuqq@evmtv.com
     * @namespace evm.util.common
     */
    evm.util.common = {};
    var config = {
        logSwitch: false /*默认关闭调试日志*/
        /**
         * 默认日志模式complex，simple|complex|server
         * 默认为复杂模式，日志对多有用户隐藏
         * 复杂模式可能部分盒子出现问题，可以设置为简单模式
         * server模式为输出日志到服务器，需要上传laravel项目到服务器
         */
        , logMode: 'simple'
        , logLevel: 'debug' /*日志输出级别 debug,info,warn,error*/
    };
    //日志
    var logLevelMap = {
        debug: 0,
        info: 1,
        warn: 2,
        error: 3
    };
    //日志定时器id
    var logIntervalId = null;
    var logCookieName = 'evm.util.common.logger';
    //日志列表
    var logReposity = [];
    //默认开启日志打印权限
    var hasAuth = true;

    function checkAuth() {
        if (!evm.isFunction(evm.util.common.logger.config.setAuthToken)) {
            console && console.log('<evm.util checkAuth()>未配置setAuthToken函数');
            return;
        }
        var authToken = evm.util.common.logger.config.setAuthToken ? evm.util.common.logger.config.setAuthToken() : '',
            authArr = evm.util.common.logger.config.auth;
        for (var i = 0, len = authArr.length; i < len; i++) {
            if(authArr[i] === authToken){
                return true;
            }
        }
        return false;
    }

    var log = function (currentLogLevel, str) {
        //输入客户端检测
        //PC端不检测
        if(evm.util.common.logger.config.onCheckAuth){
            if(evm.stbInfo.browser !== 'pc'){
                if(hasAuth && !checkAuth()){
                    //目的是防止checkAuth里面调用日志打印，照成循环引用
                    hasAuth = false;
                    return;
                }
            }
        }

        var lastPageLogger = evm.cookie(logCookieName);
        if (lastPageLogger) {
            evm.cutCookie(logCookieName);
            lastPageLogger = evm.util.common.str2json(lastPageLogger);
            logReposity = (lastPageLogger && lastPageLogger.concat(logReposity)) || [];
        }
        var config = evm.util.common.logger.config,
            logSwitch = config.logSwitch,
            logMode = config.logMode,
            logLevel = logLevelMap[config.logLevel],
            currentLevel = logLevelMap[currentLogLevel];
        if (!logSwitch) {//关闭日志
            return;
        }
        if (currentLevel < logLevel) {//输出日志级别大于currentLevel的日志
            return;
        }
        if (logMode === 'simple') {
            var logElm = evm.$('log');
            if (!logElm) {
                logElm = document.createElement('div');
                logElm.id = 'log';
                document.getElementsByTagName('body')[0].appendChild(logElm);
                logElm.style.position = 'absolute';
                logElm.style.left = '50px';
                logElm.style.top = '50px';
                logElm.style.width = '50%';
                logElm.style.fontSize = '22px';
                logElm.style.color = 'yellow';
                logElm.style.backgroundColor = '#000000';
                //evm.createElem同洲中间件报错
                /*logElm = evm.createElem('div', {
                 id: 'log',
                 style: 'position:absolute;left:50px;top:50px;font-size:22px;color:#000;'
                 });*/
            }
            var newNode = document.createElement('p');
            if (logElm.firstChild) {
                logElm.insertBefore(newNode, logElm.firstChild);
            }
            else {
                logElm.appendChild(newNode);
            }
            newNode.innerText = str;
        }
        else if (logMode === 'complex') {
            evm.log(str);
        }
        else if (logMode === 'server') {
            logReposity.push(str);
            if (!config.postUrl) {
                throw new Error('未设置postUrl');
            }
            if (logIntervalId == null) {
                logIntervalId = setInterval(function () {
                    var len = 50;
                    var postLogString = logReposity.splice(0, 50).join('\n*');
                    if (!postLogString) {
                        return;
                    }
                    evm.post(config.postUrl, 'log=' + encodeURIComponent(postLogString+'\n'), function (response) {
                        //保存日志成功
                        // if (console) {
                        //     console.log('<util.common> log response:' + response);
                        // }
                    });
                }, config.intervalTime * 1000);
            }
        }
    };
    /**
     * @desc
     * <pre>
     * 日志打印
     * 依赖最新版本的/swatapps/ape/util-keymap.min.js（SVN下载）
     * 依赖最新版本的/swatapps/ape/evm.min.js（SVN下载）
     * 依赖最新版本/swatapps/ape/util/common-版本号-min.js（SVN下载）
     *
     * 依赖php文件打印日志
     * svn地址：svn://www.evmtv.com:19919/public/code/swatapps/ape/interface/logger.php
     * 配置：
     * 1.上传logger.php到服务器webroot/swatapps/ape/interface目录下
     *
     * 日志服务器输出路径
     * 1./opt/lampp/logs/stb-page-日期.log
     * </pre>
     * @type {{config: {logSwitch: boolean, logMode: string, logLevel: string}, setConfig: evm.util.common.logger.setConfig, debug: evm.util.common.logger.debug, info: evm.util.common.logger.info, warn: evm.util.common.logger.warn, error: evm.util.common.logger.error, clear: evm.util.common.logger.clear, stop: evm.util.common.logger.stop, getNowStr: evm.util.common.logger.getNowStr}}
     * @namespace
     * @example
     * //配置日志输入客户端（盒子），建议在项目的配置文件(所有页面都调用，比如config.js)中一次性配置
     * //采用此默认配置，输出日志到配置的服务器
     * //使用之后，需要测试之前添加的日志输出的页面，以前可能会直接打印到页面上面
     * evm.util.common.logger.setConfig({
     *      auth: [],//机顶盒特征值...
     *      setAuthToken: function () {
     *          //机顶盒卡号检测方法返回机顶盒卡号或者其他特征值
     *          //需要配置测试用的机顶盒
     *          //onCheckAuth设置false打印盒子的特征信息，配置好输入的机顶盒后，打开onCheckAuth
     *          return config.platform.getStbId();
     *      },
     *      onCheckAuth:true,
     *      logSwitch: true,
     *      logMode: 'server',
     *      postUrl: '/swatapps/ape/interface/logger.php'
     *  });
     *
     *
     * //日志开启在页面js中开启一次即可
     * //默认开启简单日志
     * evm.util.common.logger.setConfig(true);
     * //设置打印级别为warn
     * evm.util.common.logger.setConfig(true,'simple','warn');
     * //开启复杂打印，级别为info，级别可以设置
     * //按159键呼出或关闭打印面板
     * evm.util.common.logger.setConfig(true,'complex','info');
     * //注意，项目上线之后关闭打印
     * //可以注释或者如下设置
     * evm.util.common.logger.setConfig(false);
     * //新增日志打印方式，可以打印输出到服务器
     * evm.util.common.logger.setConfig({
     *      logSwitch: true,
     *      logMode: 'server'
     *  });
     */
    evm.util.common.logger = {
        /**
         * logger配置
         * @private
         */
        config: {
            logSwitch: config.logSwitch,
            logMode: config.logMode,
            logLevel: config.logLevel,
            intervalTime: 5,
            postUrl: '/swatapps/ape/interface/logger.php',
            onCheckAuth:true,
            //输入客户端特征字符串数组
            auth: [],
            /**
             * 输入客户端特征检测函数
             * 如：函数返回机顶盒的CA卡、MAC地址等
             */
            setAuthToken: null
        },
        /**
         * 设置配置参数
         * @param {Object} args 传入参数
         * @param {Boolean} [args.logSwitch] 是否开启日志
         * @param {String} [args.logMode] simple|complex|server，注：server模式依赖laravel项目
         * @param {String} [args.logLevel] debug,info,warn,error
         * @param {Number} [args.intervalTime] 传入页面日志到后台的时间间隔
         * @param {String} [args.postUrl] 处理页面日志后台接口地址
         * @param {Boolean} [args.onCheckAuth] 是否开启输入端检测
         * @param {Array} [args.auth] 输入客户端特征字符串数组
         * @param {Function} [args.setAuthToken] 输入客户端特征检测函数,如：函数返回机顶盒的CA卡、MAC地址等
         * @example
         * //默认开启简单日志
         * evm.util.common.logger.setConfig(true);
         * //设置打印级别为warn
         * evm.util.common.logger.setConfig(true,'simple','warn');
         * //传入对象
         * evm.util.common.logger.setConfig({
         *      logSwitch:true,
         *      logMode:'simple',
         *      logLevel:'warn',
         *      intervalTime:5
         * });
         */
        setConfig: function (args) {
            var obj = {};
            if (evm.isObject(args)) {//传入对象
                obj = args;
            }
            else {//单个传入参数
                obj.logSwitch = arguments[0];
                obj.logMode = arguments[1];
                obj.logLevel = arguments[2];
            }
            if (obj.logSwitch != null) {
                this.config.logSwitch = obj.logSwitch;
            }
            if (obj.logMode != null) {
                this.config.logMode = obj.logMode;
            }
            if (obj.logLevel != null) {
                this.config.logLevel = obj.logLevel;
            }
            if (obj.intervalTime != null) {
                this.config.intervalTime = obj.intervalTime;
            }
            if (obj.postUrl != null) {
                this.config.postUrl = obj.postUrl;
            }
            if (obj.auth != null) {
                this.config.auth = obj.auth;
            }
            if (obj.setAuthToken != null) {
                this.config.setAuthToken = obj.setAuthToken;
            }
            if(obj.onCheckAuth != null){
                this.config.onCheckAuth = obj.onCheckAuth;
            }
        },
        /**
         * debug方法
         * @param {String} str 打印日志内容
         */
        debug: function (str) {
            str = '[debug][' + this.getNowStr() + ']' + str;
            log('debug', str);
        },
        /**
         * info方法
         * @param {String} str 打印日志内容
         */
        info: function (str) {
            str = '[info][' + this.getNowStr() + ']' + str;
            log('info', str);
        },
        /**
         * warn方法
         * @param {String} str 打印日志内容
         */
        warn: function (str) {
            str = '[warn][' + this.getNowStr() + ']' + str;
            log('warn', str);
        },
        /**
         * error方法
         * @param {String} str 打印日志内容
         */
        error: function (str) {
            str = '[error][' + this.getNowStr() + ']' + str;
            log('error', str);
        },
        /**
         * 清空所有日志
         * @since 1.1.0
         */
        clear: function () {
            var logElm = evm.$('log');
            evm.html(logElm, '');
        },
        /**
         * 停止打印日志
         * @since 1.1.0
         */
        stop: function () {
            var str = '[info ' + this.getNowStr() + ']停止打印日志';
            log('error', str);
            this.setConfig(false);
        },
        /**
         * 获取当前时间字符串
         */
        getNowStr: function () {
            var now = new Date();
            return evm.util.common.dateFormat(now);
        },
        /**
         * 页面销毁时候纪录日志到cookie中
         */
        persist: function () {
            if (logIntervalId != null) {
                clearInterval(logIntervalId);
            }
            if (logReposity && logReposity.length) {
                evm.cookie(logCookieName, evm.util.common.json2str(logReposity));
            }
        }
    };
    /**
     * 日志打印
     * 保留支持老版本
     * @deprecated
     * @param {String} str 打印字符串
     */
    evm.util.common.log = function (str) {
        this.logger.info(str);
    };
    /**
     * 日期格式化
     * @param {Date} date 日期
     * @param {String} F 格式化字符，如：YYYY-mm-dd HH:ii:ss
     * @returns {String} 时间字符串
     */
    evm.util.common.dateFormat = function (date, F) {
        try {
            F = F || 'YYYY-mm-dd HH:ii:ss';
            var o =
            {
                'm+': date.getMonth() + 1	//月
                , 'd+': date.getDate()		//日
                , 'H+': date.getHours()		//时
                , 'i+': date.getMinutes()	//分
                , 's+': date.getSeconds()	//秒
                , 'q+': Math.floor((date.getMonth() + 3) / 3)	//季
                , 'S+': date.getMilliseconds()	//毫秒
                , 'w+': ['0', '1', '2', '3', '4', '5', '6'][date.getDay()]
                , 'W+': ['日', '一', '二', '三', '四', '五', '六'][date.getDay()]
            };
            if (/(Y+)/.test(F)) F = F.replace(RegExp.$1, ( date.getFullYear() + '' ).substr(4 - RegExp.$1.length));
            for (var K in o) {
                if (new RegExp('(' + K + ')').test(F))
                    F = F.replace(RegExp.$1
                        , ( RegExp.$1.length == 1 ) ? ( o[K] ) : ( (( K == 'S+' ? '000' : '00' ) + o[K]).substr(('' + o[K]).length)));
            }
        }
        catch (e) {
            throw new Error('日期格式化失败:' + e.message);
        }
        return F;
    };
    /**
     * json字符串转换为json对象
     * @param {String} str json字符串
     * @returns {Object} json对象
     */
    evm.util.common.str2json = function (str) {
        evm.util.common.logger.info('<evm.util.common.str2json> str in:' + str);
        if (!evm.isString(str)) {
            return str;
        }
        var json;
        try {
            json = eval("(" + str + ")");
        }
        catch (e) {
            evm.util.common.logger.error('<evm.util.common.str2json> error:' + e.message);
            json = null;
        }
        evm.util.common.logger.info('<evm.util.common.str2json> typeof str out:' + typeof json);
        return json;
    };
    /**
     * 将json对象转换为json字符串
     * @param {Object} O json对象
     * @returns {string} json字符串
     */
    evm.util.common.json2str = function (O) {
        // Key 不使用引号, 所以不能包含空格
        //if( JSON && JSON.stringify ) return JSON.stringify( O );
        var S = '{', sEnd = '}';
        if (evm.util.common.isArray(O)) {
            S = '[';
            sEnd = ']';
        }
        for (var i in O) {
            if (evm.isObject(O)) {
                S += i + ':';
            }
            if (evm.isUndefined(O[i])) S += '""';
            else if (evm.isObject(O[i])) S += evm.util.common.json2str(O[i]);
            else if (evm.util.common.isArray(O[i])) S += evm.util.common.json2str(O[i]);
            else if (evm.isString(O[i])) S += '"' + O[i] + '"';
            else S += O[i];
            S += ',';
        }
        S = S.substr(0, S.length - 1);
        S += sEnd;
        return S;
    };
    /**
     * 获取查询字符串的值
     * @param {String} key 查询字符串key
     * @returns {string}
     *
     * @example
     * //注意window.location要符合URI格式
     * //比如：不能出现两个问号情况，查询字符串的value中问号要编码
     * var value = evm.util.common.getQueryString('name');
     */
    evm.util.common.getQueryString = function (key) {
        var searchStr = window.location.search.substr(1);
        var reg = new RegExp(key + '=([^=&]*)');
        var value = '';
        if (reg.test(searchStr)) {
            value = window.decodeURIComponent(searchStr.match(reg)[1]);
        }
        return value;
    };
    /**
     * 判断是否为数组
     * @param {*} obj 传入标识符
     * @returns {Boolean}
     */
    evm.util.common.isArray = function (obj) {
        return Object.prototype.toString.call(obj) === '[object Array]';
    };
    /**
     * 判断对象是否为空
     * @param {*} [obj] 传入标识符
     * @returns {Boolean}
     */
    evm.util.common.isEmptyObject = function (obj) {
        for (var name in obj) {
            return false;
        }
        return true;
    };
    /**
     * 判断值是否为空
     * @param {*} value
     * @returns {Boolean}
     */
    evm.util.common.isEmptyValue = function (value) {
        var type;
        if (value == null) {
            return true;
        }
        type = Object.prototype.toString.call(value).slice(8, -1);
        //evm.log('evm.util.common.isEmptyValue->type:'+type);
        switch (type) {
            case 'String':
                return !value;
            case 'Array':
                return !value.length;
            case 'Object':
                return evm.util.common.isEmptyObject(value);
            default://其他对象均视作非空
                return false;
        }
    };
    /**
     * 删除字符串首尾空白符
     * @param {String} [value]
     * @returns {*}
     */
    evm.util.common.trim = function (value) {
        if (evm.isString(value)) {
            var reg = /^\s+|\s+$/g;
            return value.replace(reg, '');
        }
        return value;
    };
    /**
     * 修改jQuery.extend，用于扩展对象
     * @example
     * //深复制
     * evm.util.common.extend(true,target,source);
     * //浅复制
     * evm.util.common.extend(target,source);
     */
    evm.util.common.extend = function () {
        var src, copyIsArray, copy, name, options, clone,
            target = arguments[0] || {},
            i = 1,
            length = arguments.length,
            deep = false;
        // Handle a deep copy situation
        if (typeof target === "boolean") {
            deep = target;
            target = arguments[1] || {};
            // skip the boolean and the target
            i = 2;
        }
        // Handle case when target is a string or something (possible in deep copy)
        if (typeof target !== "object" && !evm.isFunction(target)) {
            target = {};
        }
        // if only one argument is passed
        if (length === i) {
            target = {};
            --i;
        }
        for (; i < length; i++) {
            // Only deal with non-null/undefined values
            if ((options = arguments[i]) != null) {
                // Extend the base object
                for (name in options) {
                    src = target[name];
                    copy = options[name];
                    // Prevent never-ending loop
                    if (target === copy) {
                        continue;
                    }
                    // Recurse if we're merging plain objects or arrays
                    if (deep && copy && ( (copyIsArray = evm.util.common.isArray(copy)) || evm.isObject(copy))) {
                        if (copyIsArray) {
                            copyIsArray = false;
                            clone = src && evm.util.common.isArray(src) ? src : [];
                        } else {
                            clone = src && evm.isObject(src) ? src : {};
                        }
                        // Never move original objects, clone them
                        target[name] = evm.util.common.extend(deep, clone, copy);
                        // Don't bring in undefined values
                    } else if (copy !== undefined) {
                        target[name] = copy;
                    }
                }
            }
        }
        // Return the modified object
        return target;
    };
    /*DOM操作*/
    /**
     * 显示DOM节点
     * @param {HTMLDOMElement} el
     */
    evm.util.common.show = function (el) {
        if (el) {
            el.style.display = 'block';
        }
    };
    /**
     * 隐藏DOM节点
     * @param {HTMLDOMElement} el
     */
    evm.util.common.hide = function (el) {
        if (el) {
            el.style.display = 'none';
        }
    };
    /**
     * 调整函数执行的上下文
     * @example
     * evm.util.common.proxy(obj,function(){
     *      //this指向obj
     * });
     */
    evm.util.common.proxy = function (context, fn) {
        if (evm.isFunction(context)) {
            fn = context;
            context = window;
        }
        if (!evm.isObject(context)) {
            evm.util.common.logger.warn('<util.common> proxy() 执行上下文不是对象');
            return;
        }
        if (!evm.isFunction(fn)) {
            evm.util.common.logger.warn('<util.common> proxy() fn不是函数');
            return;
        }
        return function () {
            return fn.apply(context, [].slice(arguments));
        };
    };
    /**
     * 持久化数据
     * @namespace
     */
    evm.util.common.persist = {
        /**
         * @param {String} cookieName
         * @return {Object} json对象
         */
        load: function (cookieName) {
            var cookieObj = null,
                cookieStr = evm.cookie(cookieName);
            if (cookieName) {
                evm.util.common.logger.info('cookieStr:' + cookieStr);
                cookieObj = evm.util.common.str2json(cookieStr);
                evm.cutCookie(cookieName);
            }
            return cookieObj;
        },
        /**
         * @param {string} cookieName cookie的名称
         * @param {object|string} obj
         */
        save: function (cookieName, obj) {
            if (cookieName && evm.isString(obj)) {
                evm.cookie(cookieName, obj);
                return true;
            }
            else if (cookieName && evm.isObject(obj)) {
                //组件数据持久化
                //需要注意对象的结构
                //json对象转换成字符串格式如：{mainView:{module1:{activeIndex:0,pageIndex:1},module2:{isActive:true,activeIndex:2,pageIndex:2}}}
                var jsonStr;
                try {
                    jsonStr = evm.util.common.json2str(obj);
                    evm.util.common.logger.info('<evm.util.common.persist.save>cookie字符串: ' + jsonStr);
                }
                catch (e) {
                    evm.util.common.logger.error('<evm.util.common.persist.save>数据持久化失败，转换为json字符串报错');
                }
                if (jsonStr) {
                    evm.cookie(cookieName, jsonStr);
                    return true;
                }
                else {
                    evm.util.common.logger.error('<evm.util.common.persist.save>数据持久化失败，obj->jsonStr为空');
                }
            }
            else {
                evm.util.common.logger.error('<evm.util.common.persist.save>数据持久化失败，第一个参数为cookieName,第二个参数为对象或者字符串类型');
            }
            return false;
        },
        /**
         * 恢复组件状态
         * 必须放到默认焦点组件的userEvent方法的onInitComplete回调里面执行
         * @param {Object} cookieObj 保存组件状态的json对象
         */
        recoverModuleState: function (cookieObj) {
            if ($WebApp && $WebApp.UI) {
                if (cookieObj && !evm.util.common.isEmptyObject(cookieObj)) {
                    for (var viewName in cookieObj) {
                        var view = $WebApp.UI[viewName];
                        var viewObj = cookieObj[viewName];
                        if (view && !evm.util.common.isEmptyObject(viewObj)) {
                            for (var i = 0; i < view.module_array.length; i++) {
                                var module = view.module_array[i];
                                if (module) {
                                    var moduleInst = module.inst;
                                    var moduleName = module.moduleName;
                                    if (moduleInst && moduleName) {
                                        for (var moduleNameProp in viewObj) {
                                            var moduleObj = viewObj[moduleNameProp];
                                            if (moduleName === moduleNameProp && !evm.util.common.isEmptyObject(moduleObj)) {
                                                //对于默认焦点组件，清除其焦点状态
                                                if (moduleInst.isActive) {
                                                    moduleInst.onBlur && moduleInst.onBlur();
                                                }
                                                evm.util.common.extend(true, moduleInst, moduleObj);
                                                if (moduleInst.isActive) {
                                                    moduleInst.onWidgetFocus && moduleInst.onWidgetFocus();
                                                }
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    };
    /**
     * 模板化处理字符串
     * @param str
     * @param data
     * @return {*}
     */
    evm.util.common.template = function (str, data) {
        if (!str) {
            return str;
        }
        for (var prop in data) {
            var reg = new RegExp('\\{' + prop + '\\}', 'ig');
            str = str.replace(reg, data[prop]);
        }
        return str;
    };
    /**
     * 解析xml字符串
     * @param xmlString
     * @return {*}
     */
    evm.util.common.loadXML = function (xmlString) {
        evm.util.common.logger.info('<evm.util.common.ajax.loadXML> xmlString = ' + xmlString);

        //过滤两端一些空白字符
        xmlString = xmlString.replace(/^[\s\uFEFF\xa0\u3000]+|[\uFEFF\xa0\u3000\s]+$/g, "");

        var xmlDoc = null;
        //判断浏览器的类型
        //支持IE浏览器
        if (!window.DOMParser && window.ActiveXObject) {   //window.DOMParser 判断是否是非ie浏览器
            evm.util.common.logger.info('<evm.util.common.ajax.loadXML>支持IE浏览器解析xml字符串');
            var xmlDomVersions = ['MSXML.2.DOMDocument.6.0', 'MSXML.2.DOMDocument.3.0', 'Microsoft.XMLDOM'];
            for (var i = 0; i < xmlDomVersions.length; i++) {
                try {
                    xmlDoc = new ActiveXObject(xmlDomVersions[i]);
                    xmlDoc.async = false;
                    xmlDoc.loadXML(xmlString); //loadXML方法载入xml字符串
                    break;
                } catch (e) {
                    evm.util.common.logger.error('<evm.util.common.ajax.loadXML>解析xml字符串失败');
                }
            }
        }
        //支持Mozilla浏览器
        /*else if(window.DOMParser && document.implementation && document.implementation.createDocument){*/
        else {
            try {
                evm.util.common.logger.info('<evm.util.common.ajax.loadXML>支持Mozilla浏览器解析xml字符串');
                /* DOMParser 对象解析 XML 文本并返回一个 XML Document 对象。
                 * 要使用 DOMParser，使用不带参数的构造函数来实例化它，然后调用其 parseFromString() 方法
                 * parseFromString(text, contentType) 参数text:要解析的 XML 标记 参数contentType文本的内容类型
                 * 可能是 "text/xml" 、"application/xml" 或 "application/xhtml+xml" 中的一个。注意，不支持 "text/html"。
                 */
                var domParser = new DOMParser();
                xmlDoc = domParser.parseFromString(xmlString, 'text/xml');
            } catch (e) {
                evm.util.common.logger.error('<evm.util.common.ajax.loadXML>解析xml字符串失败');
            }
        }
        /*else{
         evm.util.common.logger.error('<evm.util.common.ajax.loadXML>不支持解析xml字符串');
         return null;
         }*/
        return xmlDoc;
    };
    /**
     * ajax
     * @type {{post: evm.util.common.ajax.post}}
     */
    var ajaxSettings = {
        method: 'GET',
        accept: '',
        contentType: 'application/x-www-form-urlencoded;charset=utf-8'
    };
    evm.util.common.ajaxSettings = function (args) {
        if (args.method) {
            ajaxSettings.method = args.method;
        }
        if (args.contentType) {
            ajaxSettings.contentType = args.contentType;
        }
    };
    evm.util.common.ajax = function () {
        var xhrId = 0,
            xhr = null,
            create = function () {
                if (window.XMLHttpRequest) {
                    return function () {
                        return new XMLHttpRequest();
                    };
                } else {
                    return function () {
                        return new ActiveXObject('Msxml2.XMLHTTP');
                    };
                }
            }(),
            Ajax = function (url, data, callback, abort, timeout, context) {
                var args = [],
                    argsLen = arguments.length,
                    xhr = null;
                if (argsLen === 0) throw new Error('Not found parameters when invoking ajax');
                if (argsLen === 1) {
                    //No1 situation：argument is an object like {url:'', data:'', callback:fn, abort:fn, timeout:fn}
                    args = arguments[0];
                    if (typeof args == 'object') {
                        if (!args.url)
                            throw new Error('*url* attribute is undefined');
                        url = args.url;
                        if (!args.callback)
                            throw new Error('*callback* attribute is undefined');
                        callback = args.callback;
                        data = args.data || null;
                        abort = args.abort;
                        timeout = args.timeout;
                        context = args.context;
                    }
                } else {
                    //No2：many arguments
                    if (context === undefined) {
                        if (typeof timeout == 'object') {
                            context = timeout;
                            timeout = null;
                        } else if (typeof abort == 'object') {
                            context = abort;
                            abort = null;
                        } else if (typeof callback == 'object') {
                            context = callback;
                            callback = null;
                        }
                    }
                    if (evm.isFunction(data)) {
                        callback = data;
                        data = null;
                    } else {
                        if (!evm.isFunction(callback))
                            throw new TypeError();
                    }
                    if (!evm.isString(url))
                        throw new TypeError();
                }
                xhr = create();
                xhr.guid = xhrId++;
                xhr.open(ajaxSettings.method, url, true);
                if (ajaxSettings.method.toLowerCase() === 'post') {
                    xhr.setRequestHeader("Content-type", ajaxSettings.contentType);
                }
                if (ajaxSettings.accept) {
                    xhr.setRequestHeader("Accept", ajaxSettings.accept);
                }
                xhr.onreadystatechange = function () {
                    if (xhr.readyState === 4 &&
                        (xhr.status == 200 || xhr.status == 304)) {
                        callback.call(context || xhr, xhr.responseText);
                        xhr.onreadystatechange = null;
                    }
                };
                xhr.onabort = function () {
                    xhr.onreadystatechange = null;
                    (abort ? abort : callback).call(context || xhr, 'ABORT');
                };
                xhr.ontimeout = function () {
                    xhr.onreadystatechange = null;
                    (timeout ? timeout : callback).call(context || xhr, 'TIMEOUT');
                };
                xhr.send(data);
                return xhr;
            };
        return Ajax;
    }();
    //覆盖
    // evm.ajax = evm.util.common.ajax;
    /**
     * @1900-2100区间内的公历、农历互转
     * @charset  UTF-8
     * @Author  Jea杨(JJonline@JJonline.Cn)
     * @Time    2014-7-21
     * @Time    2016-8-13 Fixed 2033hex、Attribution Annals
     * @Version 1.0.1
     * @公历转农历：calendar.solar2lunar(1987,11,01); //[you can ignore params of prefix 0]
     * @农历转公历：calendar.lunar2solar(1987,09,10); //[you can ignore params of prefix 0]
     */
    var calendar = {
        /**
         * 农历1900-2100的润大小信息表
         * @Array Of Property
         * @return Hex
         */
        lunarInfo:[0x04bd8,0x04ae0,0x0a570,0x054d5,0x0d260,0x0d950,0x16554,0x056a0,0x09ad0,0x055d2,//1900-1909
            0x04ae0,0x0a5b6,0x0a4d0,0x0d250,0x1d255,0x0b540,0x0d6a0,0x0ada2,0x095b0,0x14977,//1910-1919
            0x04970,0x0a4b0,0x0b4b5,0x06a50,0x06d40,0x1ab54,0x02b60,0x09570,0x052f2,0x04970,//1920-1929
            0x06566,0x0d4a0,0x0ea50,0x06e95,0x05ad0,0x02b60,0x186e3,0x092e0,0x1c8d7,0x0c950,//1930-1939
            0x0d4a0,0x1d8a6,0x0b550,0x056a0,0x1a5b4,0x025d0,0x092d0,0x0d2b2,0x0a950,0x0b557,//1940-1949
            0x06ca0,0x0b550,0x15355,0x04da0,0x0a5b0,0x14573,0x052b0,0x0a9a8,0x0e950,0x06aa0,//1950-1959
            0x0aea6,0x0ab50,0x04b60,0x0aae4,0x0a570,0x05260,0x0f263,0x0d950,0x05b57,0x056a0,//1960-1969
            0x096d0,0x04dd5,0x04ad0,0x0a4d0,0x0d4d4,0x0d250,0x0d558,0x0b540,0x0b6a0,0x195a6,//1970-1979
            0x095b0,0x049b0,0x0a974,0x0a4b0,0x0b27a,0x06a50,0x06d40,0x0af46,0x0ab60,0x09570,//1980-1989
            0x04af5,0x04970,0x064b0,0x074a3,0x0ea50,0x06b58,0x055c0,0x0ab60,0x096d5,0x092e0,//1990-1999
            0x0c960,0x0d954,0x0d4a0,0x0da50,0x07552,0x056a0,0x0abb7,0x025d0,0x092d0,0x0cab5,//2000-2009
            0x0a950,0x0b4a0,0x0baa4,0x0ad50,0x055d9,0x04ba0,0x0a5b0,0x15176,0x052b0,0x0a930,//2010-2019
            0x07954,0x06aa0,0x0ad50,0x05b52,0x04b60,0x0a6e6,0x0a4e0,0x0d260,0x0ea65,0x0d530,//2020-2029
            0x05aa0,0x076a3,0x096d0,0x04afb,0x04ad0,0x0a4d0,0x1d0b6,0x0d250,0x0d520,0x0dd45,//2030-2039
            0x0b5a0,0x056d0,0x055b2,0x049b0,0x0a577,0x0a4b0,0x0aa50,0x1b255,0x06d20,0x0ada0,//2040-2049
            /**Add By JJonline@JJonline.Cn**/
            0x14b63,0x09370,0x049f8,0x04970,0x064b0,0x168a6,0x0ea50, 0x06b20,0x1a6c4,0x0aae0,//2050-2059
            0x0a2e0,0x0d2e3,0x0c960,0x0d557,0x0d4a0,0x0da50,0x05d55,0x056a0,0x0a6d0,0x055d4,//2060-2069
            0x052d0,0x0a9b8,0x0a950,0x0b4a0,0x0b6a6,0x0ad50,0x055a0,0x0aba4,0x0a5b0,0x052b0,//2070-2079
            0x0b273,0x06930,0x07337,0x06aa0,0x0ad50,0x14b55,0x04b60,0x0a570,0x054e4,0x0d160,//2080-2089
            0x0e968,0x0d520,0x0daa0,0x16aa6,0x056d0,0x04ae0,0x0a9d4,0x0a2d0,0x0d150,0x0f252,//2090-2099
            0x0d520],//2100
        /**
         * 公历每个月份的天数普通表
         * @Array Of Property
         * @return Number
         */
        solarMonth:[31,28,31,30,31,30,31,31,30,31,30,31],


        /**
         * 天干地支之天干速查表
         * @Array Of Property trans["甲","乙","丙","丁","戊","己","庚","辛","壬","癸"]
         * @return Cn string
         */
        Gan:["\u7532","\u4e59","\u4e19","\u4e01","\u620a","\u5df1","\u5e9a","\u8f9b","\u58ec","\u7678"],


        /**
         * 天干地支之地支速查表
         * @Array Of Property
         * @trans["子","丑","寅","卯","辰","巳","午","未","申","酉","戌","亥"]
         * @return Cn string
         */
        Zhi:["\u5b50","\u4e11","\u5bc5","\u536f","\u8fb0","\u5df3","\u5348","\u672a","\u7533","\u9149","\u620c","\u4ea5"],


        /**
         * 天干地支之地支速查表<=>生肖
         * @Array Of Property
         * @trans["鼠","牛","虎","兔","龙","蛇","马","羊","猴","鸡","狗","猪"]
         * @return Cn string
         */
        Animals:["\u9f20","\u725b","\u864e","\u5154","\u9f99","\u86c7","\u9a6c","\u7f8a","\u7334","\u9e21","\u72d7","\u732a"],


        /**
         * 24节气速查表
         * @Array Of Property
         * @trans["小寒","大寒","立春","雨水","惊蛰","春分","清明","谷雨","立夏","小满","芒种","夏至","小暑","大暑","立秋","处暑","白露","秋分","寒露","霜降","立冬","小雪","大雪","冬至"]
         * @return Cn string
         */
        solarTerm:["\u5c0f\u5bd2","\u5927\u5bd2","\u7acb\u6625","\u96e8\u6c34","\u60ca\u86f0","\u6625\u5206","\u6e05\u660e","\u8c37\u96e8","\u7acb\u590f","\u5c0f\u6ee1","\u8292\u79cd","\u590f\u81f3","\u5c0f\u6691","\u5927\u6691","\u7acb\u79cb","\u5904\u6691","\u767d\u9732","\u79cb\u5206","\u5bd2\u9732","\u971c\u964d","\u7acb\u51ac","\u5c0f\u96ea","\u5927\u96ea","\u51ac\u81f3"],


        /**
         * 1900-2100各年的24节气日期速查表
         * @Array Of Property
         * @return 0x string For splice
         */
        sTermInfo:[
            '9778397bd097c36b0b6fc9274c91aa','97b6b97bd19801ec9210c965cc920e','97bcf97c3598082c95f8c965cc920f',
            '97bd0b06bdb0722c965ce1cfcc920f','b027097bd097c36b0b6fc9274c91aa','97b6b97bd19801ec9210c965cc920e',
            '97bcf97c359801ec95f8c965cc920f','97bd0b06bdb0722c965ce1cfcc920f','b027097bd097c36b0b6fc9274c91aa',
            '97b6b97bd19801ec9210c965cc920e','97bcf97c359801ec95f8c965cc920f','97bd0b06bdb0722c965ce1cfcc920f',
            'b027097bd097c36b0b6fc9274c91aa','9778397bd19801ec9210c965cc920e','97b6b97bd19801ec95f8c965cc920f',
            '97bd09801d98082c95f8e1cfcc920f','97bd097bd097c36b0b6fc9210c8dc2','9778397bd197c36c9210c9274c91aa',
            '97b6b97bd19801ec95f8c965cc920e','97bd09801d98082c95f8e1cfcc920f','97bd097bd097c36b0b6fc9210c8dc2',
            '9778397bd097c36c9210c9274c91aa','97b6b97bd19801ec95f8c965cc920e','97bcf97c3598082c95f8e1cfcc920f',
            '97bd097bd097c36b0b6fc9210c8dc2','9778397bd097c36c9210c9274c91aa','97b6b97bd19801ec9210c965cc920e',
            '97bcf97c3598082c95f8c965cc920f','97bd097bd097c35b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa',
            '97b6b97bd19801ec9210c965cc920e','97bcf97c3598082c95f8c965cc920f','97bd097bd097c35b0b6fc920fb0722',
            '9778397bd097c36b0b6fc9274c91aa','97b6b97bd19801ec9210c965cc920e','97bcf97c359801ec95f8c965cc920f',
            '97bd097bd097c35b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa','97b6b97bd19801ec9210c965cc920e',
            '97bcf97c359801ec95f8c965cc920f','97bd097bd097c35b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa',
            '97b6b97bd19801ec9210c965cc920e','97bcf97c359801ec95f8c965cc920f','97bd097bd07f595b0b6fc920fb0722',
            '9778397bd097c36b0b6fc9210c8dc2','9778397bd19801ec9210c9274c920e','97b6b97bd19801ec95f8c965cc920f',
            '97bd07f5307f595b0b0bc920fb0722','7f0e397bd097c36b0b6fc9210c8dc2','9778397bd097c36c9210c9274c920e',
            '97b6b97bd19801ec95f8c965cc920f','97bd07f5307f595b0b0bc920fb0722','7f0e397bd097c36b0b6fc9210c8dc2',
            '9778397bd097c36c9210c9274c91aa','97b6b97bd19801ec9210c965cc920e','97bd07f1487f595b0b0bc920fb0722',
            '7f0e397bd097c36b0b6fc9210c8dc2','9778397bd097c36b0b6fc9274c91aa','97b6b97bd19801ec9210c965cc920e',
            '97bcf7f1487f595b0b0bb0b6fb0722','7f0e397bd097c35b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa',
            '97b6b97bd19801ec9210c965cc920e','97bcf7f1487f595b0b0bb0b6fb0722','7f0e397bd097c35b0b6fc920fb0722',
            '9778397bd097c36b0b6fc9274c91aa','97b6b97bd19801ec9210c965cc920e','97bcf7f1487f531b0b0bb0b6fb0722',
            '7f0e397bd097c35b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa','97b6b97bd19801ec9210c965cc920e',
            '97bcf7f1487f531b0b0bb0b6fb0722','7f0e397bd07f595b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa',
            '97b6b97bd19801ec9210c9274c920e','97bcf7f0e47f531b0b0bb0b6fb0722','7f0e397bd07f595b0b0bc920fb0722',
            '9778397bd097c36b0b6fc9210c91aa','97b6b97bd197c36c9210c9274c920e','97bcf7f0e47f531b0b0bb0b6fb0722',
            '7f0e397bd07f595b0b0bc920fb0722','9778397bd097c36b0b6fc9210c8dc2','9778397bd097c36c9210c9274c920e',
            '97b6b7f0e47f531b0723b0b6fb0722','7f0e37f5307f595b0b0bc920fb0722','7f0e397bd097c36b0b6fc9210c8dc2',
            '9778397bd097c36b0b70c9274c91aa','97b6b7f0e47f531b0723b0b6fb0721','7f0e37f1487f595b0b0bb0b6fb0722',
            '7f0e397bd097c35b0b6fc9210c8dc2','9778397bd097c36b0b6fc9274c91aa','97b6b7f0e47f531b0723b0b6fb0721',
            '7f0e27f1487f595b0b0bb0b6fb0722','7f0e397bd097c35b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa',
            '97b6b7f0e47f531b0723b0b6fb0721','7f0e27f1487f531b0b0bb0b6fb0722','7f0e397bd097c35b0b6fc920fb0722',
            '9778397bd097c36b0b6fc9274c91aa','97b6b7f0e47f531b0723b0b6fb0721','7f0e27f1487f531b0b0bb0b6fb0722',
            '7f0e397bd097c35b0b6fc920fb0722','9778397bd097c36b0b6fc9274c91aa','97b6b7f0e47f531b0723b0b6fb0721',
            '7f0e27f1487f531b0b0bb0b6fb0722','7f0e397bd07f595b0b0bc920fb0722','9778397bd097c36b0b6fc9274c91aa',
            '97b6b7f0e47f531b0723b0787b0721','7f0e27f0e47f531b0b0bb0b6fb0722','7f0e397bd07f595b0b0bc920fb0722',
            '9778397bd097c36b0b6fc9210c91aa','97b6b7f0e47f149b0723b0787b0721','7f0e27f0e47f531b0723b0b6fb0722',
            '7f0e397bd07f595b0b0bc920fb0722','9778397bd097c36b0b6fc9210c8dc2','977837f0e37f149b0723b0787b0721',
            '7f07e7f0e47f531b0723b0b6fb0722','7f0e37f5307f595b0b0bc920fb0722','7f0e397bd097c35b0b6fc9210c8dc2',
            '977837f0e37f14998082b0787b0721','7f07e7f0e47f531b0723b0b6fb0721','7f0e37f1487f595b0b0bb0b6fb0722',
            '7f0e397bd097c35b0b6fc9210c8dc2','977837f0e37f14998082b0787b06bd','7f07e7f0e47f531b0723b0b6fb0721',
            '7f0e27f1487f531b0b0bb0b6fb0722','7f0e397bd097c35b0b6fc920fb0722','977837f0e37f14998082b0787b06bd',
            '7f07e7f0e47f531b0723b0b6fb0721','7f0e27f1487f531b0b0bb0b6fb0722','7f0e397bd097c35b0b6fc920fb0722',
            '977837f0e37f14998082b0787b06bd','7f07e7f0e47f531b0723b0b6fb0721','7f0e27f1487f531b0b0bb0b6fb0722',
            '7f0e397bd07f595b0b0bc920fb0722','977837f0e37f14998082b0787b06bd','7f07e7f0e47f531b0723b0b6fb0721',
            '7f0e27f1487f531b0b0bb0b6fb0722','7f0e397bd07f595b0b0bc920fb0722','977837f0e37f14998082b0787b06bd',
            '7f07e7f0e47f149b0723b0787b0721','7f0e27f0e47f531b0b0bb0b6fb0722','7f0e397bd07f595b0b0bc920fb0722',
            '977837f0e37f14998082b0723b06bd','7f07e7f0e37f149b0723b0787b0721','7f0e27f0e47f531b0723b0b6fb0722',
            '7f0e397bd07f595b0b0bc920fb0722','977837f0e37f14898082b0723b02d5','7ec967f0e37f14998082b0787b0721',
            '7f07e7f0e47f531b0723b0b6fb0722','7f0e37f1487f595b0b0bb0b6fb0722','7f0e37f0e37f14898082b0723b02d5',
            '7ec967f0e37f14998082b0787b0721','7f07e7f0e47f531b0723b0b6fb0722','7f0e37f1487f531b0b0bb0b6fb0722',
            '7f0e37f0e37f14898082b0723b02d5','7ec967f0e37f14998082b0787b06bd','7f07e7f0e47f531b0723b0b6fb0721',
            '7f0e37f1487f531b0b0bb0b6fb0722','7f0e37f0e37f14898082b072297c35','7ec967f0e37f14998082b0787b06bd',
            '7f07e7f0e47f531b0723b0b6fb0721','7f0e27f1487f531b0b0bb0b6fb0722','7f0e37f0e37f14898082b072297c35',
            '7ec967f0e37f14998082b0787b06bd','7f07e7f0e47f531b0723b0b6fb0721','7f0e27f1487f531b0b0bb0b6fb0722',
            '7f0e37f0e366aa89801eb072297c35','7ec967f0e37f14998082b0787b06bd','7f07e7f0e47f149b0723b0787b0721',
            '7f0e27f1487f531b0b0bb0b6fb0722','7f0e37f0e366aa89801eb072297c35','7ec967f0e37f14998082b0723b06bd',
            '7f07e7f0e47f149b0723b0787b0721','7f0e27f0e47f531b0723b0b6fb0722','7f0e37f0e366aa89801eb072297c35',
            '7ec967f0e37f14998082b0723b06bd','7f07e7f0e37f14998083b0787b0721','7f0e27f0e47f531b0723b0b6fb0722',
            '7f0e37f0e366aa89801eb072297c35','7ec967f0e37f14898082b0723b02d5','7f07e7f0e37f14998082b0787b0721',
            '7f07e7f0e47f531b0723b0b6fb0722','7f0e36665b66aa89801e9808297c35','665f67f0e37f14898082b0723b02d5',
            '7ec967f0e37f14998082b0787b0721','7f07e7f0e47f531b0723b0b6fb0722','7f0e36665b66a449801e9808297c35',
            '665f67f0e37f14898082b0723b02d5','7ec967f0e37f14998082b0787b06bd','7f07e7f0e47f531b0723b0b6fb0721',
            '7f0e36665b66a449801e9808297c35','665f67f0e37f14898082b072297c35','7ec967f0e37f14998082b0787b06bd',
            '7f07e7f0e47f531b0723b0b6fb0721','7f0e26665b66a449801e9808297c35','665f67f0e37f1489801eb072297c35',
            '7ec967f0e37f14998082b0787b06bd','7f07e7f0e47f531b0723b0b6fb0721','7f0e27f1487f531b0b0bb0b6fb0722'],


        /**
         * 数字转中文速查表
         * @Array Of Property
         * @trans ['日','一','二','三','四','五','六','七','八','九','十']
         * @return Cn string
         */
        nStr1:["\u65e5","\u4e00","\u4e8c","\u4e09","\u56db","\u4e94","\u516d","\u4e03","\u516b","\u4e5d","\u5341"],


        /**
         * 日期转农历称呼速查表
         * @Array Of Property
         * @trans ['初','十','廿','卅']
         * @return Cn string
         */
        nStr2:["\u521d","\u5341","\u5eff","\u5345"],


        /**
         * 月份转农历称呼速查表
         * @Array Of Property
         * @trans ['正','一','二','三','四','五','六','七','八','九','十','冬','腊']
         * @return Cn string
         */
        nStr3:["\u6b63","\u4e8c","\u4e09","\u56db","\u4e94","\u516d","\u4e03","\u516b","\u4e5d","\u5341","\u51ac","\u814a"],


        /**
         * 返回农历y年一整年的总天数
         * @param lunar Year
         * @return Number
         * @eg:var count = calendar.lYearDays(1987) ;//count=387
         */
        lYearDays:function(y) {
            var i, sum = 348;
            for(i=0x8000; i>0x8; i>>=1) { sum += (calendar.lunarInfo[y-1900] & i)? 1: 0; }
            return(sum+calendar.leapDays(y));
        },


        /**
         * 返回农历y年闰月是哪个月；若y年没有闰月 则返回0
         * @param lunar Year
         * @return Number (0-12)
         * @eg:var leapMonth = calendar.leapMonth(1987) ;//leapMonth=6
         */
        leapMonth:function(y) { //闰字编码 \u95f0
            return(calendar.lunarInfo[y-1900] & 0xf);
        },


        /**
         * 返回农历y年闰月的天数 若该年没有闰月则返回0
         * @param lunar Year
         * @return Number (0、29、30)
         * @eg:var leapMonthDay = calendar.leapDays(1987) ;//leapMonthDay=29
         */
        leapDays:function(y) {
            if(calendar.leapMonth(y))  {
                return((calendar.lunarInfo[y-1900] & 0x10000)? 30: 29);
            }
            return(0);
        },


        /**
         * 返回农历y年m月（非闰月）的总天数，计算m为闰月时的天数请使用leapDays方法
         * @param lunar Year
         * @return Number (-1、29、30)
         * @eg:var MonthDay = calendar.monthDays(1987,9) ;//MonthDay=29
         */
        monthDays:function(y,m) {
            if(m>12 || m<1) {return -1}//月份参数从1至12，参数错误返回-1
            return( (calendar.lunarInfo[y-1900] & (0x10000>>m))? 30: 29 );
        },


        /**
         * 返回公历(!)y年m月的天数
         * @param solar Year
         * @return Number (-1、28、29、30、31)
         * @eg:var solarMonthDay = calendar.leapDays(1987) ;//solarMonthDay=30
         */
        solarDays:function(y,m) {
            if(m>12 || m<1) {return -1} //若参数错误 返回-1
            var ms = m-1;
            if(ms==1) { //2月份的闰平规律测算后确认返回28或29
                return(((y%4 == 0) && (y%100 != 0) || (y%400 == 0))? 29: 28);
            }else {
                return(calendar.solarMonth[ms]);
            }
        },

        /**
         * 农历年份转换为干支纪年
         * @param  lYear 农历年的年份数
         * @return Cn string
         */
        toGanZhiYear:function(lYear) {
            var ganKey = (lYear - 3) % 10;
            var zhiKey = (lYear - 3) % 12;
            if(ganKey == 0) ganKey = 10;//如果余数为0则为最后一个天干
            if(zhiKey == 0) zhiKey = 12;//如果余数为0则为最后一个地支
            return calendar.Gan[ganKey-1] + calendar.Zhi[zhiKey-1];

        },

        /**
         * 公历月、日判断所属星座
         * @param  cMonth [description]
         * @param  cDay [description]
         * @return Cn string
         */
        toAstro:function(cMonth,cDay) {
            var s   = "\u9b54\u7faf\u6c34\u74f6\u53cc\u9c7c\u767d\u7f8a\u91d1\u725b\u53cc\u5b50\u5de8\u87f9\u72ee\u5b50\u5904\u5973\u5929\u79e4\u5929\u874e\u5c04\u624b\u9b54\u7faf";
            var arr = [20,19,21,21,21,22,23,23,23,23,22,22];
            return s.substr(cMonth*2 - (cDay < arr[cMonth-1] ? 2 : 0),2) + "\u5ea7";//座
        },


        /**
         * 传入offset偏移量返回干支
         * @param offset 相对甲子的偏移量
         * @return Cn string
         */
        toGanZhi:function(offset) {
            return calendar.Gan[offset%10] + calendar.Zhi[offset%12];
        },


        /**
         * 传入公历(!)y年获得该年第n个节气的公历日期
         * @param y公历年(1900-2100)；n二十四节气中的第几个节气(1~24)；从n=1(小寒)算起
         * @return day Number
         * @eg:var _24 = calendar.getTerm(1987,3) ;//_24=4;意即1987年2月4日立春
         */
        getTerm:function(y,n) {
            if(y<1900 || y>2100) {return -1;}
            if(n<1 || n>24) {return -1;}
            var _table = calendar.sTermInfo[y-1900];
            var _info = [
                parseInt('0x'+_table.substr(0,5)).toString() ,
                parseInt('0x'+_table.substr(5,5)).toString(),
                parseInt('0x'+_table.substr(10,5)).toString(),
                parseInt('0x'+_table.substr(15,5)).toString(),
                parseInt('0x'+_table.substr(20,5)).toString(),
                parseInt('0x'+_table.substr(25,5)).toString()
            ];
            var _calday = [
                _info[0].substr(0,1),
                _info[0].substr(1,2),
                _info[0].substr(3,1),
                _info[0].substr(4,2),

                _info[1].substr(0,1),
                _info[1].substr(1,2),
                _info[1].substr(3,1),
                _info[1].substr(4,2),

                _info[2].substr(0,1),
                _info[2].substr(1,2),
                _info[2].substr(3,1),
                _info[2].substr(4,2),

                _info[3].substr(0,1),
                _info[3].substr(1,2),
                _info[3].substr(3,1),
                _info[3].substr(4,2),

                _info[4].substr(0,1),
                _info[4].substr(1,2),
                _info[4].substr(3,1),
                _info[4].substr(4,2),

                _info[5].substr(0,1),
                _info[5].substr(1,2),
                _info[5].substr(3,1),
                _info[5].substr(4,2),
            ];
            return parseInt(_calday[n-1]);
        },


        /**
         * 传入农历数字月份返回汉语通俗表示法
         * @param lunar month
         * @return Cn string
         * @eg:var cnMonth = calendar.toChinaMonth(12) ;//cnMonth='腊月'
         */
        toChinaMonth:function(m) { // 月 => \u6708
            if(m>12 || m<1) {return -1} //若参数错误 返回-1
            var s = calendar.nStr3[m-1];
            s+= "\u6708";//加上月字
            return s;
        },


        /**
         * 传入农历日期数字返回汉字表示法
         * @param lunar day
         * @return Cn string
         * @eg:var cnDay = calendar.toChinaDay(21) ;//cnMonth='廿一'
         */
        toChinaDay:function(d){ //日 => \u65e5
            var s;
            switch (d) {
                case 10:
                    s = '\u521d\u5341'; break;
                case 20:
                    s = '\u4e8c\u5341'; break;
                    break;
                case 30:
                    s = '\u4e09\u5341'; break;
                    break;
                default :
                    s = calendar.nStr2[Math.floor(d/10)];
                    s += calendar.nStr1[d%10];
            }
            return(s);
        },


        /**
         * 年份转生肖[!仅能大致转换] => 精确划分生肖分界线是“立春”
         * @param y year
         * @return Cn string
         * @eg:var animal = calendar.getAnimal(1987) ;//animal='兔'
         */
        getAnimal: function(y) {
            return calendar.Animals[(y - 4) % 12]
        },


        /**
         * 传入公历年月日获得详细的公历、农历object信息 <=>JSON
         * @param y  solar year
         * @param m solar month
         * @param d  solar day
         * @return JSON object
         * @eg:console.log(calendar.solar2lunar(1987,11,01));
         */
        solar2lunar:function (y,m,d) { //参数区间1900.1.31~2100.12.31
            if(y<1900 || y>2100) {return -1;}//年份限定、上限
            if(y==1900&&m==1&&d<31) {return -1;}//下限
            if(!y) { //未传参  获得当天
                var objDate = new Date();
            }else {
                var objDate = new Date(y,parseInt(m)-1,d)
            }
            var i, leap=0, temp=0;
            //修正ymd参数
            var y = objDate.getFullYear(),m = objDate.getMonth()+1,d = objDate.getDate();
            var offset   = (Date.UTC(objDate.getFullYear(),objDate.getMonth(),objDate.getDate()) - Date.UTC(1900,0,31))/86400000;
            for(i=1900; i<2101 && offset>0; i++) { temp=calendar.lYearDays(i); offset-=temp; }
            if(offset<0) { offset+=temp; i--; }

            //是否今天
            var isTodayObj = new Date(),isToday=false;
            if(isTodayObj.getFullYear()==y && isTodayObj.getMonth()+1==m && isTodayObj.getDate()==d) {
                isToday = true;
            }
            //星期几
            var nWeek = objDate.getDay(),cWeek = calendar.nStr1[nWeek];
            if(nWeek==0) {nWeek =7;}//数字表示周几顺应天朝周一开始的惯例
            //农历年
            var year = i;

            var leap = calendar.leapMonth(i); //闰哪个月
            var isLeap = false;

            //效验闰月
            for(i=1; i<13 && offset>0; i++) {
                //闰月
                if(leap>0 && i==(leap+1) && isLeap==false){
                    --i;
                    isLeap = true; temp = calendar.leapDays(year); //计算农历闰月天数
                }
                else{
                    temp = calendar.monthDays(year, i);//计算农历普通月天数
                }
                //解除闰月
                if(isLeap==true && i==(leap+1)) { isLeap = false; }
                offset -= temp;
            }

            if(offset==0 && leap>0 && i==leap+1)
                if(isLeap){
                    isLeap = false;
                }else{
                    isLeap = true; --i;
                }
            if(offset<0){ offset += temp; --i; }
            //农历月
            var month   = i;
            //农历日
            var day     = offset + 1;

            //天干地支处理
            var sm      =   m-1;
            var gzY     =   calendar.toGanZhiYear(year);

            //月柱 1900年1月小寒以前为 丙子月(60进制12)
            var firstNode   = calendar.getTerm(year,(m*2-1));//返回当月「节」为几日开始
            var secondNode  = calendar.getTerm(year,(m*2));//返回当月「节」为几日开始

            //依据12节气修正干支月
            var gzM     =   calendar.toGanZhi((y-1900)*12+m+11);
            if(d>=firstNode) {
                gzM     =   calendar.toGanZhi((y-1900)*12+m+12);
            }

            //传入的日期的节气与否
            var isTerm = false;
            var Term   = null;
            if(firstNode==d) {
                isTerm  = true;
                Term    = calendar.solarTerm[m*2-2];
            }
            if(secondNode==d) {
                isTerm  = true;
                Term    = calendar.solarTerm[m*2-1];
            }
            //日柱 当月一日与 1900/1/1 相差天数
            var dayCyclical = Date.UTC(y,sm,1,0,0,0,0)/86400000+25567+10;
            var gzD = calendar.toGanZhi(dayCyclical+d-1);
            //该日期所属的星座
            var astro = calendar.toAstro(m,d);

            return {'lYear':year,'lMonth':month,'lDay':day,'Animal':calendar.getAnimal(year),'IMonthCn':(isLeap?"\u95f0":'')+calendar.toChinaMonth(month),'IDayCn':calendar.toChinaDay(day),'cYear':y,'cMonth':m,'cDay':d,'gzYear':gzY,'gzMonth':gzM,'gzDay':gzD,'isToday':isToday,'isLeap':isLeap,'nWeek':nWeek,'ncWeek':"\u661f\u671f"+cWeek,'isTerm':isTerm,'Term':Term,'astro':astro};
        },


        /**
         * 传入公历年月日以及传入的月份是否闰月获得详细的公历、农历object信息 <=>JSON
         * @param y  lunar year
         * @param m lunar month
         * @param d  lunar day
         * @param isLeapMonth  lunar month is leap or not.
         * @return JSON object
         * @eg:console.log(calendar.lunar2solar(1987,9,10));
         */
        lunar2solar:function(y,m,d,isLeapMonth) {   //参数区间1900.1.31~2100.12.1
            var isLeapMonth = !!isLeapMonth;
            var leapOffset  = 0;
            var leapMonth   = calendar.leapMonth(y);
            var leapDay     = calendar.leapDays(y);
            if(isLeapMonth&&(leapMonth!=m)) {return -1;}//传参要求计算该闰月公历 但该年得出的闰月与传参的月份并不同
            if(y==2100&&m==12&&d>1 || y==1900&&m==1&&d<31) {return -1;}//超出了最大极限值
            var day  = calendar.monthDays(y,m);
            var _day = day;
            //bugFix 2016-9-25
            //if month is leap, _day use leapDays method
            if(isLeapMonth) {
                _day = calendar.leapDays(y,m);
            }
            if(y < 1900 || y > 2100 || d > _day) {return -1;}//参数合法性效验

            //计算农历的时间差
            var offset = 0;
            for(var i=1900;i<y;i++) {
                offset+=calendar.lYearDays(i);
            }
            var leap = 0,isAdd= false;
            for(var i=1;i<m;i++) {
                leap = calendar.leapMonth(y);
                if(!isAdd) {//处理闰月
                    if(leap<=i && leap>0) {
                        offset+=calendar.leapDays(y);isAdd = true;
                    }
                }
                offset+=calendar.monthDays(y,i);
            }
            //转换闰月农历 需补充该年闰月的前一个月的时差
            if(isLeapMonth) {offset+=day;}
            //1900年农历正月一日的公历时间为1900年1月30日0时0分0秒(该时间也是本农历的最开始起始点)
            var stmap   =   Date.UTC(1900,1,30,0,0,0);
            var calObj  =   new Date((offset+d-31)*86400000+stmap);
            var cY      =   calObj.getUTCFullYear();
            var cM      =   calObj.getUTCMonth()+1;
            var cD      =   calObj.getUTCDate();

            return calendar.solar2lunar(cY,cM,cD);
        }
    };
    evm.util.common.calendar = calendar;

    evm.util.common.logger.debug('common.js 加载执行结束');
})();
