(function (window, undefined) {
    /**
     * @desc
     * <pre>
     *  注意：
     *  1.日志是间隔5s发送到服务端，如果跳转页面之前需要注意日志是否保存
     *  2.跳转之前的日志保存，利用cookie，一般保存最新1024个字符的日志
     *  3.此版本日志文件不依赖其他js文件
     *
     *  更新日志：
     *  v0.0.2 2017-11-10
     *  1.处理页面跳转时候日志保存到cookie，保存长度为最新日志最后1024个字符
     *  2.增加了window.unload事件绑定，处理日志保存到cookie
     *
     *  v0.0.3 2018-05-16
     *  1.log方法开始判断，如果logSwitch未开启不执行log剩余代码(fix浏览器不能输出打印bug)
     *
     *  v0.0.4 2018-08-28
     *  1.修改日志输出打印，增加客户端标识，需要配置setAuthToken获取客户端标识
     *
     *  v0.0.5 2018-08-30
     *  1.修复getUid循环调用的Bug
     *
     *  v0.0.6 2018-09-04
     *  1.修改persist方法，页面销毁时候，保存最后size条日志记录
     * </pre>
     * @see 创建时间:2017-09-08
     * @version 0.0.6
     * @dependent /swatapps/ape/evm.js
     * @author wuqq@evmtv.com
     * @namespace evm.util.common.logger
     */
    if (!window.evm) {
        window.evm = {};
    }
    window.evm.util = window.evm.util || {};
    window.evm.util.common = window.evm.util.common || {};

    var config = {
        /*默认关闭调试日志*/
        logSwitch: false
        /**
         * 默认日志模式server，simple|complex|server|browser
         * 复杂模式可能部分盒子出现问题，可以设置为简单模式
         * server模式为输出日志到服务器，服务器处理文件/swatapps/ape/interface/logger.php
         */
        , logMode: 'server'
        /*日志输出级别 debug,info,warn,error*/
        , logLevel: 'debug'
    };
    //日志
    var logLevelMap = {
        debug: 0,
        info: 1,
        warn: 2,
        error: 3
    };
    //日志定时器id
    var logIntervalId = null;
    var logCookieName = 'evm.util.common.logger';
    //日志列表
    var logReposity = [location.href];
    //默认开启日志打印权限
    //用于避免checkAuth内部调用方法使用日志打印，造成循环调用
    var hasAuth = true;
    //鉴权缓存变量
    var authPassed = -1;

    var configCompleted = false;

    loadLogFromCookie();

    var checkAuthCounter = 0;

    function checkAuth() {
        if (checkAuthCounter > 0) {
            return evm.util.common.logger.config.logSwitch;
        }
        checkAuthCounter++;

        if (evm.util.common.logger.config.logMode === 'browser') {
            return true;
        }

        if (!isFunction(evm.util.common.logger.config.setAuthToken)) {
            evm.util.common.logger.warn('<evm.util checkAuth()>未配置setAuthToken函数');
            return false;
        }
        var authToken = evm.util.common.logger.config.setAuthToken ? evm.util.common.logger.config.setAuthToken() : '',
            authArr = evm.util.common.logger.config.auth;

        for (var i = 0, len = authArr.length; i < len; i++) {
            if (authArr[i] === authToken) {
                return true;
            }
        }
        return false;
    }

    var uid = '[uid:=Unknown Client]';
    var uidCounter = 0;

    // 用于辨识客户端用户
    function getUid() {
        // 防止循环调用
        if (uidCounter > 0) {
            return uid;
        }

        var token = evm.util.common.logger.config.setAuthToken && evm.util.common.logger.config.setAuthToken() || 'Unknown Client';
        uid = '[uid:=' + token + ']';

        uidCounter++;

        return uid;
    }

    var log = function (currentLogLevel, str) {
        // 未开启logSwitch
        if (!evm.util.common.logger.config.logSwitch) {
            return;
        }
        if (!authPassed) {//鉴权失败，直接退出
            return;
        }
        //输入客户端检测
        if (authPassed === -1 && evm.util.common.logger.config.onCheckAuth) {
            if (!hasAuth) {
                return;
            }

            hasAuth = false;
            authPassed = checkAuth();
            hasAuth = true;

            if (evm.stbInfo && evm.stbInfo.browser !== 'pc') {
                //PC端不检测
                if (!authPassed) {
                    return;
                }
            } else {
                //未引入evm.js
                //stb和pc端都需要检测
                if (!authPassed) {
                    return;
                }
            }
        }

        var config = evm.util.common.logger.config,
            logSwitch = config.logSwitch,
            logMode = config.logMode,
            logLevel = logLevelMap[config.logLevel],
            currentLevel = logLevelMap[currentLogLevel];
        if (!logSwitch) {//关闭日志
            return;
        }
        if (currentLevel < logLevel) {//输出日志级别大于currentLevel的日志
            return;
        }
        if (logMode === 'simple') {
            var logElm = document.getElementById('log');
            if (!logElm) {
                logElm = document.createElement('div');
                logElm.id = 'log';
                document.getElementsByTagName('body')[0].appendChild(logElm);
                logElm.style.position = 'absolute';
                logElm.style.left = '50px';
                logElm.style.top = '50px';
                logElm.style.width = '50%';
                logElm.style.fontSize = '22px';
                logElm.style.color = 'yellow';
                logElm.style.backgroundColor = '#000000';
                //evm.createElem同洲中间件报错
                /*logElm = evm.createElem('div', {
                 id: 'log',
                 style: 'position:absolute;left:50px;top:50px;font-size:22px;color:#000;'
                 });*/
            }
            var newNode = document.createElement('p');
            if (logElm.firstChild) {
                logElm.insertBefore(newNode, logElm.firstChild);
            } else {
                logElm.appendChild(newNode);
            }
            newNode.innerText = str;
        } else if (logMode === 'complex') {
            if (isFunction(evm.log)) {
                evm.log(str);
            }
        } else if (logMode === 'server') {
            logReposity.push(str);
            if (!config.postUrl) {
                throw new Error('未设置postUrl');
            }
        } else if (logMode === 'browser') {
            try {
                console.log(str);
            } catch (e) {
                throw e;
            }
        }
    };

    /**
     * @desc
     * <pre>
     * 日志打印
     * 注意：如果存在引用evm.js，logger.js需要在evm.js之后引用
     * 依赖php文件打印日志
     * svn地址：svn://www.evmtv.com:19919/public/code/swatapps/ape/interface/logger.php
     * 配置：
     * 1.上传logger.php到服务器webroot/swatapps/ape/interface目录下
     *
     * 日志服务器输出路径
     * 1./opt/lampp/logs/stb-page-日期.log
     * 2.shell查看日志命令：tailf /opt/lampp/logs/stb-page-日期.log
     * </pre>
     * @type {{config: {logSwitch: boolean, logMode: string, logLevel: string}, setConfig: evm.util.common.logger.setConfig, debug: evm.util.common.logger.debug, info: evm.util.common.logger.info, warn: evm.util.common.logger.warn, error: evm.util.common.logger.error, clear: evm.util.common.logger.clear, stop: evm.util.common.logger.stop, getNowStr: evm.util.common.logger.getNowStr}}
     * @namespace
     * @example
     * //配置日志输入客户端（盒子），建议在项目的配置文件(所有页面都调用，比如config.js)中一次性配置
     * //采用此默认配置，输出日志到配置的服务器
     * //使用之后，需要测试之前添加的日志输出的页面，以前可能会直接打印到页面上面
     * evm.util.common.logger.setConfig({
     *      auth: [],//机顶盒特征值...
     *      setAuthToken: function () {
     *          //机顶盒卡号检测方法返回机顶盒卡号或者其他特征值
     *          //需要配置测试用的机顶盒
     *          //onCheckAuth设置false打印盒子的特征信息，配置好输入的机顶盒后，打开onCheckAuth
     *          return config.platform.getStbId();
     *      },
     *      onCheckAuth:true,
     *      logSwitch: true,
     *      logMode: 'server',
     *      postUrl: '/swatapps/ape/interface/logger.php'
     *  });
     *
     *
     * //日志开启在页面js中开启一次即可
     * //默认开启简单日志
     * evm.util.common.logger.setConfig(true);
     * //设置打印级别为warn
     * evm.util.common.logger.setConfig(true,'simple','warn');
     * //开启复杂打印，级别为info，级别可以设置
     * //按159键呼出或关闭打印面板
     * evm.util.common.logger.setConfig(true,'complex','info');
     * //注意，项目上线之后关闭打印
     * //可以注释或者如下设置
     * evm.util.common.logger.setConfig(false);
     * //新增日志打印方式，可以打印输出到服务器
     * evm.util.common.logger.setConfig({
     *      logSwitch: true,
     *      logMode: 'server'
     *  });
     */
    evm.util.common.logger = {
        /**
         * logger配置
         * @private
         */
        config: {
            logSwitch: config.logSwitch,
            logMode: config.logMode,
            logLevel: config.logLevel,
            intervalTime: 5,
            postUrl: '/swatapps/ape/interface/logger.php',
            onCheckAuth: true,
            //输入客户端特征字符串数组
            auth: [],
            /**
             * 输入客户端特征检测函数
             * 如：函数返回机顶盒的CA卡、MAC地址等
             */
            setAuthToken: null
        },
        /**
         * 设置配置参数
         * @param {Object} args 传入参数
         * @param {Boolean} [args.logSwitch] 是否开启日志
         * @param {String} [args.logMode] simple|complex|server，注：server模式依赖laravel项目
         * @param {String} [args.logLevel] debug,info,warn,error
         * @param {Number} [args.intervalTime] 传入页面日志到后台的时间间隔
         * @param {String} [args.postUrl] 处理页面日志后台接口地址
         * @param {Boolean} [args.onCheckAuth] 是否开启输入端检测
         * @param {Array} [args.auth] 输入客户端特征字符串数组
         * @param {Function} [args.setAuthToken] 输入客户端特征检测函数,如：函数返回机顶盒的CA卡、MAC地址等
         * @example
         * //默认开启简单日志
         * evm.util.common.logger.setConfig(true);
         * //设置打印级别为warn
         * evm.util.common.logger.setConfig(true,'simple','warn');
         * //传入对象
         * evm.util.common.logger.setConfig({
         *      logSwitch:true,
         *      logMode:'simple',
         *      logLevel:'warn',
         *      intervalTime:5
         * });
         */
        setConfig: function (args) {
            var obj = {};
            if (isObject(args)) {//传入对象
                obj = args;
            } else {//单个传入参数
                obj.logSwitch = arguments[0];
                obj.logMode = arguments[1];
                obj.logLevel = arguments[2];
            }
            if (obj.logSwitch != null) {
                this.config.logSwitch = obj.logSwitch;
            }
            if (obj.logMode != null) {
                this.config.logMode = obj.logMode;
            }
            if (obj.logLevel != null) {
                this.config.logLevel = obj.logLevel;
            }
            if (obj.intervalTime != null) {
                this.config.intervalTime = obj.intervalTime;
            }
            if (obj.postUrl != null) {
                this.config.postUrl = obj.postUrl;
            }
            if (obj.auth != null) {
                this.config.auth = obj.auth;
            }
            if (obj.setAuthToken != null) {
                this.config.setAuthToken = obj.setAuthToken;
            }
            if (obj.onCheckAuth != null) {
                this.config.onCheckAuth = obj.onCheckAuth;
            }

            configCompleted = true;
        },
        /**
         * debug方法
         * @param {String} str 打印日志内容
         */
        debug: function (str) {
            if (!configCompleted) {
                return;
            }
            str = '[*][debug][' + this.getNowStr() + ']' + getUid() + str;
            log('debug', str);
        },
        /**
         * info方法
         * @param {String} str 打印日志内容
         */
        info: function (str) {
            if (!configCompleted) {
                return;
            }
            str = '[*][info][' + this.getNowStr() + ']' + getUid() + str;
            log('info', str);
        },
        /**
         * warn方法
         * @param {String} str 打印日志内容
         */
        warn: function (str) {
            if (!configCompleted) {
                return;
            }
            str = '[*][warn][' + this.getNowStr() + ']' + getUid() + str;
            log('warn', str);
        },
        /**
         * error方法
         * @param {String} str 打印日志内容
         */
        error: function (str) {
            if (!configCompleted) {
                return;
            }
            str = '[*][error][' + this.getNowStr() + ']' + getUid() + str;
            log('error', str);
        },
        /**
         * 清空所有日志
         * @since 1.1.0
         */
        clear: function () {
            var logElm = document.getElementById('log');
            logElm.innerHTML = '';
        },
        /**
         * 停止打印日志
         * @since 1.1.0
         */
        stop: function () {
            var str = '[info ' + this.getNowStr() + ']停止打印日志';
            log('error', str);
            this.setConfig(false);
        },
        /**
         * 获取当前时间字符串
         */
        getNowStr: function () {
            var now = new Date();
            return dateFormat(now);
        },
        /**
         * 页面销毁时候纪录日志到cookie中
         * log日志跳转来不及保存，保存最后size条日志记录
         */
        persist: function () {
            var size = 10;
            var len = logReposity.length;
            if (len) {
                var logStr = logReposity.splice(len - 10, 10).join('\n');

                // 增加提示日志丢掉提示
                logStr = '\n\n[ ' + this.getNowStr() + ']' + getUid() + '页面跳转丢失部分日志\n' + logStr;

                cookie(logCookieName, logStr, {
                    path: '/'
                });
            }
        }
    };

    /**
     * 日志打印
     * 保留支持老版本
     * @deprecated
     * @param {String} str 打印字符串
     */

    evm.util.common.log = function (str) {
        this.logger.info(str);
    };

    /**
     * 发送日志
     */
    function postData() {
        var config = evm.util.common.logger.config;
        // logMode为server发送日志到服务端
        if (config.logMode !== 'server') {
            return;
        }
        if (config.postUrl) {
            var len = 50;
            var postLogString = logReposity.splice(0, len).join('\n');

            if (postLogString) {
                post(config.postUrl, 'log=' + encodeURIComponent('\n' + postLogString + '\n'), function (response) {
                    // 保存日志成功
                });
            }
        }

        setTimeout(postData, config.intervalTime * 1000);
    }

    postData();

    /**
     * 帮助函数
     */
    function isObject(o) {
        return typeof o === 'object';
    }

    function isFunction(fn) {
        return typeof fn === 'function';
    }

    function isUndefined(obj) {
        return typeof obj === 'undefined';
    }

    function isString(str) {
        return typeof str === 'string';
    }

    function isArray(obj) {
        return Object.prototype.toString.call(obj) === '[object Array]';
    }

    function cookie(name, value, options) {
        if (typeof value != 'undefined') { // name and value given, set cookie
            options = options || {};
            if (value === null) {
                value = '';
                options.expires = -1;
            }
            var expires = '';
            if (options.expires && (typeof options.expires == 'number' || options.expires.toUTCString)) {
                var date;
                if (typeof options.expires == 'number') {
                    date = new Date();
                    date.setTime(date.getTime() + (options.expires * 24 * 60 * 60 * 1000));
                } else {
                    date = options.expires;
                }
                expires = '; expires=' + date.toUTCString(); // use expires attribute, max-age is not supported by IE
            }
            var path = options.path ? '; path=' + options.path : '';
            var domain = options.domain ? '; domain=' + options.domain : '';
            var secure = options.secure ? '; secure' : '';
            document.cookie = [name, '=', encodeURIComponent(value), expires, path, domain, secure].join('');
        } else { // only name given, get cookie
            var cookieValue = null;
            if (document.cookie && document.cookie != '') {
                var cookies = document.cookie.split(';');
                for (var i = 0; i < cookies.length; i++) {
                    var cookie = evm.trim(cookies[i]);
                    // Does this cookie string begin with the name we want?
                    if (cookie.substring(0, name.length + 1) == (name + '=')) {
                        cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                        break;
                    }
                }
            }
            return cookieValue;
        }
    }

    function str2json(str) {
        if (!isString(str)) {
            return str;
        }
        var json;
        try {
            json = eval("(" + str + ")");
        } catch (e) {
            json = str;
        }
        return json;
    }

    function json2str(O) {
        // Key 不使用引号, 所以不能包含空格
        //if( JSON && JSON.stringify ) return JSON.stringify( O );
        var S = '{', sEnd = '}';
        if (isArray(O)) {
            S = '[';
            sEnd = ']';
        }
        for (var i in O) {
            if (isObject(O) && !isArray(O)) {
                S += i + ':';
            }
            if (isUndefined(O[i])) S += '""';
            else if (isObject(O[i])) S += json2str(O[i]);
            else if (isArray(O[i])) S += json2str(O[i]);
            else if (isString(O[i])) S += '"' + O[i] + '"';
            else S += O[i];
            S += ',';
        }
        S = S.substr(0, S.length - 1);
        S += sEnd;
        return S;
    }

    var xhr = null;

    function createXhr() {
        if (xhr) {
            return xhr;
        }
        if (window.XMLHttpRequest)
            return new XMLHttpRequest();
        else {
            var msxmls = new Array('Msxml2.XMLHTTP.5.0', 'Msxml2.XMLHTTP.4.0', 'Msxml2.XMLHTTP.3.0', 'Msxml2.XMLHTTP', 'Microsoft.XMLHTTP');
            for (var i = 0; i < msxmls.length; i++) {
                try {
                    return new ActiveXObject(msxmls[i]);
                } catch (e) {

                }
            }
        }
    }

    function post(url, param, callback, context) {
        xhr = createXhr();
        xhr.open('POST', url, true);
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        xhr.onreadystatechange = function () {
            if (xhr && xhr.readyState === 4 &&
                (xhr.status == 200 || xhr.status == 304)) {
                if ("function" == typeof callback) {
                    callback.call(context || xhr, xhr.responseText);
                    xhr.onreadystatechange = null;
                }

            }
        };
        if (!param) param = null;
        xhr.send(param);
    }

    //输出日期格式：YYYY-mm-dd HH:ii:ss
    function dateFormat(date) {
        function check(n) {
            return n < 10 ? '0' + n : n;
        }

        var yyyy = date.getFullYear(),
            mm = check(date.getMonth() + 1),
            dd = check(date.getDate()),
            HH = check(date.getHours()),
            ii = check(date.getMinutes()),
            ss = check(date.getSeconds());

        return yyyy + '-' + mm + '-' + dd + ' ' +
            HH + ':' + ii + ':' + ss;
    }

    function loadLogFromCookie() {
        var lastPageLogger = evm.cookie(logCookieName);
        cookie(logCookieName, null, {
            path: '/'
        });

        if (lastPageLogger) {
            logReposity.splice(0, 0, lastPageLogger);
        }
    }

    //绑定unload事件
    if (typeof window.addEventListener != "undefined") {
        // 表示不是IE，支持addEventListener方法
        window.addEventListener('unload', function () {
            evm.util.common.logger.persist();
        }, false);
    } else {
        window.attachEvent("onunload", function () {
            evm.util.common.logger.persist();
        });
    }
})(this);

/**
 * 日志打印配置部分参考
 * 该配置代码建议放到项目的全局配置文件中，比如global.js
 */
/**
 try {
    evm.util.common.logger.setConfig({
        logSwitch: true,
        logMode: 'server',
        logLevel: 'debug',
        intervalTime: 5,
        postUrl: '/swatapps/ape/interface/logger.php',
        //正式上线时候必须配置为true
        onCheckAuth: false,
        //输入客户端特征字符串数组
        //正式上线必须配置指定的测试终端
        auth: [],
        //输入客户端特征检测函数
        //如：函数返回机顶盒的CA卡、MAC地址等
        setAuthToken: function () {
            try {
                return evm.stb.extend.execute('CACardNum');
            }
            catch (e) {
                return '';
            }
        }
    });
}
 catch (e) {
    if (console) {
        console.log('logger.js 日志配置报错: ' + e.message);
    }
}
 */
