/**
 * 播放器组件
 * 基于广东省网规范
 * 应用于广东清远项目
 * 依赖：/swatapps/ape/evm.js
 * UI依赖：/swatapps/ape/util/mediaPlayer/ui/main.js
 * @author wuqq@evmtv.com
 * @version 1.0
 *
 * 已测试：1)媒体源为HTTP方式
 * todo:进度条拖动不流畅
 */
(function (window) {
    function MediaPlayer_Gcable(options) {
        //默认配置
        var defaults = {
            //视频显示模式
            windowMode: 'fullscreen',
            //全屏尺寸
            fsSize: {
                width: 1280,
                height: 720
            },
            //窗口尺寸
            wndSize: {
                width: 1280,
                height: 720
            },
            //窗口位置
            wndPos: {
                left: 0,
                top: 0
            },
            //设置循环播放
            loop: true,
            //播放器UI对象
            uiInst: null,
            //进度条移动步长60s
            seekStep: 60
        };
        evm.extend(this, defaults, options);
        //内部属性
        //媒体播放对象
        this._mpInst = null;
        //播放器实例ID
        this._mpID = -1;
        //媒体的来源
        this._source = '';
        //当前状态
        //ToPlay-待播放，Pause-暂停，Stop-停止，Play-播放，Pacing-调节播放速度中，Finish-播放结束
        this._status = 'N/A';
        //媒体类型
        this._mediaType = '';
        //媒体总时长,单位为s
        this._duration = 0;
        //当前播放速度
        this._playSpeed = 1;
        //静音
        this._isMute = true;
        //当前音量
        this._volume = 16;
        //音量调节步长
        this._volumeStep = 1;

        //播放速度取值范围
        this._paceList = this.paceList || [-32, -16, -8, -4, -2, -1, 1, 2, 4, 8, 16, 32];
        //媒体播放系统消息
        this._systemMessage = {
            10931: {code: 10931, t: '开始加载流媒体'},
            10932: {code: 10932, t: '成功加载流媒体'},
            10933: {code: 10933, t: '加载流媒体失败'},
            10934: {code: 10934, t: '成功获取流媒体视频长度'},
            10935: {code: 10935, t: '流媒体视频播放失败'},
            10936: {code: 10936, t: '当前流媒体已经播放结束'}
        }
        //播放时间拖动中
        this._isSeeking = false;
        this._seekingPoint = -1;

        this._intervalId = -1;
    }

    evm.extend(MediaPlayer_Gcable.prototype, {
        _startTimerTask: function () {
            var self = this;
            this._intervalId = setInterval(function () {
                if (!self.getIsSeeking()) {
                    //不是执行拖动进度条操作时候执行
                    self._uiTrigger('updateSeek', self.getElapse(), self.getDuration());
                }
                else {
                    //拖动滚动条状态
                    if (!self._uiTrigger('isBoxVisible')) {
                        self._isSeeking = false;
                    }
                }
                if (!self._duration) {
                    self._duration = self.getDuration();
                    self._uiTrigger('updateTotalTime', self.getDuration());
                }
            }, 1000);
        },
        _stopTimerTask: function () {
            clearInterval(this._intervalId);
        },
        _systemEvent: function (event) {
            var code = event.which;
            //log('code:'+code);
            //log('mediaType:'+this._mediaType);
            //HTTP流媒体
            if (this._mediaType === 'HTTP') {
                switch (code) {
                    case 10931:
                        break;
                    case 10932:
                        break;
                    case 10933:
                        break;
                    case 10934:
                        break;
                    case 10935:
                        break;
                    case 10936:
                        //log('play finished');
                        if (this.loop) {
                            this._status = 'Finish';
                            //收到播放结束消息，先暂停才能重新播放
                            this.pause();
                            this.resume();
                        }
                        break;
                }
            }
        },
        //获取媒体类型
        _getMediaType: function (url) {
            //log('url:'+url);
            var mt = '',
                proto = (url.match(/.*:\/\//) + '').replace('://', '').toLowerCase();

            //log('proto:'+proto);

            switch (proto) {
                case 'ngod':
                case 'rtsp':
                    mt = 'VOD';
                    break;
                case 'c4cam_live':
                case 'https':
                case 'http':
                    mt = 'HTTP';
                    break;
                case 'delivery':
                    mt = 'DVB';
                    break;
                case 'udp':
                    mt = 'IP-UDP';
                    break;
                case 'igmp':
                    mt = 'LiveTV';
                    break;
                case 'file':
                    mt = 'FILE';
                    break;
                default:
                    mt = 'Unknown';
                    break;
            }
            return mt;

        },
        //播放器对应ui对象的方法统一调用入口
        _uiTrigger: function (method) {
            var args = [];
            if (arguments.length > 1) {
                args = Array.prototype.slice.call(arguments, 1);
            }
            if (method && this.uiInst && this.uiInst[method]) {
                return this.uiInst[method].apply(this.uiInst, args);
            }
        },
        //设置媒体的来源
        setSource: function (url) {
            var res = 0;
            this._source = url;
            var mp = this.getInstance();
            try {
                if (mp) {
                    mp.source = this._source;
                    res = 1;
                    this._status = 'ToPlay';

                    this._mediaType = this._getMediaType(url);
                }
            }
            catch (e) {
            }
            return res;
        },
        //播放器相关实例化，实现惰性加载播放器实例
        getInstance: function () {
            var self = this;
            if (this._mpInst == null) {
                try {
                    this._mpInst = new MediaPlayer();
                    this._mpID = this._mpInst.createPlayerInstance("video", 2);

                    this._volume = parseInt(DataAccess.getInfo('MediaSetting', 'OutputVolumn')) || this._volume;
                    this._isMute = this._mpInst.getMute() == 1 ? true : false;

                    this._uiTrigger('adjustVolume', this._volume);
                    this._uiTrigger('toggleMute', this._isMute);

                    this._startTimerTask();

                    evm.on(window, 'systemevent', function (event) {
                        self._systemEvent.call(self, event);
                    }, this);
                }
                catch (e) {
                }
            }
            return this._mpInst;
        },
        getStatus: function () {
            return this._status;
        },
        getIsSeeking: function () {
            return this._isSeeking;
        },
        //功能
        /**
         * 播放
         * @returns {number} 是否成功启用播放 1为是，0为否
         */
        play: function (url) {
            var mp = this.getInstance();
            var res = 0;
            try {
                if (mp) {
                    if (url) {
                        this.setSource(url);
                    }
                    //todo：部分中间件盒子设置媒体源有延时，导致不能及时获取到
                    //暂时屏蔽检测代码
                    /*if (mp.source == '') {
                     throw new Error('媒体来源未设置!');
                     }*/
                    evm.util.common.logger.debug('<gcable> play() status = ' + this._status);
                    if (this._status == 'ToPlay' || this._status == 'Pause' || this._status === 'Finish') {
                        res = mp.play();

                        evm.util.common.logger.debug('<gcable> play() res = ' + res);
                    }
                    else {
                        throw new Error('媒体播放器非"待播放"或者"暂停"状态!');
                    }
                    if (res == 0) {
                        throw new Error('媒体播放器启动播放失败!');
                    }
                    else {
                        this._status = 'Play';
                    }
                }
            }
            catch (e) {
            }
            //log('play start');
            return res;
        },
        /**
         * 暂停
         * @param pauseMode 数值型，取1表示将画面保持在暂停前最后一帧，取0表示将画面设为黑场，默认值为1
         */
        pause: function (pauseMode) {
            var mp = this.getInstance();
            pauseMode = pauseMode == undefined ? 0 : pauseMode;
            var res = 0;
            try {
                if (mp) {
                    res = mp.pause(pauseMode);
                    if (res == 0) {
                        throw new Error('媒体播放器暂停操作失败!');
                    }
                    else {
                        this._status = 'Pause';
                    }
                }
            }
            catch (e) {
            }
            return res;
        },
        /**
         * 按确认键切换播放状态
         * @param {string} op 按键操作别名
         */
        togglePlay: function (op) {
            var mp = this.getInstance(),
                res = 0;

            //log('isSeeking:'+this.getIsSeeking());
            //log('status:'+this._status);
            if (op === 'SELECT') {
                try {
                    if (this.getIsSeeking()) {
                        if (this._status === 'Play') {
                            res = mp.refresh();
                        }
                        this._isSeeking = false;
                        if (res === 0 && this._lastStatus === 'Pause') {
                            this.pause();
                        }
                        return;
                    }

                    if (this._status === 'Pacing') {
                        this.adjustPace(1);
                    }
                    else if (this._status === 'Play') {
                        this.pause();
                    }
                    else if (this._status === 'Pause') {
                        this.play();
                    }
                    else if (this._status === 'Finish') {
                        this.resume();
                    }
                }
                catch (e) {

                }
            }
        },
        //停止退出
        stop: function () {
            var mp = this.getInstance();
            var res = 0;
            try {
                if (mp) {
                    res = mp.releasePlayerInstance();
                }
                if (res == 0) {
                    throw new Error('销毁当前绑定的播放器实例失败!');
                }
                else {
                    this._status = 'Stop';
                }
            }
            catch (e) {
            }
            return res;
        },
        /**
         * 重新播放
         */
        //todo:测试重播不了
        resume: function () {
            //log('resume start.');
            var mp = this.getInstance();
            var res = 0;
            try {
                if (mp) {
                    mp.point = 0;
                    this.play();

                    if (res == 0) {
                        throw new Error('媒体重新播放失败!');
                    }
                    else {
                        this._status = 'Play';
                    }
                }
            }
            catch (e) {

            }
        },
        /**
         * 调节音量
         * @param {string} type
         * @param {number} step 可选
         */
        adjustVolume: function (type, step) {
            if (this._isMute) {
                //调节音量取消静音
                this.toggleMute();
            }
            if (!evm.isNumber(step)) {
                step = 1;
            }
            try {
                if (type == '+' || type == '-') {
                    if (type == '-') {
                        step = -step;
                    }

                    var currentVolume = this.getVolume();
                    var volume = currentVolume + step;

                    if (volume < 0) {
                        volume = 0;
                    }
                    else if (volume > 32) {
                        volume = 32;
                    }

                    if (volume >= 0 && volume <= 32) {
                        this._volume = volume;
                        DataAccess.setInfo('MediaSetting', 'OutputVolumn', volume);

                        this._uiTrigger('adjustVolume', volume);
                    }
                }
                else {
                    throw new Error('调节音量参数type只能取值+或-');
                }
            }
            catch (e) {

            }
        },
        //获取音量
        getVolume: function () {
            return this._volume;
        },
        //切换静音状态
        toggleMute: function () {
            var mp = this.getInstance();
            if (mp) {
                if (this._isMute) {
                    mp.audioUnmute();
                    this._isMute = false;
                }
                else {
                    mp.audioMute();
                    this._isMute = true;
                }
                this._uiTrigger('toggleMute', this._isMute);
            }
        },
        //设置视频窗口位置和大小
        setPosition: function () {
            var mp = this.getInstance();
            var res = 0;
            try {
                if (mp) {
                    if (this.windowMode == 'fullscreen' || this.windowMode == 'fs') {
                        mp.position = '1,0,0,0,0';
                    }
                    else {
                        mp.position = '0,' + this.wndPos.left + ',' + this.wndPos.top + ',' + this.wndSize.width + ',' + this.wndSize.height;
                    }

                    res = mp.refresh();

                    if (res == 0) {
                        throw new Error('设置视频窗口位置和大小失败!');
                    }
                }
            }
            catch (e) {

            }
            return res;

        },
        //切换视频全屏或窗口
        toggleFullScreen: function () {
            var res = 0;
            var lastWM = this.windowMode;
            if (this.windowMode == 'fullscreen' || this.windowMode == 'fs') {
                this.windowMode = 'window';
            }
            else {
                this.windowMode = 'fullscreen';
            }

            res = this.setPosition();
            try {
                if (res == 0) {
                    this.windowMode = lastWM;
                    throw new Error('切换视频全屏或窗口失败!');
                }
            }
            catch (e) {

            }
            return res;
        },
        /**
         * 调节播放速度
         * @param {string|number} type 取值+或-或数字
         * 测试不支持快退操作
         */
        adjustPace: function (type) {
            var index;
            var currentPace = this.getPace();
            evm.util.common.logger.debug('<gcable> adjustPace() currentPace = ' + currentPace);
            var len = this._paceList.length;
            var res = 0;
            var targetPace;
            //快进和快退的默认值
            //快进取_paceList最小的正值
            //快退取_paceList最大的负值
            var defaultFF;
            var defaultFB;

            if (!isNaN(parseInt(type))) {
                targetPace = parseInt(type);
            }
            else {
                type = type || '+';
                if (currentPace === 1) {
                    for (var i = 0; i < len; i++) {
                        if (this._paceList[i] < 0) {
                            defaultFB = this._paceList[i];
                        }
                        else {
                            defaultFF = this._paceList[i];
                            break;
                        }
                    }

                    if (type === '+') {
                        targetPace = defaultFF;
                    }
                    else {
                        targetPace = defaultFB;
                    }
                }
                else {
                    for (var i = 0; i < len; i++) {
                        if (this._paceList[i] === currentPace) {
                            index = i;
                            break;
                        }
                    }

                    if (type == '+' && (index + 1 < len)) {
                        index++;
                    }
                    else if (type == '-' && (index - 1 >= 0)) {
                        index--;
                    }
                    targetPace = this._paceList[index];
                }
            }
            try {
                evm.util.common.logger.debug('<gcable> adjustPace() targetPace = ' + targetPace);
                if (targetPace != null) {
                    var mp = this.getInstance();
                    if (mp) {
                        mp.pace = targetPace;
                        res = mp.refresh();
                    }

                    evm.util.common.logger.debug('<gcable> adjustPace() 调节倍速操作结果res = ' + res);

                    //todo 如果到达边界时什么情况
                    if (this.getPace() != 1 && this._status != 'Pacing') {
                        this._status = 'Pacing';
                    }
                    else if (this.getPace() == 1 && res == 1) {
                        this._status = 'Play';
                    }
                    if (res === 0) {
                        throw new Error('设置播放速度失败!');
                    }
                }
                else {
                    throw new Error('获取播放速度值不正确!');
                }
            }
            catch (e) {
            }
        },
        //获取播放速度
        getPace: function () {
            var mp = this.getInstance();
            if (mp) {
                return mp.pace;
            }
        },
        /**
         * 设置播放点
         * @param {string} type '+'或'-'
         */
        seek: function (type) {
            var mp = this.getInstance();
            try {
                if (mp) {
                    var duration = this.getDuration(),
                        val;

                    if (!this.getIsSeeking()) {
                        val = this._seekingPoint = this.getElapse();
                    }
                    else {
                        val = this._seekingPoint;
                    }
                    //log('seek.val.1:'+val);

                    if (type === '-') {
                        val -= this.seekStep;
                    }
                    else {
                        val += this.seekStep;
                    }
                    //log('seek.val.2:'+val);
                    //log('seek.duration:'+duration);

                    if (!duration) {
                        throw new Error('媒体播放时长未获取到!');
                    }

                    if (val < 0) {
                        val = 0;
                    }
                    else if (val > duration) {
                        val = duration;
                    }
                    if (val >= 0 && val <= duration) {
                        mp.point = val;
                        this._seekingPoint = val;

                        if (!this._isSeeking) {
                            this._isSeeking = true;
                            if (this.getStatus() === 'Pause') {
                                this._lastStatus = 'Pause';
                                this.play();
                            }
                        }

                        this._uiTrigger('seek', val, duration);
                    }
                }
            }
            catch (e) {

            }
        },
        /**
         * 获取播放媒体的总时长
         * @returns {number}
         */
        getDuration: function () {
            if (this._duration) {
                return this._duration;
            }

            var mp = this.getInstance();
            var totalDuration = 0;
            if (mp) {
                totalDuration = mp.getMediaDuration();//返回格式为'hh:mm:ss'
                if (totalDuration != "0") {
                    var timeArr = totalDuration.split(':');
                    if (timeArr.length === 3) {
                        timeArr[0] = parseInt(timeArr[0]);
                        timeArr[1] = parseInt(timeArr[1]);
                        timeArr[2] = parseInt(timeArr[2]);

                        totalDuration = timeArr[0] * 3600 + timeArr[1] * 60 + timeArr[2];
                    }
                }
            }

            if (isNaN(totalDuration)) {
                totalDuration = 0;
            }
            return totalDuration;
        },
        /**
         * 获取已播放的时长
         * @returns {number} 单位为秒
         */
        getElapse: function () {
            var mp = this.getInstance();
            var elapsed = 0;
            if (mp) {
                elapsed = mp.currentPoint;
            }
            return elapsed;
        }
    });

    evm.namespace('evm.util.mediaPlayer');
    evm.util.mediaPlayer.gcable = function (options) {
        return new MediaPlayer_Gcable(options);
    }
})(this);