/**
 * 播放器组件
 * 应用地区：
 * 		1.ipanel
 * 应用项目：
 * 		1.政府公众号-电子白板
 * 依赖：/swatapps/ape/evm.js
 * UI依赖：/swatapps/ape/util/mediaPlayer/ui/main.js
 * @author yangty@evmtv.com
 * @version 1.0
 *
 */
(function (window) {
	function MediaPlayer_ipanel(options) {
		//默认配置
		var defaults = {
			//视频显示模式
			windowMode: 'fullscreen',
			//全屏尺寸
			fsSize: {
				width: 1280,
				height: 720
			},
			//窗口尺寸
			wndSize: {
				width: 1280,
				height: 720
			},
			//窗口位置
			wndPos: {
				left: 0,
				top: 0
			},
			//设置循环播放
			loop:true,
			//播放器UI对象
			uiInst:null,
			//进度条移动步长60s
			seekStep:60,
			maxVolume:32
		};
		evm.extend(this, defaults, options);
		//内部属性
		//媒体播放对象
		this._mpInst = null;
		//播放器实例ID
		this._mpID = -1;
		//媒体的来源
		this._source = '';
		//当前状态
		//ToPlay-待播放，Pause-暂停，Stop-停止，Play-播放，Pacing-调节播放速度中，Finish-播放结束
		this._status = 'N/A';
		//媒体类型
		this._mediaType = '';
		//媒体总时长,单位为s
		this._duration = 0;
		//当前播放速度
		this._playSpeed = 1;
		//静音
		this._isMute = true;
		//当前音量
		this._volume = 16;
		//音量调节步长
		this._volumeStep = 1;

		//播放速度取值范围
		this._paceList = [-33,-12,1,12,33];
		//播放速度
		this._pace = 1;
		//播放时间拖动中
		this._isSeeking = false;
		this._seekingPoint = -1;

		this._intervalId = -1;

		//播放时间
		this._mediaElapsed = 0;
		//上次校准播放时间
		this._lastElapsed = 0;
		//恢复播放延迟校准播放时间
		this._elapsedTimeOut = null;
		this._intervalId = null;
		this._lastElapsedCheckId = null;
		//上次播放时间
		this._lastMediaElapsed = 0;
	}

	evm.extend(MediaPlayer_ipanel.prototype, {
		_startTimerTask:function(){
			var self = this;
			this._intervalId = setInterval(function(){
				//累加播放时间
				self._mediaElapsedAdd();

				if(!self.getIsSeeking()){
					//不是执行拖动进度条操作时候执行
					self._uiTrigger('updateSeek',self.getElapsed(),self.getDuration());
				}
				else{
					//拖动滚动条状态
					if(!self._uiTrigger('isBoxVisible')) {
						self._isSeeking = false;
					}
				}
				if(!self._duration){
					self._duration = self.getDuration();
					self._uiTrigger('updateTotalTime',self.getDuration());
				}

				if(self.getElapsed() >= self.getDuration() && self.getDuration() != 0 || self.getElapsed() === self._lastMediaElapsed){
					//播放结束
					self.resume();
				}
				self._lastMediaElapsed = self.getElapsed();
			},1000);
		},
		_stopTimerTask:function(){
			clearInterval(this._intervalId);
		},
		//获取媒体类型
		_getMediaType:function(url){
			//log('url:'+url);
			var mt = '',
				proto = (url.match(/.*:\/\//)+'').replace('://','').toLowerCase();

			//log('proto:'+proto);

			switch(proto){
				case 'ngod':
				case 'rtsp':
					mt = 'VOD';
					break;
				case 'c4cam_live':
				case 'https':
				case 'http':
					mt = 'HTTP';
					break;
				case 'delivery':
					mt = 'DVB';
					break;
				case 'udp':
					mt = 'IP-UDP';
					break;
				case 'igmp':
					mt = 'LiveTV';
					break;
				case 'file':
					mt = 'FILE';
					break;
				default:
					mt = 'Unknown';
					break;
			}
			return mt;

		},
		//播放器对应ui对象的方法统一调用入口
		_uiTrigger:function(method){
			var args = [];
			if(arguments.length > 1){
				args = Array.prototype.slice.call(arguments,1);
			}
			if(method && this.uiInst && this.uiInst[method]){
				return this.uiInst[method].apply(this.uiInst,args);
			}
		},
		//播放时间累加器
		_mediaElapsedAdd:function(){
			var mp = this.getInstance();
			if(mp){
				var status = this.getStatus();
				if(status === 'play'){
					this._mediaElapsed++;
				}
				else if(status === 'forward'){
					this._mediaElapsed += this._pace;
				}
				else if(status === 'backward'){
					if(this._mediaElapsed + this._pace < 0){
						this._mediaElapsed = 0;
					}
					else{
						this._mediaElapsed += this._pace;
					}
				}
				else if(status === 'stop'){
					return 0;
				}
				else if(status === 'pause'){
				}
			}
		},
		//上次播放时间校准
		_lastElapsedCheck:function(){
			var nowElapsed = media.AV.elapsed;
			if (this._lastElapsed != nowElapsed) {
				this._lastElapsed = nowElapsed;
				this._mediaElapsed = this._lastElapsed;
			}
		},
		//播放器相关实例化，实现惰性加载播放器实例
		getInstance: function () {
			var self = this;
			if (this._mpInst == null) {
				try {
					this._mpInst = media.AV;

					this._volume = media.sound.value === 0 ? 0 : (media.sound.value || this._volume);
					this._isMute = media.sound.isMute == 1 ? true : false;

					//log('media.sound.isMute:'+media.sound.isMute);
					//log('media.sound.value:'+media.sound.value);

					this._uiTrigger('adjustVolume',this._volume,this.maxVolume);
					this._uiTrigger('toggleMute',this._isMute);
				}
				catch (e) {
				}
			}
			return this._mpInst;
		},
		getStatus:function(){
			var mp = this.getInstance();
			if(mp){
				return mp.status;
			}
			return 'N/A';
		},
		getIsSeeking:function(){
			return this._isSeeking;
		},
		//功能
		/**
		 * 播放
		 * @returns {number} 是否成功启用播放 1为是，0为否
		 */
		play: function (url,mediaType) {
			var mp = this.getInstance();
			
			var res = 0;
			var playType = 7;//参考抓包代码，具体不清楚意义
			try {
				if (mp) {
					if(url){
    					
    					DVB.stopAV(1);
    					
						if(mediaType){
							this._mediaType = mediaType;
						}
						else{
							this._mediaType = this._getMediaType(url);
						}

						this._source = url;
						iPanel.eventFrame.service_mode_change("IP");
						res = mp.open(url, "VOD");
						evm.log('mediaPlaer.play return by open:'+res);
					}
					
					/*
					if (res == 1) {
						res = mp.play();

						evm.log('mediaPlaer.play return by play:'+res);
						if(mediaType !== 'LIVE'){
							//直播不执行时间校准和定时任务
							var self = this;
							this._lastElapsedCheckId = setInterval(function(){
								self._lastElapsedCheck();
							},500);
							this._startTimerTask();
						}

					}
					else {
						throw new Error('提示<br><br>播放异常(errorCode:200)请退出<br><br>咨询请拨客服热线88617777');
					}
					this._status = this.getStatus();
					*/
				}
			}
			catch (e) {
			}
			//log('play start');
			return res;
		},
		/**
		 * 暂停
		 */
		pause: function () {
			var mp = this.getInstance();
			var res = 0;
			try {
				if (mp) {
					res = mp.pause();
					this._status = mp.status;
				}
			}
			catch (e) {
			}
			return res;
		},
		/**
		 * 按确认键切换播放状态
		 * @param {string} op 按键操作别名
		 */
		togglePlay:function(op){
			var self = this,
				mp = this.getInstance(),
				res = 0;

			//log('isSeeking:'+this.getIsSeeking());
			//log('status:'+this._status);
			if(op === 'SELECT'){
				if(mp){
					var status = this.getStatus();
					if (status == "pause" || status == "forward" || status == "backward" || status == "stop") {
						mp.play();
						if (this._elapsedTimeOut != null) {
							clearTimeout(this._elapsedTimeOut);
						}
						this._elapsedTimeOut = setTimeout(function () {
							self._mediaElapsed = mp.elapsed + 1;
						}, 1000);
					}
					else if(status === 'play'){
						this.pause();
					}
				}
			}
		},
		//停止退出
		stop: function () {
			var mp = this.getInstance();
			var res = 0;
			try {
				if (mp) {
    				DVB.stopAV(1);
    				mp.stop();
					res = mp.close();
					DVB.stopAV();
					evm.log('mediaPlayer.stop:'+res);
					this._status = this.getStatus();
				}
			}
			catch (e) {
			}
			return res;
		},
		/**
		 * 重新播放
		 */
		resume: function () {
			//log('resume start.');
			var mp = this.getInstance();
			var res = 0;
			try{
				if(mp){
					//可能是推流不支持seek方法
					//this.seek(0);
					//log('resume:close media');
					this._lastElapsed = 0;
					this._mediaElapsed = 0;
					if(this._lastElapsedCheckId != null){
						clearInterval(this._lastElapsedCheckId);
					}
					if(this._intervalId != null){
						clearInterval(this._intervalId);
					}

					mp.close();
					//log('resume:start to play');
					this.play(this._source);
				}
			}
			catch(e){

			}
		},
		/**
		 * 调节音量
		 * @param {string} type
		 * @param {number} step 可选
		 */
		adjustVolume: function (type,step) {
			if(this._isMute){
				//调节音量取消静音
				this.toggleMute();
			}
			if(!evm.isNumber(step)){
				step = 1;
			}
			try{
				if(type == '+' || type == '-'){
					if(type == '-'){
						step = -step;
					}

					var currentVolume = this.getVolume();
					var volume = currentVolume + step;

					if(volume < 0){
						volume = 0;
					}
					else if(volume > this.maxVolume){
						volume = this.maxVolume;
					}

					if(volume >= 0 && volume <= this.maxVolume){
						this._volume = volume;
						media.sound.value = volume;

						this._uiTrigger('adjustVolume',volume,this.maxVolume);
					}
				}
				else{
					throw new Error('调节音量参数type只能取值+或-');
				}
			}
			catch(e){

			}
		},
		//获取音量
		getVolume: function () {
			return this._volume;
		},
		//切换静音状态
		toggleMute: function () {
			var mp = this.getInstance();
			var res = 0;
			try{
				if(mp){
					if(this._isMute){
						media.sound.mute(0);
						this._isMute = false;
					}
					else{
						media.sound.mute();
						this._isMute = true;
					}
					this._uiTrigger('toggleMute',this._isMute);
				}
			}
			catch(e){

			}
		},
		//设置视频窗口位置和大小
		setPosition: function () {
			var mp = this.getInstance();
			try{
				if(mp){
					if(this.windowMode == 'fullscreen' || this.windowMode == 'fs'){
						media.video.fullScreen(1);
					}
					else{
						media.video.setPosition(this.wndPos.left,this.wndPos.top,this.wndSize.width,this.wndSize.height);
					}
				}
			}
			catch(e){

			}

		},
		//切换视频全屏或窗口
		toggleFullScreen: function () {
			var res = 0;
			if(this.windowMode == 'fullscreen' || this.windowMode == 'fs'){
				this.windowMode = 'window';
			}
			else{
				this.windowMode = 'fullscreen';
			}

			this.setPosition();
		},
		/**
		 * 调节播放速度
		 * @param {string|number} type 取值+或-或数字
		 * 测试不支持快退操作
		 */
		adjustPace: function (type) {
			var index;
			var currentPace = this.getPace();
			//log('currentPace:'+currentPace);
			var len = this._paceList.length;
			var res = 0;
			var targetPace;

			if(!isNaN(parseInt(type))){
				targetPace = parseInt(type);
			}
			else{
				type = type || '+';
				for(var i = 0; i < len;i++){
					if(this._paceList[i] === currentPace){
						index = i;
						break;
					}
				}

				if(type == '+' && (index+1 < len)){
					index++;
				}
				else if(type == '-' && (index-1 >= 0)){
					index--;
				}
				targetPace = this._paceList[index];

				//log('targetPace:'+targetPace);
			}
			try{
				if(targetPace != null){
					var mp = this.getInstance();
					var status = this.getStatus();
					if(targetPace > 0){
						if(status === 'play' || status === 'backward'){
							setTimeout(function(){
								mp.forward(targetPace);
							},300);
						}
						else if(status === 'pause'){
							mp.resume();
							setTimeout(function(){
								mp.forward(targetPace);
							},500);
						}
						else if(status === 'forward'){
							setTimeout(function(){
								mp.forward(targetPace);
							},300);
						}
						this._pace = targetPace;
					}
					else if(targetPace < 0){
						if(status === 'play' || status === 'backward'){
							setTimeout(function(){
								mp.backward(targetPace);
							},300);
						}
						else if(status === 'pause'){
							mp.resume();
							setTimeout(function(){
								mp.backward(targetPace);
							},500);
						}
						else if(status === 'forward'){
							setTimeout(function(){
								mp.backward(targetPace);
							},300);
						}
						this._pace = targetPace;
					}

				}
				else{
					throw new Error('获取播放速度值不正确!');
				}
			}
			catch(e){
			}
		},
		//获取播放速度
		getPace:function(){
			return this._pace;
		},
		/**
		 * 设置播放点
		 * @param {string} type '+'或'-'
		 */
		seek: function (type) {
			var mp = this.getInstance();
			try{
				if(mp){
					var duration = this.getDuration(),
						val;

					if(!this.getIsSeeking()){
						val = this._seekingPoint = this.getElapsed();
					}
					else{
						val = this._seekingPoint;
					}
					//log('seek.val.1:'+val);

					if(type === '-'){
						val -= this.seekStep;
					}
					else{
						val += this.seekStep;
					}
					//log('seek.val.2:'+val);
					//log('seek.duration:'+duration);

					if(!duration){
						throw new Error('媒体播放时长未获取到!');
					}

					if(val < 0){
						val = 0;
					}
					else if(val > duration){
						val = duration;
					}
					if(val >= 0 && val <= duration){
						mp.seek(val);
						this._mediaElapsed = val;
						this._seekingPoint = val;

						if(!this._isSeeking){
							this._isSeeking = true;
						}

						this._uiTrigger('seek',val,duration);
					}
				}
			}
			catch(e){

			}
		},
		/**
		 * 获取播放媒体的总时长
		 * @returns {number}
		 */
		getDuration: function () {
			if(this._duration){
				return this._duration;
			}

			var mp = this.getInstance();
			var totalDuration = 0;
			if(mp){
				totalDuration = mp.duration;
			}

			if(isNaN(totalDuration)){
				totalDuration = 0;
			}
			return totalDuration;
		},
		/**
		 * 获取已播放的时长
		 * @returns {number} 单位为秒
		 */
		getElapsed: function () {
			var mp = this.getInstance();
			var elapsed = 0;
			if(mp){
				elapsed = this._mediaElapsed;
			}
			return elapsed;
		}
	});

	evm.namespace('evm.util.mediaPlayer');
	evm.util.mediaPlayer.ipanel = function (options) {
		return new MediaPlayer_ipanel(options);
	}
})(this);