evm.namespace('evm.util');
(function () {
    /**
     * toast提示组件
     */
    function logIt(type, message) {
        if (arguments.length === 1) {
            message = type;
            type = 'debug';
        }
        if (arguments.length >= 1) {
            try {
                evm.util.common.logger[type]('evm.util.Tipbox' + message);
            } catch (e) {
                if (console) {
                    console.warn('打印日志报错:' + e.message);
                    console.warn('可能原因:未引入日志打印文件，如果/swatapps/ape/util/common-1.3.1.js，之后版本，需要单独引入/swatapps/ape/util/logger.js');
                }
            }
        }
    }

    var Toast = function (options) {
        var defaults = {
            /*基类配置项*/
            prefix: 'Ape',
            compName: 'toast',
            parentComp: null,
            /*本身的配置项*/
            // toast类型
            // warn error normal
            type: 'normal',
            msg: '',
            timeout: 5
        };

        var config = evm.util.common.extend(defaults, options);
        evm.ext.comp.BaseComponent.call(this, config);

        //保存原始配置
        this._config = evm.util.common.extend(true, {}, config);
        evm.util.common.extend(this, config);

        this._timer = -1;
    };
    Toast.prototype = {
        constructor: Toast,
        /**
         * 初始化容器DOM元素
         * @param callback
         * @private
         */
        _createDom: function (callback) {
            var contentTpl =
                '<div class="toast-top"><span class="top-left-corner"></span><span class="toast-blank"></span><span class="top-right-corner"></span></div>' +
                '<div class="toast-content">' +
                '<div class="toast-icon"></div>' +
                '<div class="toast-msg"></div>' +
                '</div>' +
                '<div class="toast-bottom"><span class="bottom-left-corner"></span><span class="toast-blank"></span><span class="bottom-right-corner"></span></div>';

            var containerElm = evm.$('container') || document.getElementsByTagName('body')[0];

            this._boxId = this.comp_name;
            this._boxElem = evm.$(this.boxId);
            // 生成容器DOM
            if (!this._boxElem) {
                this._boxElem = evm.createElem('div', {
                    id: this._boxId,
                    className: this.class_name
                }, containerElm);

                evm.html(this._boxElem, contentTpl);
                callback();
            }
        },
        _destroy: function () {
            if (this._boxElem) {
                var parent = this._boxElem.parentElement;
                parent.removeChild(this._boxElem);

                this._boxElem = null;
            }
        },
        destroy: function () {
            this._destroy();
        },
        /**
         * 提示框初始化
         */
        init: function () {
            var self = this;
            this._createDom(evm.util.common.proxy(this, function () {
                this._updateMsg();
                this._updateIcon();

                setTimeout(function () {
                    self._destroy();
                }, this.timeout * 1000);
            }));
        },
        /**
         * 更新提示内容
         * @param {String} [newMsg] 新提示内容
         * @private
         */
        _updateMsg: function (newMsg) {
            if (newMsg) {
                this.msg = newMsg;
            }

            var msgElem = evm.getElementsByClassName('toast-msg', this._boxElem, 'div')[0];
            evm.html(msgElem, this.msg);
        },
        /**
         * 更新提示图标
         */
        _updateIcon: function (type) {
            if (type) {
                this.type = type;
            }

            var msgElem = evm.getElementsByClassName('toast-icon', this._boxElem, 'div')[0];
            evm.addClass(msgElem, this.type);
        },
        /**
         * 设置没有按钮操作时，提示框自动隐藏的时间，单位为s
         * @param {Number} timer
         */
        setTimer: function (timer) {
            timer = +timer;
            if (timer >= 0) {
                this.timeout = timer;
            }
        }
    };

    evm.util.toast = function (options) {
        return new Toast(options);
    };
})();
