/**
 * Created by aliewnare on 2015/11/26.
 * 盒子端文章展示组件
 * 可以分页或者滚动浏览文章，分页是利用CSS模拟分页
 * @import /swatapps/ape/evm.js /swatapps/ape/widget-v1/base.js
 *
 * DOM结构
 * <div id="article">
 *     <div>内容</div>
 * </div>
 */
function Article(options) {
	this._parent = typeof options.parent === 'string' ? evm.$(options.parent) : options.parent;
	this._article = options.article;
	this._width = parseFloat(options.width);
	this._height = parseFloat(options.height);
	this._baseLeft = parseFloat(options.baseLeft) || 0;
	this._baseTop = parseFloat(options.baseTop) || 0;
	//浏览文章方式 scroll或paging
	this._type = options.type === 'scroll' ? 'scroll' : 'paging';
	if (!this._parent) {
		throw new Error('Article实例化失败：未找到父元素节点');
	}
	if (!this._height) {
		throw new Error('Article实例化失败：父元素高度height未正确配置');
	}
	this._innerElm = null;
	//标识所有图片加载完成
	//用于计算innerElm的正确高度
	this._allImagesLoadFinished = false;
	this._innerHeight = 0;
	this.timer = null;
	//分页属性
	this._pageIndex = 1;
	this._pageTotalNum = 1;
	//滚动配置属性
	this.dir;
	this.step = 3;
	this.factor = 1;
	this.max = 0;
	this.min = 0;
	//接口方法
	this.onFlipover = options.onFlipover;
	this._init();
}
extend(Article, KeyListenBase);
evm.extend(Article.prototype, {
	action: function (alias) {
		if (!this._allImagesLoadFinished) {
			return;
		}
		if (this._type === 'paging') {
			return;
		}
		switch (alias) {
			case 'UP':
				clearTimeout(this.timer);
				this._scrollAction(alias);
				break;
			case 'RIGHT':
				this._speedControl('+');
				break;
			case 'DOWN':
				clearTimeout(this.timer);
				this._scrollAction(alias);
				break;
			case 'LEFT':
				this._speedControl('-');
				break;
			case 'SELECT':
				this._doSelect();
				break;
		}
	},
	destroy:function(){
		this.removeDOM();

		//销毁数据
		this._article = '';

		this._trigger('onDestroy',[]);
	},
	init:function(){
		this._init();
	},
	prev: function () {
		var pi = this._pageIndex;
		if (pi - 1 >= 1) {
			this._pageIndex--;
			this._trigger('onFlipover', [this._pageIndex, this._pageTotalNum,'prev']);
			var innerTop = parseFloat(evm.getStyle(this._innerElm, 'top'));
			this._innerElm.style.top = innerTop + this._height + 'px';
		}
	},
	next: function () {
		var pi = this._pageIndex;
		if (pi + 1 <= this._pageTotalNum) {
			this._pageIndex++;
			this._trigger('onFlipover', [this._pageIndex, this._pageTotalNum,'next']);
			var innerTop = parseFloat(evm.getStyle(this._innerElm, 'top'));
			this._innerElm.style.top = innerTop - this._height + 'px';
		}
	},
	removeDOM:function(){
		evm.removeChildren(this._parent);
	},
	setArticle:function(article){
		this._article = article;
	},
	_allImagesLoad: function () {
		if (!this._innerElm) {
			return;
		}
		var imgs = this._innerElm.getElementsByTagName('img');
		var loadImgCount = 1;
		var me = this;
		if(imgs.length){
			for (var i = 0; i < imgs.length; i++) {
				var img = imgs[i];
				img.onload = function () {
					if (loadImgCount === imgs.length) {
						//ipanel3.0上面动态获取高度需要延时处理
						setTimeout(function(){
							me._allImagesLoadFinished = true;
							me._innerHeight = parseFloat(evm.getStyle(me._innerElm, 'height')) || me._innerElm.clientHeight;
							me._pageTotalNum = Math.ceil(me._innerHeight / me._height);
							me._trigger('onFlipover', [me._pageIndex, me._pageTotalNum]);

							//处理图片
							//获取图片的宽度
							imgW = parseFloat(evm.getStyle(this,'width'));
							imgH = parseFloat(evm.getStyle(this,'height'));
							if(imgW > me._width){
								evm.css(this,'width:'+me._width+'px;');

								//设置图片padding-bottom使整体高度为36的整数倍
								imgH = (imgH/imgW)*me._height;

								evm.css(this,'padding-bottom:'+(Math.ceil(imgH/33)*33-imgH)+'px');
							}
							else{
								//evm.css(this,'padding-left:'+(me._height-imgW)/2+'px;');

								evm.css(this,'padding-bottom:'+(Math.ceil(imgH/33)*33-imgH)+'px');
							}
						},200);
					}
					loadImgCount++;
				};
				img.onerror = function () {
					loadImgCount++;
				}
			}
		}
		else{
			//ipanel3.0上面动态获取高度需要延时处理
			setTimeout(function(){
				me._allImagesLoadFinished = true;
				me._innerHeight = parseFloat(evm.getStyle(me._innerElm, 'height')) || me._innerElm.clientHeight;
				me._pageTotalNum = Math.ceil(me._innerHeight / me._height);
				me._trigger('onFlipover', [me._pageIndex, me._pageTotalNum]);
			},200);
		}

	},
	_init: function () {
		if(!this._article){
			return;
		}
		this._parent.style.position = 'absolute';
		evm.addClass(this._parent,'article');
		if (this._width) {
			this._parent.style.width = this._width + 'px';
		}
		this._parent.style.height = this._height + 'px';
		this._parent.style.left = this._baseLeft + 'px';
		this._parent.style.top = this._baseTop + 'px';
		this._parent.style.overflow = 'hidden';
		this._create();
		this._allImagesLoad();
	},
	_create: function () {
		this._innerElm = document.createElement('DIV');
		this._innerElm.className = 'inner';
		this._parent.appendChild(this._innerElm);
		this._innerElm.style.position = 'absolute';
		this._innerElm.style.width = this._width+'px';
		this._innerElm.style.top = '0px';
		this._innerElm.style.left = '0px';
		evm.html(this._innerElm, this._article);
	},
	_doSelect: function () {
		clearTimeout(this.timer);
	},
	_trigger: function (eventName, args) {
		if (typeof this[eventName] === 'function') {
			this[eventName].apply(this, args);
		}
	},
	_scrollAction: function (dir) {
		var me = this;
		if (this.dir !== dir) {
			//更换滚动方向
			clearTimeout(this.timer);
		}
		this.dir = dir;
		//配置滚动
		var step = this.step * this.factor;
		if (dir === 'DOWN' && step > 0) {
			step = -step;
		}
		else if (dir === 'UP' && step < 0) {
			step = -step;
		}
		function check() {
			if (isNaN(me.min) || me.min >= 0) {
				return false;
			}
			var top = parseFloat(evm.getStyle(me.inner, 'top')) || 0;
			top += step;
			if (top >= me.min && top <= me.max) {
				me.inner.style.top = top + 'px';
				return true;
			}
			return false;
		}

		if (check()) {
			this.timer = setTimeout(function () {
				me._scrollAction.call(me, dir);
			}, 100);
		}
		else {
			clearTimeout(this.timer);
		}
	},
	_speedControl: function (type) {
		if (type === '+' && this.factor < 3) {
			this.factor += 1;
		}
		else if (type === '-' && this.factor > 1) {
			this.factor -= 1;
		}
	}
});
