/**
 * @file 输入框
 * @import /swatapps/ape/evm.js /swatapps/ape/widget-v1/base.js
 * @param options
 * @constructor
 * @createTime:2015-11-16
 * @des
 * options
 *
 * @example
 *
 */
function Input(options){
    //组件DOM元素的容器id属性
    this._boxId = options.boxId;
    //构建class属性
    this._prefix = 'input';

    //输入框的值
    this._value = '';
	//Input框的高度和宽度
	this._width = options.width;
	this._height = options.height;
    //Input绝对定位的初始位置
    this._baseLeft = options.baseLeft || 0;
    this._baseTop = options.baseTop || 0;

    //创建选中的高亮背景层
    this._activeLayerElm;
    this._activeLayerWidth = options.activeLayerWidth;
    this._activeLayerHeight = options.activeLayerHeight;
    this._activeLayerDx = options.activeLayerDx;
    this._activeLayerDy = options.activeLayerDy;

    //配置接口方法
	this.onWidgetInit = options.onWidgetInit;
    this.onWidgetFocus = options.onWidgetFocus;
    this.onWidgetBlur = options.onWidgetBlur;
    this.doSelect = options.doSelect || function(){};
    this.onEdit = options.onEdit || function(){};
}
extend(Input,KeyListenBase);
evm.extend(Input.prototype,{
    action:function(alias,event){
        switch(alias){
            case 'UP':
            case 'RIGHT':
            case 'DOWN':
            case 'LEFT':
                this._blur();
                this._trigger('onWidgetBlur',[alias]);
                break;
            case 'SELECT':
                this.doSelect(alias);
                break;
            case undefined:
                this._focus();
                break;
        }

        this.onEdit(alias,event);
    },
	getBaseLeft:function(){
		return this._baseLeft;
	},
	getBaseTop:function(){
		return this._baseTop;
	},
	getBoxId:function(){
		return this._boxId;
	},
	getHeight:function(){
		return this._height;
	},
    getValue:function(){
        var boxElm = evm.$(this._boxId);
        var input = evm.getElementsByClassName('input','div',boxElm);
        if(input){
            this._value = evm.text(input[0]);
            return this._value;
        }
    },
	getWidth:function(){
		return this._width;
	},
    init:function(){
        this._create();
		this._trigger('onWidgetInit',[]);
    },
    setValue:function(str){
        var boxElm = evm.$(this._boxId);
        var input = evm.getElementsByClassName('input','div',boxElm);
        if(input){
            evm.text(input[0],str);
        }
    },
    _create:function(){
        var boxElm = evm.$(this._boxId);

        var input = document.createElement('DIV');
        input.className = 'input';
        input.style.position = 'absolute';
        input.style.left = this._baseLeft + 'px';
        input.style.top = this._baseTop + 'px';
		input.style.width = this._width + 'px';
		input.style.height = this._height + 'px';

        boxElm.appendChild(input);

        //生成高亮背景层
        if(this._activeLayerWidth && this._activeLayerHeight && !this._activeLayerElm){
            this._activeLayerElm = document.createElement('div');
            this._activeLayerElm.style.display = 'none';
            this._activeLayerElm.style.position = 'absolute';
            this._activeLayerElm.className = this._boxId + '-' + this._prefix + '-activeLayer activeLayer';
            this._activeLayerElm.style.width = this._activeLayerWidth + 'px';
            this._activeLayerElm.style.height = this._activeLayerHeight + 'px';
            this._activeLayerElm.style.left = this._baseLeft + this._activeLayerDx + 'px';
            this._activeLayerElm.style.top = this._baseTop + this._activeLayerDy + 'px';
            document.getElementsByTagName('body')[0].appendChild(this._activeLayerElm);
        }
    },
    _focus:function(){
        var boxElm = evm.$(this._boxId);
        var input = evm.getElementsByClassName('input','div',boxElm);
        if(input.length){
            evm.addClass(input[0],'focus');
            this._activeLayerElm.style.display = 'block';
        }

        this._trigger('onWidgetFocus',[]);
    },
    _blur:function(){
        var boxElm = evm.$(this._boxId);
        var input = evm.getElementsByClassName('input','div',boxElm);
        if(input.length){
            evm.removeClass(input[0],'focus');
            this._activeLayerElm.style.display = 'none';
        }
    },
	_trigger:function(eventName,args){
		if(typeof this[eventName] === 'function'){
			this[eventName].apply(this,args);
		}
	}
});