/*******************************************************************************

 ext.mediaPlayer.js
 媒体播放支持

 author    : scumyang@gmail.com
 create    : 2014-12-01
 update    :
 2015-01-08: 已兼容平台: PC(HTML5), EVM, iPanel, Inspur

 *******************************************************************************/

/*
 #
 # DEMO
 #

 var $PLAT = CPlatform.newInstance();
 var $MP = CMediaPlayer.newInstance
 ({
 browser: $PLAT.browserType
 ,instanceName: '$MP'
 ,windowPos: G_APP_CONF.vod.videoWindowPos

 ,onEvent: function( E )
 {
 switch( E.msg )
 {
 case 'canplay':
 case 'PLAYER_BUFFERING_END':
 $MP.play();
 $LOG.log('节目时长: ' + $MP.duration());
 break;

 case 'PLAYER_FINISH': $MP.play(); break;

 default:
 if( $PLAT.browserType.toLowerCase() == 'chrome' ) break;
 $LOG.log( 'EVT: ' + E.msga + ', orgMsg=' + E.msgo );
 }
 }
 });

 */

//------------------------------------------------------------------------------
// CMediaPlayer 单体类
//------------------------------------------------------------------------------
var CMediaPlayer =
{
	// 获取新实现
	newInstance: function (P) {
		switch (P.browser.toLowerCase()) {
			case 'ipanel':
				return new CMediaPlayer_iPanel(P);
				break;
			case 'evm':
				return new CMediaPlayer_EVM(P);
				break;
			case 'rocme':
				return new CMediaPlayer_ROCME(P);
				break;
			case 'inspur':
				return new CMediaPlayer_Inspur(P);
				break;
			case 'idtv' :
				return new CMediaPlayer_IDTV(P);
				break;
			case 'evmandroidapk':
				return new CMediaPlayer_EVMAndroidApk(P);
				break;
			case 'safari':
			case 'firefox':
			case 'opera':
			case 'chrome':
				return new CMediaPlayer_Generic(P);
				break;
			default:
				$.log('Unsupported Platform: ' + P.browser);
		}
		return {}
	}

	// 获取媒体类型
	, getMediaType: function (M) {
		if (evm.isUndefined(M)) return '';
		var mt = ''
			, proto = ( M.match(/.*:\/\//) + '' ).replace('://', '').toLowerCase();
		switch (proto) {
			case 'ngod':
			case 'rtsp':
				mt = 'VOD';
				break;
			case 'c4cam_live':
			case 'https':
			case 'http':
				mt = 'HTTP';
				break;
			case 'delivery':
				mt = 'DVB';
				break;
			case 'udp':
				mt = "IP-UDP";
				break;
			case 'igmp':
				mediaType = 'LiveTV';
				break;
			case 'file':
				mt = 'FILE';
				break;
			default:
				mt = 'Unknown';
		}
		return mt;
	}
}
//------------------------------------------------------------------------------
// CMediaPlayer_Base
// 基类
//------------------------------------------------------------------------------
function CMediaPlayer_Base(P) {
	// 属性 --------------------------------------------------------------------
	this.className = 'CMediaPlayer_Base';
	this.CLSID = evm.makeGlobalId(this.className);
	this._windowMode = 'window';	// 视频显示模式
	//this._windowMode = 'fullscreen'; // 视频显示模式
	this._fsSize = {};		// 全屏模式尺寸
	this._wndSize = {};		// 窗口模式尺寸
	this._mpInst = null;	// 本地化 MediaPlayer 实例
	this._mpInstId = 0;		// 实例 ID (兼容 EVM)
	this._instName = null;	// JS 实例名 (String, 兼容 Inspur)
	this._status = 'N/A';	// 当前状态
	this._lastMRL = '';		// 最近一次 MRL
	this._mediaType = '';	// 媒体类型
	this._duration = 0;		// 媒体总时长
	this._npt = 0;			// 当前播放时间点
	this._playSpeed = 1;	// 播放速率
	this._muted = 0;		// 静音
	this._volume = 50;		// 当前音量
	this._volumeStep = 5;	// 音量调节的步长
	this._videoMode = '';	// 视频模式
	this._videoAspect = '';	// 显示比例
	this._visible = 1;		// 可见性
	this._transMode = 'IP';	// 传输方式 (IP,QAM,IPQAM)
	this.onEvent = new Function();		// 事件回调
	// 方法 ----------------------------------------------------------------------
	this.getInstance = function (P) {
		with (this) {
			try {
				if (!evm.isObject(P)) throw new Error('Need parameter P');
				__windowSize(P.windowSize
				|| {left: 0, top: 0, width: 480, height: 384});
				__fullscreenSize(P.fullscreenSize
				|| {
					left: 0, top: 0
					, width: document.body ? document.body.clientWidth : 1280
					, height: document.body ? document.body.clientHeight : 720
				});
			}
			catch (E) {
				//$.log(E);
			}
		}
	}
	// 获取/设置窗口模式下的视频窗口矩形
	this.__windowSize = function (M, F) {
		with (this) {
			if (evm.isUndefined(M)) return _wndSize;
			evm.isNumber(M.left) ? _wndSize.left = M.left : 0;
			evm.isNumber(M.top) ? _wndSize.top = M.top : 0;
			evm.isNumber(M.width) ? _wndSize.width = M.width : 0;
			evm.isNumber(M.height) ? _wndSize.height = M.height : 0;
			return this;
		}
	}
	// 获取/设置全屏模式下的视频窗口矩形
	this.__fullscreenSize = function (M, F) {
		with (this) {
			if (evm.isUndefined(M)) return _fsSize;
			evm.isNumber(M.left) ? _fsSize.left = M.left : 0;
			evm.isNumber(M.top) ? _fsSize.top = M.top : 0;
			evm.isNumber(M.width) ? _fsSize.width = M.width : 0;
			evm.isNumber(M.height) ? _fsSize.height = M.height : 0;
			return this;
		}
	}
	// 获取/设置视频窗口显示模式
	// @X: undefined: 时返回当前值
	//     Number: 视为联合后面 3 个参数指定窗口坐标和尺寸
	//     String: 视为模式名
	this.windowMode = function (X, Y, W, H) {
		with (this) {
			if (evm.isUndefined(X)) return _windowMode;
			if (evm.isNumber(X)) {
				__windowSize({left: X, top: Y, width: W, height: H});
				_windowMode = 'window';
			}
			else if (evm.isString(X)) {
				switch (X) {
					case 'fs':
					case 'fullscreen':
						_windowMode = 'fullscreen';
						break;
					case 'wnd':
					case 'window':
						_windowMode = 'window';
						break;
					case 'toggle':
						_windowMode = ( _windowMode == 'window' ? 'fullscreen' : 'window' );
						break
				}
			}
			// 调用子类实现
			_windowMode == 'fullscreen' ? __impl_setFullscreen() : __impl_setWindow();
			return this;
		}
	}
	// 获取本地化实例
	this.instance = function () {
		return this._mpInst;
	}
	// 获取本地化实例 ID
	this.instanceId = function () {
		return this._mpInstId;
	}
	// 获取实例变量名
	this.instanceName = function () {
		return this._instName;
	}
	// 获取/设置视频格式
	this.videoMode = function (M) {
	}
	// 获取/设置视频画面格式
	this.videoAspect = function (M) {
	}
	// 获取/设置视频窗口可见性
	this.visible = function (M, F) {
	}
	// 获取/设置传输方式
	this.transferMode = function (M) {
		with (this) {
			if (evm.isUndefined(M)) return _transMode;
			_transMode = M ? M.toUpperCase() : 'ip';
			return this;
		}
	}
	// 同步关键属性
	this.synchronize = function () {
	}
	// 获取当前状态
	// N/A-未初始化
	// Open:打开媒体, Ready-就绪, Play-播放, Pause-暂停, Stop-停止(已播放过),
	// Scale-倍速调整, Seek-定位中, Invalid-无效(实例已销毁)
	this.status = function () {
		return this._status;
	}
	// 获取当前状态
	this.lastMRL = function () {
		return _lastMRL;
	}
	// 获取当前媒体类型
	this.mediaType = function () {
		return this._mediaType;
	}
	// 获取媒体时长
	this.duration = function () {
		return this._duration;
	}
	// 获取当前播放时间点
	this.npt = function () {
		return this._npt;
	}
	// 获取/设置播放速率
	this.playSpeed = function (M) {
		with (this) {
			if (evm.isUndefined(M)) return _playSpeed;
			return __impl_playSpeed(M);
		}
	}
	// 获取/设置静音
	// @M: undefined: 返回当前音量, 1: 设置静音, 0: 取消静音
	this.mute = function (M) {
		with (this) {
			switch (M) {
				case 0:
					_muted = 0;
					break;
				case 1:
					_muted = 1;
					break;
				case 'toggle':
					_muted = 1 - _muted;
					break;
			}
			if (M) {
				return __impl_mute(_muted);
			}
			else {
				return __impl_mute();
			}
		}
	}
	// 获取/设置音量
	// @M: undefined: 返回当前音量
	//     '+','-': 增大/减小音量
	//     Number: 显式指定音量 (0~100)
	this.volume = function (M) {
		with (this) {
			if (evm.isUndefined(M)) return _volume;
			switch (M) {
				case '+':
					_volume += _volumeStep;
					break;
				case '-':
					_volume -= _volumeStep;
					break;
				default:
					_volume = M;
			}
			_volume > 100 ? _volume = 100 : 0;
			_volume < 0 ? _volume = 0 : 0;
			return __impl_volume(_volume);
		}
	}
	// 播放
	this.open = function (M) {
		if (!evm.isString(M)) return false;
		with (this) {
			_status = 'invoke open';
			var rv = __impl_open(M);
			_status = rv ? 'ready' : 'error';
			rv ? _lastMRL = M : 0;
			return rv;
		}
	}
	this.close = function () {
		with (this) {
			_status = 'invoke close';
			var rv = __impl_close();
			_status = rv ? 'closed' : 'error';
			return rv;
		}
	}
	this.play = function (M) {
		with (this) {
			var rv;
			if (M) {
				rv = open(M);
				if (!rv) return rv;
			}
			_status = 'invoke play';
			rv = __impl_play(M);
			_status = rv ? 'playing' : 'error';
			return rv;
		}
	}
	this.replay = function () {
		with (this) {
			var rv;
			if (_lastMRL) {
				stop();
				rv = open(_lastMRL);
				if (!rv) return rv;
			}
			_status = 'invoke replay';
			rv = __impl_play(_lastMRL);
			_status = rv ? 'playing' : 'error';
			return rv;
		}
	}
	this.stop = function (M) {
		with (this) {
			var rv = __impl_stop();
			_status = 'stop';
			return rv;
		}
	}
	this.pause = function (M) {
		with (this) {
			if (evm.isUndefined(M)) return _status == 'paused';
			var rv;
			switch (M) {
				case 0:
				{
					if (_status == 'paused') {
						rv = __impl_resume();
						_status = rv ? 'playing' : 'error';
					}
				}
					break;
				case 1:
				{
					if (_status == 'playing') {
						rv = __impl_pause();
						_status = rv ? 'paused' : 'error';
					}
				}
					break;
				case 'toggle':
				{
					if (_status == 'paused') rv = pause(0);
					else if (_status == 'playing') rv = pause(1);
					else rv = false;
				}
					break;
			}
			return rv;
		}
	}
	// 获取详细媒体信息
	this.mediaInfo = function () {
		return {MRL: _lastMRL, duration: _duration, progress: _npt, elapsed: _npt, speed: _playSpeed}
	}
	// 绑定播放器对象到指定的播放实例
	this.bindPlayerInstance = function (M) {
	}
	// 解绑播放器对象
	this.unbindPlayerInstance = function (M) {
	}
	// 释放播放实例资源
	this.releasePlayerInstance = function () {
	}
	// 获取基本流信息
	this.elementaryStreamInfo = function () {
	}
	// 获取系统事件对应的字符串
	this.__eventData = function (__E) {
		var D =
		{
			'0': {c: 0, t: 'Unknown Event', a: 'N/A'}

			// Chrome
			, 'loadstart': {c: 2401, t: '缓冲开始', a: '开始请求数据'}
			, 'progress': {c: 2401, t: '缓冲中', a: '正在请求数据'}
			, 'suspend': {c: 2401, t: 'suspend', a: '延迟下载'}
			, 'abort': {c: 2401, t: 'abort', a: '客户端主动终止下载'}
			, 'error': {c: 2401, t: 'error', a: '请求数据时遇到错误 '}
			, 'stalled': {c: 2401, t: 'stalled', a: '网速失速'}
			, 'play': {c: 2401, t: 'play', a: '尝试开始播放'}
			, 'pause': {c: 2401, t: 'pause', a: '暂停'}
			, 'loadedmetadata': {c: 2401, t: 'loadedmetadata', a: '成功获取资源长度'}
			, 'loadeddata': {c: 2401, t: 'loadeddata', a: '等待下一帧数据'}
			, 'waiting': {c: 2401, t: 'waiting', a: '等待数据'}
			, 'playing': {c: 2401, t: 'playing', a: '开始播放'}
			, 'canplay': {c: 2401, t: '缓冲完成', a: '可以播放, 部分加载'}
			, 'canplaythrough': {c: 2401, t: '缓冲完成', a: '可以播放, 全部加载'}
			, 'seeking': {c: 2401, t: 'seeking', a: '定位中'}
			, 'seeked': {c: 2401, t: 'seeked', a: '定位完成'}
			, 'timeupdate': {c: 2401, t: '时间已更新', a: '播放时间改变'}
			, 'ended': {c: 2401, t: '播放结束', a: '播放已结束'}
			, 'ratechange': {c: 2401, t: 'ratechange', a: '播放速率改变'}
			, 'durationchange': {c: 2401, t: 'durationchange', a: '资源长度改变'}
			, 'volumechange': {c: 2401, t: 'volumechange', a: '音量改变'}

			// Inspur
			, 'LOAD_FINISHED': {c: 2401, t: '页面加载完成', a: '页面加载完成'}
			, 'DVB_TUNER_UNLOCK': {c: 2011, t: '锁频失败', a: '信号线断开'}
			, 'DVB_TUNER_LOCKED': {c: 2012, t: '锁频成功', a: '信号线断开'}
			, 'EVENT_NETWORK_STATUS': {c: 2101, t: '网络状态', a: '网络状态'}
			, 'EVENT_NETWORK_STATUS_3': {c: 2103, t: '网络已断开', a: '网络已断开'}
			, 'EVENT_NETWORK_STATUS_2': {c: 2112, t: '网络已连接', a: '网络已连接'}
			, 'EVENT_RTSP_HEARTBEAT': {c: 8011, t: 'EVENT_RTSP_HEARTBEAT', a: 'RTSP 心跳'}
			, 'PLAYER_BUFFERING_START': {c: 8021, t: '缓冲开始', a: '开始缓冲'}
			, 'PLAYER_BUFFERING_PROGRESS': {c: 8022, t: '缓冲中', a: '缓冲中'}
			, 'PLAYER_BUFFERING_END': {c: 8023, t: '缓冲完成', a: '缓冲完成'}
			, 'PLAYER_ERROR': {c: 8101, t: '播放出错', a: '播放失败'}
			, 'PLAYER_FINISH': {c: 8061, t: '播放结束', a: '播放结束'}
			, 'PLAYER_PLAY_STOP': {c: 8081, t: '播放停止', a: '停止播放'}
			, 'STB_RUNNING_TIME_OVERFLOW': {c: 1011, t: 'STB运行超时', a: 'STB 超时'}
			, 'PLAYER_SEEK_END': {c: 8032, t: '时间已更新', a: 'SEEK 成功'}
		}
		var id = __E;
		D[id] ? 0 : id = '0';
		return {code: D[id].c, msg: D[id].t, msga: D[id].a, msgo: __E, instance: this}
	}
	// 子类应实现的方法 -------------------------------------------------------
	this.__impl_setFullscreen = function () {
	}
	this.__impl_setWindow = function () {
	}
	this.__impl_mute = function (N) {
	}
	this.__impl_volume = function (N) {
	}
	this.__impl_open = function (M) {
	}
	this.__impl_close = function () {
	}
	this.__impl_play = function (M) {
	}
	this.__impl_stop = function () {
	}
	this.__impl_pause = function (M) {
	}
	this.__impl_resume = function () {
	}
	this.__impl_playSpeed = function (N) {
	}
	this.getInstance(P);
}
//------------------------------------------------------------------------------
// CMediaPlayer_Generic
// 通用(HTML5) Generic 子类
//------------------------------------------------------------------------------
function CMediaPlayer_Generic(P) {
	// 继承
	CMediaPlayer_Base.call(this, P);
	with (this) {
		// 重载 ------------------------------------------------------------------
		className = 'CMediaPlayer_Generic';
		CLSID = evm.makeGlobalId(className);
		// 创建实例
		getInstance = function (P) {
			try {
				if (_mpInst == null) {
					_instName = P.instanceName || '';
					evm.isFunction(P.onEvent) ? onEvent = P.onEvent : 0;
					// 创建播放器对象
					_mpInst = evm.createElem('video', {
						id: CLSID,
						style: 'position:absolute;left:0px;top:0px'
					}, document.body);
					_mpInst.controls = P.constrols ? 'controls' : '';
					var I = this;
					evm.addEvent(_mpInst,'click',function(){
						I.pause('toggle');
					});
					evm.addEvent(_mpInst,'dblclick',function(){
						I.windowMode('toggle');
					});
					_volume = Math.floor(_mpInst.volume * 100);
					// 注册事件回调
					var evt = ['loadstart', 'progress', 'suspend', 'abort', 'error', 'stalled', 'play', 'pause', 'loadedmetadata', 'loadeddata', 'waiting', 'playing', 'canplay', 'canplaythrough', 'seeking', 'seeked', 'timeupdate', 'ended', 'ratechange', 'durationchange', 'volumechange']
					for (var i in evt) {
						(function (I) {
							_mpInst.addEventListener(evt[i], function (EVT) {
								var eo = __eventData(EVT.type);
								eo.event = EVT;
								I.onEvent(eo);
							})
						})(this);
					}
				}
				return _mpInst;
			}
			catch (E) {
				//$.log(E.stack)
			}
		}
		synchronize = function () {
			return getInstance().currentTime;
		}
		duration = function () {
			return getInstance().duration;
		}
		npt = function (M) {
			if (evm.isUndefined(M)) return getInstance().currentTime;
			getInstance().currentTime = M;
			return getInstance().currentTime;
		}
		mediaInfo = function () {
			var I = getInstance();
			return {
				MRL: _lastMRL,
				duration: I.duration,
				progress: I.currentTime,
				elapsed: I.currentTime - ( I.startTime || 0 ),
				speed: I.playbackRate
			}
		}
		// 子类方法 ----------------------------------------------------------------
		__impl_setFullscreen = function () {
			var elm = evm.$(getInstance());
			evm.css(elm,{left:_fsSize.left+'px'});
			evm.css(elm,{top:_fsSize.top+'px'});
			evm.css(elm,{width:_fsSize.width+'px'});
			evm.css(elm,{height:_fsSize.height+'px'});
		}
		__impl_setWindow = function () {
			var elm = evm.$(getInstance());
			evm.css(elm,{left:_wndSize.left+'px'});
			evm.css(elm,{top:_wndSize.top+'px'});
			evm.css(elm,{width:_wndSize.width+'px'});
			evm.css(elm,{height:_wndSize.height+'px'});
		}
		__impl_mute = function (N) {
			getInstance().muted = N;
			return getInstance().muted;
		}
		__impl_volume = function (N) {
			getInstance().volume = N / 100;
			_volume = Math.floor(getInstance().volume * 100);
			return _volume;
		}
		__impl_open = function (M) {
			getInstance().src = M;
			return true;
		}
		__impl_close = function () {
			getInstance().close();
			return true;
		}
		__impl_play = function (M) {
			getInstance().play();
			return true;
		}
		__impl_stop = function () {
			getInstance().pause();
			getInstance().src = '';
			return true;
		}
		__impl_pause = function () {
			getInstance().pause();
			return true;
		}
		__impl_resume = function () {
			getInstance().play();
			return true;
		}
		// 播放速率(0~1)
		__impl_playSpeed = function (N) {
			if (evm.isUndefined(N)) return getInstance().playbackRate;
			getInstance().playbackRate = ( N < 0 ? -1 / ( N / 2 ) : N > 1 ? N / 4 : 1 );
			return this;
		}
		getInstance(P);
	}
}
//------------------------------------------------------------------------------
// CMediaPlayer_iPanel
// iPanel 实现类
//------------------------------------------------------------------------------
/*
 function CMediaPlayer_iPanel( P )
 {
 // Properties:
 this.className = 'CMediaPlayer_iPanel';
 this.CLSID = $.makeGlobalId( this.className );
 }

 CMediaPlayer_iPanel.prototype = new CMediaPlayer_Base();
 CMediaPlayer_iPanel.prototype.extend = swat.extend;
 CMediaPlayer_iPanel.prototype.extend(
 {
 getInstance: function()
 {
 return media.AV;
 },

 open: function( __url )
 {
 try
 {
 with( this )
 {
 var mt = getMediaType( __url );

 VOD.changeServer( 'isma_v2', 'ip_ts' );
 //VOD.serviceStart();
 //VOD.server.mode = "IP";

 // mediaType: MPEG,MPG,DivX,MP3,AVI,VOD,TSTV,TVOD,LiveTV,UDP,DVB,localfile,PVR
 media.AV.open( __url, mt );
 }
 }
 catch( E )
 {
 }

 return this;
 },

 play: function()
 {
 try
 {
 media.AV.play();
 }
 catch( E )
 {
 }

 return this;
 },

 stop: function()
 {
 try
 {
 media.AV.stop();
 }
 catch( E )
 {
 }

 return this;
 },

 pause: function()
 {
 try
 {
 media.AV.pause();
 }
 catch( E )
 {
 }

 return this;
 },

 close: function()
 {
 try
 {
 media.AV.stop();
 media.AV.close();
 DVB.stopAV();
 }
 catch( E )
 {
 }

 return this;
 },

 setWindowPosition: function( __x, __y, __w, __h )
 {
 try
 {
 media.video.setPosition( __x, __y, __w, __h );
 }
 catch( E )
 {
 }

 return this;
 },

 fullScreen: function()
 {
 try
 {
 media.video.fullScreen();
 }
 catch( E )
 {
 }

 return this;
 },

 setVolume: function( A )
 {
 try
 {
 // var o = iPanel.pageWidgets.getByName('volume');
 // o.init();
 // o.show();
 switch( A )
 {
 case '+': media.sound.value += 5; break;
 case '-': media.sound.value -= 5; break;
 default: media.sound.value = A;
 }
 }
 catch( E ){}

 return this;
 },

 scale: function( A, N )
 {
 try
 {
 if( A > 0 ) media.AV.forward( A );
 else media.AV.backward( A );
 }
 catch( E )
 {
 }

 return this;
 },

 seek: function( N )
 {
 try
 {
 media.AV.seek( N );
 }
 catch( E )
 {
 }

 return this;
 },

 getStatus: function()
 {
 try
 {
 var s = '';
 switch( media.AV.status )
 {
 case 'play': s = '播放中'; break;
 case 'pause': s = '暂停'; break;
 case 'forward': s = '快进'; break;
 case 'backward': s = '快退'; break;
 case 'repeat': s = '重播'; break;
 case 'slow': s = '慢放'; break;
 case 'stop': s = '停止'; break;
 }
 }
 catch( E )
 {
 }
 },

 getServerList: function()
 {
 try
 {
 return VOD.serverList;
 }
 catch( E )
 {
 }

 return null;
 },

 getMediaInfo: function()
 {
 try
 {
 return {duration: media.AV.duration, progress: media.AV.progress, elapsed: media.AV.elapsed, speed: media.AV.speed}
 }
 catch( E )
 {
 return {duration:0, progress:0, elapsed:0, speed:0}
 }
 },

 strerror: function( __errcode )
 {
 var s = '';
 switch( __errcode )
 {
 case 5974: s = '页面加载完成'; break;
 case 5200: s = '请求时移列表'; break;
 case 5220: s = 'VOD 模块启动成功'; break;
 case 5225: s = 'User Exception'; break;
 case 5228: s = '正在连接服务器...'; break;
 case 5202: s = '缓冲完成'; break;				// EIS_VOD_PREPAREPLAY_SUCCESS
 case 5203: s = '连接服务器失败'; break;
 case 5205: s = '媒体播放成功'; break;			// EIS_VOD_PLAY_SUCCESS
 case 5206: s = '媒体播放失败'; break;
 case 5210: s = '播放结束'; break;
 case 5228: s = '停止播放成功'; break;
 case 5351: s = '解码器关闭'; break;				// CA_MESSAGE_CLOSE
 case 8002: break;
 default:;
 }

 return s;
 }
 });
 */
//------------------------------------------------------------------------------
// CMediaPlayer_EVM
// EVM 实现类
//------------------------------------------------------------------------------
/*
 function CMediaPlayer_EVM( P )
 {
 // Properties:
 this.className = 'CMediaPlayer_EVM';
 this.CLSID = $.makeGlobalId( this.className );

 // 初始化实例
 if( P.windowPos ) this.windowPos( P.windowPos );
 if( P.fullscreenPos ) this.fullscreenPos( P.fullscreenPos );

 this.getInstance( P );

 // 注册系统事件
 var I = this;
 $(document).event
 (
 'systemevent',
 (
 function()
 {
 return function( E )
 {
 if( evm.isFunction( I.onEvent )) I.onEvent( I.eventObject( E ) );
 }
 }
 )()
 );
 }

 CMediaPlayer_EVM.prototype = new CMediaPlayer_Base();
 CMediaPlayer_EVM.prototype.extend = swat.extend;
 CMediaPlayer_EVM.prototype.extend(
 {
 mpName: 'Video'		// 播放器实例名
 ,mpZindex: 1		// 视频窗口 OSD 层次

 // 获取播放器实例
 ,getInstance: function()
 {
 with( this )
 {
 if( mpInst == null )
 {
 mpInst = new MediaPlayer();
 mpInstId = mpInst.createPlayerInstance( mpName, mpZindex );
 currVolume = mpInst.getVolume();
 }

 return mpInst;
 }
 }

 // 绑定播放器对象到指定的播放实例
 ,bindPlayerInstance: function( __id )
 {
 try
 {
 return this.getInstance().bindPlayerInstance( __id );
 }
 catch( E ){}
 }

 // 解绑播放器对象
 ,unbindPlayerInstance: function()
 {
 try
 {
 return this.getInstance().unBindPlayerInstance();
 }
 catch( E ){}
 }

 // 释放播放实例资源
 ,releasePlayerInstance: function()
 {
 try
 {
 return this.getInstance().releasePlayerInstance();
 }
 catch( E ){}
 }

 // 设置视频格式
 // @__A:
 //	1:PAL, 2:NTSC, 3:SECAM, 4:(默认)Auto
 ,videoMode: function( __A )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A )) return getInstance().videoMode;

 getInstance().videoMode = __A;
 }
 }
 catch( E ){}

 return this;
 }

 // 获取/设置视频画面格式
 // @__A:
 //	1-16:9, 2-4:3 Combined, 3-4:3 Pan-Scan, 4-4:3 Letter-Box, 5-全屏, 6-(默认)Auto
 ,videoAspect: function( __A )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A )) return getInstance().videoAspect;

 getInstance().videoAspect = __A;
 }
 }
 catch( E ){}

 return this;
 }

 // 获取/设置视频窗口可见性
 // @__A:
 //	1:可见, 0:不可见
 ,visible: function( __A, __F )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A )) return getInstance().visible;

 getInstance().visible = __A == 1 ? 1 : 0;
 if( !__F ) getInstance().refresh();
 }
 }
 catch( E ){}

 return this;
 }

 // 获取/设置视频窗口位置
 ,windowPos: function( __x, __y, __w, __h, __F )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __h )) return { left:left, top:top, width:width, height:height }

 left = __x;
 top = __y;
 width = __w;
 height = __h;
 getInstance().position = isFullscreen + ', ' + left + ', ' + top + ', ' + width + ', ' + height;

 if( __F ) getInstance().refresh();
 }
 }
 catch( E ){}

 return this;
 }

 // 获取/设置全屏播放
 // @__A:
 //	1:全屏, 0:取消全屏
 ,fullscreen: function( __A, __F )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A )) return isFullscreen;

 isFullscreen = __A;
 getInstance().position = __A == 1 ? '1, 0, 0, 0, 0' : ( '0, ' + left + ', ' + top + ', ' + width + ', ' + height );

 if( !__F ) getInstance().refresh();
 }
 }
 catch( E ){}

 return this;
 }

 // 获取/设置视频播放速度(数值型)
 // 1:(默认)前进, -1:后退
 // 2|4|8|16|32:快进, -2|-4|-8|-16|-32:快退
 // 1/2|1/4|8/1|1/16:慢进, -1/2|-1/4|-8/1|-1/16:慢退
 ,playScale: function( __A, __F )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A ))
 {
 currScale = getInstance().pace;
 return currScale;
 }

 getInstance().pace = __A;

 if( !__F ) getInstance().refresh();
 currScale = getInstance().pace;
 }
 }
 catch( E ){}

 return this;
 }

 // 获取/设置当前播放时间点(秒)
 ,playPoint: function( __A, __F )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A )) return getInstance().currentPoint;

 getInstance().point = __A;
 if( !__F ) getInstance().refresh();
 }
 }
 catch( E ){}

 return this;
 }

 // 播放定位
 ,seek: function( __A, __F )
 {
 this.currStatus = 'Seek';
 return this.playPoint( __A, __F );
 }

 // 获取媒体时长
 // 返回格式: "hh:mm:ss"
 ,duration: function()
 {
 try
 {
 return this.getInstance().getMediaDuration;
 }
 catch( E ){}
 }

 // 获取/设置音量
 // @__A:
 //	空值, 返回当前音量
 //	0~31, 设置当前音量
 ,volume: function( __A )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A ))
 {
 currVolume = getInstance().getVolume();
 return currVolume;
 }

 switch( __A )
 {
 case '+': currVolume += 5; break;
 case '-': currVolume -= 5; break;
 default: currVolume = __A;
 }
 }

 getInstance().setVolume( currVolume );
 currVolume = getInstance().getVolume();
 }
 catch( E ){}

 return this;
 }

 // 获取/设置静音状态(只影响当前播放实例)
 // @__A:
 //	空值, 返回当前音量
 //	0:设置静音, 1:取消静音
 ,mute: function( __A )
 {
 try
 {
 with( this )
 {
 if( evm.isUndefined( __A )) return this.getInstance().getMute() == 1 ? true : false;

 __A == 1 ? getInstance().audioMute() : getInstance().audioUnmute();
 }
 }
 catch( E ){}

 return this;
 }

 //取基本流信息
 ,getElementaryStreamInfo: function()
 {
 try
 {
 return this.getInstance().EleStreams;
 }
 catch( E ){}
 }

 // 开启媒体
 ,open: function( __url )
 {
 try
 {
 with( this )
 {
 mediaType = getMediaType( __url );
 getInstance().source = __url;
 currStatus = 'Open';
 }
 }
 catch( E ){}

 return this;
 }

 // 播放
 ,play: function()
 {
 try
 {
 with( this )
 {
 currStatus = 'Play';
 return getInstance().play();
 }
 }
 catch( E ){}
 }

 // 停止
 ,stop: function()
 {
 try
 {
 with( this )
 {
 currStatus = 'Stop';
 return getInstance().stop();
 }
 }
 catch( E ){}
 }

 // 暂停播放
 // 1: 显示最后帧, 0: 黑场
 ,pause: function( __A )
 {
 try
 {
 with( this )
 {
 currStatus = 'Pause';
 return getInstance().pause( __A == 1 ? 1 : 0 );
 }
 }
 catch( E ){}

 return this;
 }

 // 停止播放
 ,close: function()
 {
 try
 {
 with( this )
 {
 currStatus = 'Invalid';
 return releasePlayerInstance();
 }
 }
 catch( E ){}
 }

 // 获取播放状态
 ,status: function()
 {
 try
 {
 return this.currStatus;
 }
 catch( E ){}
 }

 // 获取播放信息
 ,getPlayInfo: function()
 {
 try
 {
 return '';
 //return {duration: media.AV.duration, progress: media.AV.progress, elapsed: media.AV.elapsed, speed: media.AV.speed}
 }
 catch( E )
 {
 return {duration:0, progress:0, elapsed:0, speed:0}
 }
 }

 // 获取系统事件对应的字符串
 ,eventObject: function( __E )
 {
 var D =
 {
 '0': {c:0,		t:'Unknown Event'				,a:'N/A'}
 ,'10901': {c:10901, t:'MEDIA_PLAY_END'				,a:'END 结束播放'}
 ,'10902': {c:10902, t:'MEDIA_PLAY_START'			,a:'START 开始播放'}
 ,'10903': {c:10903, t:'MEDIA_SM_S1_SETUP'			,a:'S1.SETUP'}
 ,'10904': {c:10904, t:'MEDIA_SM_S1_TEARDOWN'		,a:'S1.TEARDOWN'}
 ,'10905': {c:10905, t:'MEDIA_SM_S1_ANNOUNCE'		,a:'S1.ANNOUNCE'}
 ,'10906': {c:10906, t:'MEDIA_SM_S1_PING'			,a:'S1.PING'}
 ,'10907': {c:10907, t:'MEDIA_VSS_C1_PLAY'			,a:'C1.PLAY 播放'}
 ,'10908': {c:10908, t:'MEDIA_VSS_C1_PAUSE'			,a:'C1.PAUSE 暂停'}
 ,'10909': {c:10909, t:'MEDIA_VSS_C1_ANNOUNCE'		,a:'C1.ANNOUNCE 通知'}
 ,'10910': {c:10910, t:'MEDIA_VSS_C1_GETPARAMETER'	,a:'C1.GETPARAMETER 获取参数'}
 ,'10911': {c:10911, t:'MEDIA_VSS_C1_SETPARAMETER'	,a:'C1.SETPARAMETER 设置参数'}
 ,'10912': {c:10912, t:'MEDIA_SM_SETUP_TIMEOUT'		,a:'SM.SETUP 超时'}
 ,'10913': {c:10913, t:'MEDIA_SM_TEARDOWN_TIMEOUT'	,a:'SM.TEARDOWN 超时'}
 ,'10914': {c:10914, t:'MEDIA_SM_PING_TIMEOUT'		,a:'SM.PING 超时'}
 ,'10915': {c:10915, t:'MEDIA_VSS_PLAY_TIMEOUT'		,a:'C1.PLAY 超时'}
 ,'10916': {c:10916, t:'MEDIA_VSS_PAUSE_TIMEOUT'		,a:'C1.PAUSE 超时'}
 ,'10917': {c:10917, t:'MEDIA_VSS_GETP_TIMEOUT'		,a:'C1.GETPARAMETER 超时'}
 ,'10918': {c:10918, t:'MEDIA_VSS_SETP_TIMEOUT'		,a:'C1.SETPARAMETER 超时'}
 ,'10919': {c:10919, t:'MEDIA_NGOD_ES_TIMEOUT'		,a:'NGOD.ES 超时'}
 ,'10920': {c:10920, t:'MEDIA_NGOD_REES_TIMEOUT'		,a:'NGOD.REES 超时'}
 ,'10921': {c:10921, t:'MEDIA_NGOD_PMT_NOTFOUND'		,a:'NGOD.PMT 未找到'}
 }

 var id = __E.which;
 D[id] ? 0 : id = 0;

 return {code:D[id].c, msg:D[id].t, msga:D[id].a, instance:this}
 }
 });

 */
//------------------------------------------------------------------------------
// CMediaPlayer_Inspur
// Inspur 子类
//------------------------------------------------------------------------------
/*function CMediaPlayer_Inspur(P) {
	// 继承
	CMediaPlayer_Base.call(this, P);
	with (this) {
		// 重载 ------------------------------------------------------------------
		className = 'CMediaPlayer_Inspur';
		CLSID = $.makeGlobalId(className);
		// 创建实例
		getInstance = function (P) {
			try {
				with (this) {
					if (_mpInst == null) {
						_instName = P.instanceName || '$MP';
						evm.isFunction(P.onEvent) ? onEvent = P.onEvent : 0;
						_mpInst = iSTB.player;
						_volume = Math.floor(_mpInst.get_volume() * _volumeFactor);
						// 注册系统回调
						iSTB.evt.set_event_callback(_instName + '.__eventCallback');
					}
					return _mpInst;
				}
			}
			catch (E) {
				$LOG.log('getInstance: ' + E.message)
			}
		}
		synchronize = function (T) {
			try {
				switch (T) {
					case 'npt':
						_npt += _playSpeed;
						return _npt;
						break;
					case 'sync':
						_npt = getInstance().get_position();
						return _npt;
						break;
				}
				return this;
			}
			catch (E) {
			}
		}
		duration = function () {
			return getInstance().duration;
		}
		npt = function (M) {
			if (evm.isUndefined(M)) return _npt;
			getInstance().seek(M);
			// 中间件无法实时返回位置, 延迟模拟事件
			var I = this;
			$.delay(function () {
					I._npt = I.getInstance().get_position();
					I.onEvent(I.__eventData('PLAYER_SEEK_END'));
				}
				, 1000);
			return _npt;
		}
		mediaInfo = function () {
			return {MRL: _lastMRL, duration: _duration, progress: _npt, elapsed: _npt, speed: _playSpeed}
		}
		// 子类方法 ----------------------------------------------------------------
		__impl_setFullscreen = function () {
			getInstance().set_video_window(_fsSize.left, _fsSize.top, _fsSize.width, _fsSize.height);
		}
		__impl_setWindow = function () {
			getInstance().set_video_window(_wndSize.left, _wndSize.top, _wndSize.width, _wndSize.height);
		}
		__impl_mute = function (N) {
			N ? getInstance().mute() : getInstance().unmute();
			return N;
		}
		__impl_volume = function (N) {
			getInstance().set_volume(Math.floor(_volume / _volumeFactor));
			return _volume;
		}
		this.__impl_open = function (M) {
			try {
				return true;
			}
			catch (E) {
			}
		}
		__impl_close = function () {
			try {
				getInstance().stop();
				return true;
			}
			catch (E) {
			}
		}
		__impl_play = function (M) {
			try {
				if (!M) return true;
				_transMode == 'IPQAM' ? getInstance().startVodPlayer(M) : getInstance().play(M);
				return true;
			}
			catch (E) {
			}
		}
		__impl_stop = function () {
			try {
				getInstance().stop();
				return true;
			}
			catch (E) {
			}
		}
		__impl_pause = function () {
			try {
				getInstance().pause();
				return true;
			}
			catch (E) {
			}
		}
		__impl_resume = function () {
			try {
				getInstance().resume();
				return true;
			}
			catch (E) {
			}
		}
		__impl_playSpeed = function (M) {
			try {
				M < 0 ? getInstance().rr(-M) : getInstance().ff(M);
				_playSpeed = M;
				return _playSpeed;
			}
			catch (E) {
				$.log('__impl_playSpeed: ' + E.message)
			}
		}
		// 子类扩展 ----------------------------------------------------------------
		// 系统事件回调
		// @T: 类型
		// @EN: 事件名
		// @ET: 事件类型
		// @EV: 事件值
		this.__eventCallback = function (T, EN, ET, EV) {
			// 分值处理
			switch (EN) {
				case 'DVB_TUNER_UNLOCK':
					break;

				// 网络状态: 3:断开, 2:连接
				case 'EVENT_NETWORK_STATUS':
				{
					if (EV == 3) EN = 'EVENT_NETWORK_STATUS_3';
					else if (EV == 2) EN = 'EVENT_NETWORK_STATUS_2';
				}
					break;
				default:
				//if( $LOG && $LOG.log ) $LOG.log( '消息: T=' + T + ', EN=' + EN + ', ET=' + ET + ', EV=' + EV );
			}
			return onEvent(__eventData(EN));
		}
		this._volumeFactor = 100 / 32;
		var t = getInstance(P);
	}
}*/
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// CMediaPlayer_ROCME
// ROCME 实现类
// 适用终端中间件规范 GCABLE-2011-001-D1.0-111012
//------------------------------------------------------------------------------
/*function CMediaPlayer_ROCME(P) {
	// 继承
	CMediaPlayer_Base.call(this, P);
	with (this) {
		// 重载 ------------------------------------------------------------------
		className = 'CMediaPlayer_ROCME';
		CLSID = $.makeGlobalId(className);
		// 创建实例
		getInstance = function (P) {
			try {
				with (this) {
					if (_mpInst == null) {
						_instName = P.instanceName || '$MP';
						evm.isFunction(P.onEvent) ? onEvent = P.onEvent : 0;
						_mpInst = new MediaPlayer();
						_mpInstID = _mpInst.createPlayerInstance("video", 2);
						_volume = Math.floor(get_volume() * _volumeFactor);
						// 注册系统回调
						var I = this;
						document.onsystemevent = function (event) {
							var type = '';
							switch (event.which) {
								case 10934:
									type = 'PLAYER_BUFFERING_END';
									break;
								case 10936:
									type = 'PLAYER_FINISH';
									break;
								default:
									break;
							}
							var eo = __eventData(type);
							eo.event = event;
							I.onEvent(eo);
						}
						*//*for( var i in evt )
						 {
						 (function(I)
						 {
						 _mpInst.addEventListener( evt[i], function( EVT )
						 {
						 var eo = __eventData( EVT.type );
						 eo.event = EVT;
						 I.onEvent( eo );
						 })
						 }
						 )(this);
						 }*//*
					}
					return _mpInst;
				}
			}
			catch (E) {
				$LOG.log('getInstance: ' + E.message)
			}
		}
		get_volume = function () {
			return parseInt(DataAccess.getInfo("MediaSetting", "OutputVolumn"));
		}
		set_volume = function (v) {
			return DataAccess.setInfo("MediaSetting", "OutputVolumn", v);
		}
		//这个方法作用是什么
		synchronize = function (T) {
			try {
				switch (T) {
					case 'npt':
						_npt += _playSpeed;
						return _npt;
						break;
					case 'sync':
						_npt = getInstance().get_position();
						return _npt;
						break;
				}
				return this;
			}
			catch (E) {
			}
		}
		duration = function () {
			return getInstance().getMediaDuration();
		}
		//获取/设置当前播放的时间点
		npt = function (M) {
			if (evm.isUndefined(M)) return _npt;
			getInstance().point = M;
			//getInstance().refresh();
			getInstance().play();
			// 中间件无法实时返回位置, 延迟模拟事件
			var I = this;
			$.delay(function () {
					I._npt = I.getInstance().currentPoint;
					I.onEvent(I.__eventData('PLAYER_SEEK_END'));
				}
				, 100);
			return _npt;
		}
		mediaInfo = function () {
			return {MRL: _lastMRL, duration: _duration, progress: _npt, elapsed: _npt, speed: _playSpeed}
		}
		// 获取系统事件对应的字符串
		this.__eventData = function (__E) {
			var D =
			{
				'PLAYER_BUFFERING_END': {c: 10934, t: '缓冲完成', a: '缓冲完成'}
				//,'LOAD_FINISHED':               {c:2401, t:'页面加载完成'     ,a:'页面加载完成'}
				//,'DVB_TUNER_UNLOCK':            {c:2011, t:'锁频失败'   ,a:'信号线断开'}
				//,'DVB_TUNER_LOCKED':            {c:2012, t:'锁频成功'   ,a:'信号线断开'}
				//,'EVENT_NETWORK_STATUS':        {c:2101, t:'网络状态'   ,a:'网络状态'}
				//,'EVENT_NETWORK_STATUS_3':      {c:2103, t:'网络已断开'  ,a:'网络已断开'}
				//,'EVENT_NETWORK_STATUS_2':      {c:2112, t:'网络已连接'  ,a:'网络已连接'}
				//,'EVENT_RTSP_HEARTBEAT':        {c:8011, t:'EVENT_RTSP_HEARTBEAT'   ,a:'RTSP 心跳'}
				//,'PLAYER_BUFFERING_START':      {c:8021, t:'缓冲开始'   ,a:'开始缓冲'}
				//,'PLAYER_BUFFERING_PROGRESS':   {c:8022, t:'缓冲中'        ,a:'缓冲中'}
				//,'PLAYER_ERROR':                {c:8101, t:'播放出错'   ,a:'播放失败'}
				, 'PLAYER_FINISH': {c: 10936, t: '播放结束', a: '播放结束'}
				//,'PLAYER_PLAY_STOP':            {c:8081, t:'播放停止'   ,a:'停止播放'}
				//,'STB_RUNNING_TIME_OVERFLOW':   {c:1011, t:'STB运行超时'    ,a:'STB 超时'}
				//,'PLAYER_SEEK_END':             {c:8032, t:'时间已更新'  ,a:'SEEK 成功'}
			}
			var id = __E;
			D[id] ? 0 : id = '0';
			return {code: D[id].c, msg: D[id].t, msga: D[id].a, msgo: __E, instance: this}
		}
		// 子类方法 ----------------------------------------------------------------
		__impl_setFullscreen = function () {
			getInstance().position = '1,0,0,0,0';
			getInstance().refresh();
		}
		__impl_setWindow = function () {
			getInstance().position = "0," + _wndSize.left + "," + _wndSize.top + "," + _wndSize.width + "," + _wndSize.height;
			getInstance().refresh();
		}
		__impl_mute = function (N) {
			if (N === undefined) {
				return getInstance().getMute();
			}
			else {
				N ? getInstance().audioMute() : 0;
				return 1;
			}
		}
		__impl_volume = function (N) {
			//取消静音状态
			if (getInstance().getMute()) {
				getInstance().audioUnmute();
			}
			set_volume(Math.floor(_volume / _volumeFactor));
			return _volume;
		}
		this.__impl_open = function (M) {
			try {
				return true;
			}
			catch (E) {
			}
		}
		__impl_close = function () {
			try {
				getInstance().stop();
				return true;
			}
			catch (E) {
			}
		}
		__impl_play = function (M) {
			//M -> 文件地址
			try {
				if (!M) return true;
				if (_transMode == 'IPQAM') {
					getInstance().startVodPlayer(M)
				} else if (_transMode == 'IP') {
					getInstance().source = M;
					getInstance().play();
				}
				return true;
			}
			catch (E) {
			}
		}
		__impl_stop = function () {
			try {
				getInstance().pause(0);
				getInstance().releasePlayerInstance();
				_mpInst = null;
				return true;
			}
			catch (E) {
			}
		}
		__impl_pause = function () {
			try {
				getInstance().pause(0);
				return true;
			}
			catch (E) {
			}
		}
		__impl_resume = function () {
			try {
				getInstance().play();
				return true;
			}
			catch (E) {
			}
		}
		// 获取/设置视频播放速度(数值型)
		// 1:(默认)前进, -1:后退
		// 2|4|8|16|32:快进, -2|-4|-8|-16|-32:快退
		// 1/2|1/4|8/1|1/16:慢进, -1/2|-1/4|-8/1|-1/16:慢退
		__impl_playSpeed = function (M) {
			try {
				getInstance().pace = M;
				getInstance().refresh();
				_playSpeed = M;
				return _playSpeed;
			}
			catch (E) {
				$.log('__impl_playSpeed: ' + E.message)
			}
		}
		// 子类扩展 ----------------------------------------------------------------
		// 系统事件回调
		// @T: 类型
		// @EN: 事件名
		// @ET: 事件类型
		// @EV: 事件值
		this.__eventCallback = function (T, EN, ET, EV) {
			// 分值处理
			switch (EN) {
				case 'DVB_TUNER_UNLOCK':
					break;

				// 网络状态: 3:断开, 2:连接
				case 'EVENT_NETWORK_STATUS':
				{
					if (EV == 3) EN = 'EVENT_NETWORK_STATUS_3';
					else if (EV == 2) EN = 'EVENT_NETWORK_STATUS_2';
				}
					break;
				default:
				//if( $LOG && $LOG.log ) $LOG.log( '消息: T=' + T + ', EN=' + EN + ', ET=' + ET + ', EV=' + EV );
			}
			return onEvent(__eventData(EN));
		}
		this._volumeFactor = 100 / 32;
		var t = getInstance(P);
	}
}*/
//------------------------------------------------------------------------------
// CMediaPlayer_IDTV
// IDTV 实现类
// iDTV EPG与STB接口规范V2.0
//------------------------------------------------------------------------------
/*function CMediaPlayer_IDTV(P) {
	// 继承
	CMediaPlayer_Base.call(this, P);
	with (this) {
		// 重载 ------------------------------------------------------------------
		className = 'CMediaPlayer_IDTV';
		CLSID = $.makeGlobalId(className);
		// 创建实例
		getInstance = function (P) {
			try {
				with (this) {
					if (_mpInst == null) {
						$LOG.log('CMediaPlayer_IDTV:getInstance:' + P.instanceName);
						_instName = P.instanceName || '$MP';
						evm.isFunction(P.onEvent) ? onEvent = P.onEvent : 0;
						_mpInst = new MediaPlayer();
						_mpInstID = _mpInst.getNativePlayerInstanceID();
						_volume = Math.floor(_mpInst.getVolume() * _volumeFactor);
						$LOG.log('CMediaPlayer_IDTV:getInstance complete');
						*//*

						 // 注册系统回调
						 var I = this;
						 document.onsystemevent = function(event){
						 var type = '';
						 switch(event.which){
						 case 10934:
						 type = 'PLAYER_BUFFERING_END';
						 break;
						 case 10936:
						 type = 'PLAYER_FINISH';
						 break;
						 default:break;
						 }
						 var eo = __eventData(type);
						 eo.event = event;
						 I.onEvent(eo);
						 }*//*
					}
					return _mpInst;
				}
			}
			catch (E) {
				$LOG.log('getInstance: ' + E.message)
			}
		}
		//这个方法作用是什么
		synchronize = function (T) {
			try {
				switch (T) {
					case 'npt':
						_npt += _playSpeed;
						return _npt;
						break;
					case 'sync':
						_npt = getInstance().get_position();
						return _npt;
						break;
				}
				return this;
			}
			catch (E) {
			}
		}
		duration = function () {
			return getInstance().getMediaDuration();
		}
		//获取/设置当前播放的时间点
		npt = function (M) {
			if (evm.isUndefined(M)) return _npt;
			getInstance().point = M;
			getInstance().play();
			// 中间件无法实时返回位置, 延迟模拟事件
			var I = this;
			$.delay(function () {
					I._npt = I.getInstance().getCurrentPlayTime();
					I.onEvent(I.__eventData('PLAYER_SEEK_END'));
				}
				, 100);
			return _npt;
		}
		mediaInfo = function () {
			return {MRL: _lastMRL, duration: _duration, progress: _npt, elapsed: _npt, speed: _playSpeed}
		}
		// 获取系统事件对应的字符串
		this.__eventData = function (__E) {
			var D =
			{
				'PLAYER_BUFFERING_END': {c: 10934, t: '缓冲完成', a: '缓冲完成'}
				//,'LOAD_FINISHED':               {c:2401, t:'页面加载完成'     ,a:'页面加载完成'}
				//,'DVB_TUNER_UNLOCK':            {c:2011, t:'锁频失败'   ,a:'信号线断开'}
				//,'DVB_TUNER_LOCKED':            {c:2012, t:'锁频成功'   ,a:'信号线断开'}
				//,'EVENT_NETWORK_STATUS':        {c:2101, t:'网络状态'   ,a:'网络状态'}
				//,'EVENT_NETWORK_STATUS_3':      {c:2103, t:'网络已断开'  ,a:'网络已断开'}
				//,'EVENT_NETWORK_STATUS_2':      {c:2112, t:'网络已连接'  ,a:'网络已连接'}
				//,'EVENT_RTSP_HEARTBEAT':        {c:8011, t:'EVENT_RTSP_HEARTBEAT'   ,a:'RTSP 心跳'}
				//,'PLAYER_BUFFERING_START':      {c:8021, t:'缓冲开始'   ,a:'开始缓冲'}
				//,'PLAYER_BUFFERING_PROGRESS':   {c:8022, t:'缓冲中'        ,a:'缓冲中'}
				//,'PLAYER_ERROR':                {c:8101, t:'播放出错'   ,a:'播放失败'}
				, 'PLAYER_FINISH': {c: 10936, t: '播放结束', a: '播放结束'}
				//,'PLAYER_PLAY_STOP':            {c:8081, t:'播放停止'   ,a:'停止播放'}
				//,'STB_RUNNING_TIME_OVERFLOW':   {c:1011, t:'STB运行超时'    ,a:'STB 超时'}
				//,'PLAYER_SEEK_END':             {c:8032, t:'时间已更新'  ,a:'SEEK 成功'}
			}
			var id = __E;
			D[id] ? 0 : id = '0';
			return {code: D[id].c, msg: D[id].t, msga: D[id].a, msgo: __E, instance: this}
		}
		// 子类方法 ----------------------------------------------------------------
		__impl_setFullscreen = function () {
			getInstance().setVideoDisplayArea();	//待测试
			getInstance().refreshVideoDisplay();
		}
		__impl_setWindow = function () {
			getInstance().setVideoDisplayArea(_wndSize.left, _wndSize.top, _wndSize.width, _wndSize.height);
			getInstance().refreshVideoDisplay();
		}
		__impl_mute = function (N) {
			if (N === undefined) {
				return getInstance().getMuteFlag();
			}
			else {
				getInstance().setMuteFlag(N);
				return 1;
			}
		}
		__impl_volume = function (N) {
			//取消静音状态
			if (getInstance().getMuteFlag()) {
				getInstance().setMuteFlag(0);
			}
			getInstance().setVolume(Math.floor(_volume / _volumeFactor));
			return _volume;
		}
		this.__impl_open = function (M) {
			try {
				return true;
			}
			catch (E) {
			}
		}
		__impl_close = function () {
			try {
				getInstance().stop();
				return true;
			}
			catch (E) {
			}
		}
		__impl_play = function (M) {
			//M -> 文件地址
			try {
				if (!M) return true;
				$LOG.log('Media Play Source:' + M);
				//var media = '[{mediaURL:"'+M+'",mediaCode:"mediaCode",mediaType:2,audioType:1,videoType:2,streamType:3,drmType:1}]';
				*//*media.mediaURL = M;
				 media.mediaCode = 'mediaCode';
				 media.mediaType = 2;
				 media.audioType = 1;
				 media.videoType = 2;
				 media.streamType = 3;
				 media.drmType = 1;*//*
				var mediaUrl = '[{mediaURL:"' + M + '"}]';
				getInstance().setSingleMedia(mediaUrl);
				getInstance().setVideoDisplayMode(0);
				getInstance().setVideoDisplayArea(_wndSize.left, _wndSize.top, _wndSize.width, _wndSize.height);
				getInstance().refreshVideoDisplay();
				getInstance().playFromStart();
				$LOG.log('Media Play Preparation Complete');
				$LOG.log('Media Position Left:' + getInstance().getVideoDisplayLeft());
				$LOG.log('Media Position Top:' + getInstance().getVideoDisplayTop());
				$LOG.log('Media Position Width:' + getInstance().getVideoDisplayWidth());
				$LOG.log('Media Position Height:' + getInstance().getVideoDisplayHeight());
				return true;
			}
			catch (E) {
				$LOG.log('Media Play Error:' + E.message);
			}
		}
		__impl_stop = function () {
			try {
				getInstance().stop();
				return getInstance().releaseMediaPlayer(this._mpInstID) === 0 ? true : false;
			}
			catch (E) {
			}
		}
		__impl_pause = function () {
			try {
				getInstance().pause();
				return true;
			}
			catch (E) {
			}
		}
		__impl_resume = function () {
			try {
				getInstance().play();
				return true;
			}
			catch (E) {
			}
		}
		// 获取/设置视频播放速度(数值型)
		// 1:(默认)前进, -1:后退
		// 2|4|8|16|32:快进, -2|-4|-8|-16|-32:快退
		// 1/2|1/4|8/1|1/16:慢进, -1/2|-1/4|-8/1|-1/16:慢退
		__impl_playSpeed = function (M) {
			try {
				M < 0 ? getInstance().fastRewind(M) : getInstance().fastForward(M);
				_playSpeed = M;
				return _playSpeed;
			}
			catch (E) {
				$.log('__impl_playSpeed: ' + E.message)
			}
		}
		// 子类扩展 ----------------------------------------------------------------
		// 系统事件回调
		// @T: 类型
		// @EN: 事件名
		// @ET: 事件类型
		// @EV: 事件值
		this.__eventCallback = function (T, EN, ET, EV) {
			// 分值处理
			switch (EN) {
				case 'DVB_TUNER_UNLOCK':
					break;

				// 网络状态: 3:断开, 2:连接
				case 'EVENT_NETWORK_STATUS':
				{
					if (EV == 3) EN = 'EVENT_NETWORK_STATUS_3';
					else if (EV == 2) EN = 'EVENT_NETWORK_STATUS_2';
				}
					break;
				default:
				//if( $LOG && $LOG.log ) $LOG.log( '消息: T=' + T + ', EN=' + EN + ', ET=' + ET + ', EV=' + EV );
			}
			return onEvent(__eventData(EN));
		}
		this._volumeFactor = 100 / 100;
		var t = getInstance(P);
	}
}*/
//------------------------------------------------------------------------------
// CMediaPlayer_EVMAndroidApk
// Enrich 安卓播放器实现类
//------------------------------------------------------------------------------
function CMediaPlayer_EVMAndroidApk(P) {
	// 继承
	CMediaPlayer_Base.call(this, P);
	with (this) {
		// 重载 ------------------------------------------------------------------
		className = 'CMediaPlayer_EVMAndroidApk';
		CLSID = evm.makeGlobalId(className);
		// 创建实例
		getInstance = function (P) {
			try {
				with (this) {
					if (_mpInst == null) {
						_instName = P.instanceName || '$MP';
						evm.isFunction(P.onEvent) ? onEvent = P.onEvent : 0;
						_mpInst = media;
					}
					return _mpInst;
				}
			}
			catch (E) {
				//$LOG.log('getInstance: ' + E.message)
			}
		}
		//这个方法作用是什么
		synchronize = function (T) {
			/*try
			 {
			 switch( T )
			 {
			 case 'npt': _npt += _playSpeed; return _npt; break;
			 case 'sync': _npt = getInstance().get_position(); return _npt; break;
			 }

			 return this;
			 }
			 catch( E ){}*/
		}
		duration = function () {
			/*return getInstance().getMediaDuration();*/
		}
		//获取/设置当前播放的时间点
		npt = function (M) {
			/*if( evm.isUndefined( M )) return _npt;

			 getInstance().point = M;
			 getInstance().play();

			 // 中间件无法实时返回位置, 延迟模拟事件
			 var I = this;
			 $.delay( function()
			 {
			 I._npt = I.getInstance().getCurrentPlayTime();
			 I.onEvent( I.__eventData( 'PLAYER_SEEK_END' ) );
			 }
			 , 100 );


			 return _npt;*/
		}
		mediaInfo = function () {
			/*return {MRL:_lastMRL, duration:_duration, progress:_npt, elapsed:_npt, speed:_playSpeed}*/
		}
		// 获取系统事件对应的字符串
		this.__eventData = function (__E) {
			/*var D =
			 {
			 'PLAYER_BUFFERING_END':        {c:10934, t:'缓冲完成'   ,a:'缓冲完成'}
			 //,'LOAD_FINISHED':               {c:2401, t:'页面加载完成'     ,a:'页面加载完成'}
			 //,'DVB_TUNER_UNLOCK':            {c:2011, t:'锁频失败'   ,a:'信号线断开'}
			 //,'DVB_TUNER_LOCKED':            {c:2012, t:'锁频成功'   ,a:'信号线断开'}
			 //,'EVENT_NETWORK_STATUS':        {c:2101, t:'网络状态'   ,a:'网络状态'}
			 //,'EVENT_NETWORK_STATUS_3':      {c:2103, t:'网络已断开'  ,a:'网络已断开'}
			 //,'EVENT_NETWORK_STATUS_2':      {c:2112, t:'网络已连接'  ,a:'网络已连接'}
			 //,'EVENT_RTSP_HEARTBEAT':        {c:8011, t:'EVENT_RTSP_HEARTBEAT'   ,a:'RTSP 心跳'}
			 //,'PLAYER_BUFFERING_START':      {c:8021, t:'缓冲开始'   ,a:'开始缓冲'}
			 //,'PLAYER_BUFFERING_PROGRESS':   {c:8022, t:'缓冲中'        ,a:'缓冲中'}
			 //,'PLAYER_ERROR':                {c:8101, t:'播放出错'   ,a:'播放失败'}
			 ,'PLAYER_FINISH':               {c:10936, t:'播放结束'   ,a:'播放结束'}
			 //,'PLAYER_PLAY_STOP':            {c:8081, t:'播放停止'   ,a:'停止播放'}
			 //,'STB_RUNNING_TIME_OVERFLOW':   {c:1011, t:'STB运行超时'    ,a:'STB 超时'}
			 //,'PLAYER_SEEK_END':             {c:8032, t:'时间已更新'  ,a:'SEEK 成功'}
			 }

			 var id = __E;
			 D[id] ? 0 : id = '0';

			 return {code:D[id].c, msg:D[id].t, msga:D[id].a, msgo:__E, instance:this}*/
		}
		// 子类方法 ----------------------------------------------------------------
		__impl_setFullscreen = function () {
			var temp = _wndSize;
			_wndSize = _fsSize;
			_fsSize = temp;
		}
		__impl_setWindow = function () {
			var temp = _fsSize;
			_fsSize = _wndSize;
			_wndSize = temp;
		}
		__impl_mute = function (N) {
			/*if(N === undefined){
			 return getInstance().getMuteFlag();
			 }
			 else{
			 getInstance().setMuteFlag(N);
			 return 1;
			 }*/
		}
		__impl_volume = function (N) {
			/*//取消静音状态
			 if(getInstance().getMuteFlag()){
			 getInstance().setMuteFlag(0);
			 }
			 getInstance().setVolume(Math.floor( _volume / _volumeFactor ));

			 return _volume;*/
		}
		this.__impl_open = function (M) {
			try {
				return true;
			}
			catch (E) {
			}
		}
		__impl_close = function () {
			try {
				getInstance().stop();
				return true;
			}
			catch (E) {
			}
		}
		__impl_play = function (M) {
			//M -> 文件地址
			try {
				if (!M) return true;
				getInstance().play(M, _wndSize.left, _wndSize.top, _wndSize.width, _wndSize.height);
				return true;
			}
			catch (E) {
				//$LOG.log('Media Play Error:' + E.message);
			}
		}
		__impl_stop = function () {
			try {
				getInstance().stop();
				//return getInstance().releaseMediaPlayer(this._mpInstID) === 0 ? true : false;
			}
			catch (E) {
			}
		}
		__impl_pause = function () {
			/*try
			 {
			 getInstance().pause();
			 return true;
			 }
			 catch( E ){}*/
		}
		__impl_resume = function () {
			/*try
			 {
			 getInstance().play();
			 return true;
			 }
			 catch( E ){}*/
		}
		// 获取/设置视频播放速度(数值型)
		// 1:(默认)前进, -1:后退
		// 2|4|8|16|32:快进, -2|-4|-8|-16|-32:快退
		// 1/2|1/4|8/1|1/16:慢进, -1/2|-1/4|-8/1|-1/16:慢退
		__impl_playSpeed = function (M) {
			/*try
			 {
			 M < 0 ? getInstance().fastRewind(M) : getInstance().fastForward(M);
			 _playSpeed = M;
			 return _playSpeed;
			 }
			 catch(E){ $.log( '__impl_playSpeed: ' + E.message ) }*/
		}
		// 子类扩展 ----------------------------------------------------------------
		// 系统事件回调
		// @T: 类型
		// @EN: 事件名
		// @ET: 事件类型
		// @EV: 事件值
		this.__eventCallback = function (T, EN, ET, EV) {
			/*// 分值处理
			 switch( EN )
			 {
			 case 'DVB_TUNER_UNLOCK': break;

			 // 网络状态: 3:断开, 2:连接
			 case 'EVENT_NETWORK_STATUS':
			 {
			 if( EV == 3 ) EN = 'EVENT_NETWORK_STATUS_3';
			 else if( EV == 2 ) EN = 'EVENT_NETWORK_STATUS_2';
			 } break;

			 default:
			 //if( $LOG && $LOG.log ) $LOG.log( '消息: T=' + T + ', EN=' + EN + ', ET=' + ET + ', EV=' + EV );
			 }

			 return onEvent( __eventData( EN ) );*/
		}
		//this._volumeFactor = 100 / 100;
		var t = getInstance(P);
	}
}