/**
 * @file 文章上下按键自动分页浏览
 * @import /swatapps/ape/evm.js
 * @des 组件接口参考杨涛云组件定义规范
 */

(function(){
    evm.ext.comp.ArticleScrollY = function(options){
        //强制使用new
        if(this === evm.ext.comp || this === window){
            return new arguments.callee(options);
        }
        //必须传参
        if(evm.isUndefined(options) && !(evm.isObject(options))){
            throw new Error('evm.ext.comp.ArticleScrollY实例化失败：没有传入参数');
        }
        function handler(options){
            // 触发事件的默认方法
            try{
                //console.log('VListBox 事件触发,默认处理方法');
            }catch(E){}
        }

        evm.ext.comp.BaseComponent.call(this,options);

        //====配置属性定义;=========================
        this.config = options;

        //组件名称
        //用于构建组件的className
        this._constrName = 'articleScrollY';
        //包含元素Id
        this._boxId = this.config.boxId || this.comp_name;
        //文章数据
        this.data = this.config.data;
        this._items = [];

        //分页属性
        this._pageIndex = 1;
        this._pageTotalNum = 1;

        //====自定义事件的回调方法,在创建对象时传入对应的回调方法;=========================
        this.onBlur = this.config.onBlur || handler;
        this.onChange = this.config.onChange || handler;
        this.onChangeOut = this.config.onChangeOut || handler;
        this.onFocus = this.config.onFocus || handler;
        this.onFilter = this.config.onFilter || handler;
        this.onSelect = this.config.onSelect || handler;
        this.onEvent  = this.config.onEvent || handler;
    };
    evm.extend(evm.ext.comp.ArticleScrollY.prototype,{
        action:function(dir,keyEvent){
            var evt = {
                dir:dir,
                type:'onChange'
            }
            switch(dir){
                case 'UP':
                    if(this._pageIndex - 1 >= 1){
                        this._pagePrev();
                    }
                    else{
                        this.onEvent(evt);
                    }
                    break;
                case 'DOWN':
                    if(this._pageIndex + 1 <= this._pageTotalNum){
                        this._pageNext();
                    }
                    else{
                        this.onEvent(evt);
                    }
                    break;
                case 'LEFT':
                    this.onEvent(evt);
                    this._onBlur();
                    break;
                case 'RIGHT':
                    this.onEvent(evt);
                    this._onBlur();
                    break;
                default:
                    this.onEvent(evt);
                    this._onFocus();
                    break;
            }
            return this;
        },
        init:function(remoteData){
            remoteData && (this.data = remoteData);

            this._items = this.data && this.data.split('<hr custompaging="true" />');
            this._pageTotalNum = this._items.length;

            if(this._pageTotalNum > 0){
                this._create();
            }
            return this;
        },
        _create:function(){
            var boxElm = document.getElementById(this._boxId),
                itemElm,
                itemId,
                scrollBarElm,
                child = boxElm && boxElm.firstChild;

            //删除boxElm的所有子元素
            while(child){
                child.parentNode.removeChild(child);
                child = boxElm.firstChild;
            }

            for(var i = 0;i < this._items.length;i++){
                itemElm = document.createElement('div');
                itemElm.id = this._boxId + '-item-'+i;
                itemElm.className = this._constrName + '-item';
                if(i !== this._pageIndex - 1){
                    itemElm.style.display = 'none';
                }
                evm.html(itemElm,this._items[i]);
                boxElm.appendChild(itemElm);
            }

            //滚动条
            scrollBarElm = document.createElement('div');
            scrollBarElm.id = this._boxId + '-scrollBar';
            scrollBarElm.className = this._constrName + '-scrollBar';
            evm.html(scrollBarElm,'<span></span>');
            boxElm.appendChild(scrollBarElm);
        },
        _pagePrev:function(){
            var boxElm = document.getElementById(this._boxId),
                childNodes = boxElm.childNodes;

            this._pageIndex--;
            for(var i = 0;i < childNodes.length;i++){
                if(i !== childNodes.length - 1){
                    if(i === this._pageIndex - 1){
                        childNodes[i].style.display = 'block';
                    }
                    else{
                        childNodes[i].style.display = 'none';
                    }
                }
            }
            this._updateScrollBar('UP');
        },
        _pageNext:function(){
            var boxElm = document.getElementById(this._boxId),
                childNodes = boxElm.childNodes;

            this._pageIndex++;
            for(var i = 0;i < childNodes.length;i++){
                if(i !== childNodes.length - 1){
                    if(i === this._pageIndex - 1){
                        childNodes[i].style.display = 'block';
                    }
                    else{
                        childNodes[i].style.display = 'none';
                    }
                }
            }
            this._updateScrollBar('DOWN');
        },
        _updateScrollBar:function(dir){
            var scrollBarElm = document.getElementById(this._boxId+'-scrollBar'),
                barElm = scrollBarElm.firstChild,
                scrollBarHeight,
                barHeight,
                barOffsetTop,
                step;

            if(!scrollBarElm){
                return;
            }
            if(this._pageTotalNum === 0){
                scrollBarElm.style.display = 'none';
            }
            else{
                scrollBarHeight = parseFloat(evm.getStyle(scrollBarElm,'height'));
                barHeight = parseFloat(evm.getStyle(barElm,'height'));
                barOffsetTop = parseFloat(evm.getStyle(barElm,'top')) || 0;

                step = (scrollBarHeight - barHeight)/(this._pageTotalNum - 1);
                if(dir === 'UP'){
                    step = -step;
                }
                barElm.style.top = barOffsetTop + step + 'px';
            }
        },
        _onFocus:function(){
            var boxElm = document.getElementById(this._boxId);
            evm.addClass(boxElm,'active');
        },
        _onBlur:function(){
            var boxElm = document.getElementById(this._boxId);
            evm.removeClass(boxElm,'active');
        }
    });
})();