
/**
 *  @file 行列格子包含分页
 *  @import /swatapps/ape/evm.js
 *  @des 组件接口参考杨涛云组件定义规范
 *      分页分为按钮点击上一页和下一页分页和到达第一个或者最后一项自动分页
 *  todo:目前接口不支持分页数据，所以列表分页是一次读取所有数据:(
 *  todo:按钮点击分页未完成
 */
(function(){
    evm.ext.comp.GridPaging = function(options){
        //强制使用new
        if(this === evm.ext.comp || this === window){
            return new arguments.callee(options);
        }
        //必须传参
        if(evm.isUndefined(options) && !(evm.isObject(options))){
            throw new Error('evm.ext.comp.VListPaging实例化失败：没有传入参数');
        }
        function handler(options){
            // 触发事件的默认方法
            try{
                //console.log('VListBox 事件触发,默认处理方法');
            }catch(E){}
        }

        evm.ext.comp.BaseComponent.call(this,options);

        //====配置属性定义;=========================
        this.config = options;
        //自定义属性
        //容器元素的id属性
        this.box_id = this.config.boxID || this.comp_name;
        this._allRemoteData = this.config.allRemoteData;
        //列表元素容器的id属性
        this._listWrapId = this.box_id+'-list';
        //分页按钮容器的id属性
        this._pagingWrapId = this.box_id+'-paging';
        //列表数据 {Array}
        /**
         *  {
         *      picBlur:'',
         *      picFocus:'',
         *      id:'',
         *      text:'',
         *      redirectUrl:''
         *  }
         */
        this.data = this.config.data;
        //分页属性
        this._pageSize = this.config.pageSize || 4;
        this._pageIndex = this.config.pageIndex || 1;
        this._pageTotalNum = 1;
        //分页操作属性
        //默认达第一个或者最后一项自动分页 可取值'auto','button'
        this._pagingType = this.config.pagingType || 'auto';
        //单页列表数据属性
        this._items = null;
        //单页列表焦点索引
        this._activeIndex = this.config.activeIndex || 0;
        //分页按钮配置
        this._pagingButtonSetting = this.config.pagingButtonSetting || {
            prev:{
                class:'prev',
                text:'上一页',
                enable:true,
                show:false
            },
            next:{
                class:'next',
                text:'下一页',
                enable:true,
                show:true
            },
            pageHome:{
                class:'pageHome',
                text:'第一页',
                enable:false,
                show:false
            },
            home:{
                class:'home',
                text:'回首页',
                enable:false,
                show:false
            },
            back:{
                class:'back',
                text:'返回',
                enable:false,
                show:false
            }
        };
        //当前焦点位于列表或者分页按钮 取值:['list','paging']
        this._activeName = 'list';
        //分页按钮组焦点位置索引
        this._pagingActiveIndex = 0;
        this._pagingEnableItemNum = 0;
        //列表项最大可显示字的数量
        this._maxTextNum = this.config.maxTextNum || 17;

        this._rowsNum = this.config.rowsNum || 3;
        this._colsNum = this.config.colsNum || 5;


        //====自定义事件的回调方法,在创建对象时传入对应的回调方法;=========================
        this.onBlur = this.config.onBlur || handler;
        this.onChange = this.config.onChange || handler;
        this.onChangeOut = this.config.onChangeOut || handler;
        this.onFocus = this.config.onFocus || handler;
        this.onFilter = this.config.onFilter || handler;
        this.onSelect = this.config.onSelect || handler;
        this.onEvent  = this.config.onEvent || handler;
    };
    evm.extend(evm.ext.comp.GridPaging.prototype,{
        action:function(dir,keyEvent){
            var item = this._items[this._activeIndex],
                activeName = this._activeName;

            switch(dir){
                case 'UP':
                    this._onBlur();
                    if(activeName === 'list'){
                        if(this._activeIndex - this._colsNum >= 0){
                            this._activeIndex = this._activeIndex - this._colsNum;
                            this._onFocus();
                            this.onChange(dir,this._activeIndex,this._allRemoteData);
                        }
                        else{
                            if(this._pagingType === 'auto'){
                                if(this._pageIndex - 1 > 0){//向上翻页
                                    if(this._pagingType === 'auto'){
                                        this._pagePrev();
                                        this._onFocus();
                                        this.onChange(dir,this._activeIndex,this._allRemoteData);
                                    }
                                }
                                else{//第一页，跳出
                                    this.onBlur(dir);
                                }
                            }
                            else{
                                this.onBlur(dir);
                            }
                        }
                    }
                    else if(activeName === 'paging'){
                        this._activeName = 'list';
                        this._onFocus();
                    }
                    break;
                case 'DOWN':
                    this._onBlur();
                    if(activeName === 'list') {
                        if (this._activeIndex + this._colsNum < this._items.length) {//单页切换栏目
                            this._activeIndex = this._activeIndex + this._colsNum;
                            this._onFocus();
                            this.onChange(dir, this._activeIndex, this._allRemoteData);
                        }
                        else {
                            if (this._pagingType === 'auto') {
                                if (this._pageIndex + 1 <= this._pageTotalNum) {//向下翻页
                                    this._pageNext();
                                    this._onFocus();
                                    this.onChange(dir, this._activeIndex, this._allRemoteData);
                                }
                                else {//最后一页，跳出
                                    this.onBlur(dir);
                                }
                            }
                            else if (this._pagingType === 'button') {
                                this._updatePaging();
                                this._activeName = 'paging';
                                this._onFocus();
                            }
                        }
                    }
                    else if(activeName === 'paging'){
                        this.onBlur(dir);
                    }
                    break;
                case 'RIGHT':
                    this._onBlur();
                    if(activeName === 'paging'){
                        if(this._pagingActiveIndex + 1 < this._pagingEnableItemNum){
                            this._pagingActiveIndex++;
                            this._onFocus();
                        }
                        else{
                            this.onBlur(dir);
                        }
                    }
                    else if(activeName === 'list'){
                        if(this._activeIndex % this._colsNum + 1 < this._colsNum && this._activeIndex % this._colsNum + 1 < this._items.length){
                            this._activeIndex++;
                            this._onFocus();
                            this.onChange(dir, this._activeIndex, this._allRemoteData);
                        }
                        else{
                            this.onBlur(dir);
                        }
                    }
                    break;
                case 'LEFT':
                    this._onBlur();
                    if(activeName === 'paging'){
                        if(this._pagingActiveIndex - 1 >= 0){
                            this._pagingActiveIndex--;
                            this._onFocus();
                        }
                        else{
                            this.onBlur(dir);
                        }
                    }
                    else if(activeName === 'list'){
                        if(this._activeIndex % this._colsNum - 1 >= 0){
                            this._activeIndex--;
                            this._onFocus();
                            this.onChange(dir, this._activeIndex, this._allRemoteData);
                        }
                        else{
                            this.onBlur(dir);
                        }
                    }
                    break;
                case 'SELECT':
                    this._onSelect();
                    break;
                default:
                    this._onFocus();
                    this.onFocus(dir,this._activeIndex,this._allRemoteData);
                    break;
            }
            return this;
        },
        init:function(remoteData){
            remoteData && (this.data = remoteData);
            if(this.data && this.data.length > 0){
                this.setPageIndex(1);
                this.setActiveIndex(0);
                this._create();
            }
            return this;
        },
        destory:function(){
            var listElm = document.getElementById(this._listWrapId),
                pagingElm = document.getElementById(this._pagingWrapId);

            if(listElm){
                listElm.parentNode.removeChild(listElm);
            }
            if(pagingElm){
                pagingElm.parentNode.removeChild(pagingElm);
            }
        },
        setPageIndex:function(pageIndex){
            var realPageIndex = Math.min(this._pageTotalNum || 0,Math.max(0,pageIndex));
            this._pageIndex = realPageIndex;
        },
        setActiveIndex:function(index){
            var realIndex = Math.min((this._items && this._items.length) || 0,Math.max(0,index));
            this._activeIndex = realIndex;
        },
        setAllRemoteData:function(data){
            this._allRemoteData = data;
        },
        _create:function(){
            var boxElm = document.getElementById(this.box_id),
                listElm = document.getElementById(this._listWrapId),
                liElm,
                item,
                spanElm,
                temp;

            //配置分页数据
            this._pageTotalNum = Math.ceil(this.data.length/this._pageSize);
            this._items = this.data.slice((this._pageIndex-1)*this._pageSize,this._pageIndex*this._pageSize);

            if(listElm){
                listElm.parentNode.removeChild(listElm);
            }

            if(this._items && this._items.length){
                listElm = document.createElement('ul');
                listElm.id = this._listWrapId;
                //加载列表
                for(var i = 0;i < this._items.length;i++){
                    liElm = document.createElement('li');
                    item = this._items[i];
                    if(item.picFocus && item.picBlur){
                        evm.css(liElm,'background-image:url('+ item.picBlur +')');
                    }
                    if(item.text){
                        if(item.text.length > this._maxTextNum){
                            liElm.setAttribute('data-autoscroll','true');
                            evm.html(liElm,'<span style="width:'+ item.text.length +'em">'+item.text+'</span>');
                        }
                        else{
                            evm.html(liElm,'<span>'+item.text+'</span>');
                        }
                    }
                    listElm.appendChild(liElm);
                }
                boxElm.appendChild(listElm);
                this._updatePaging();
            }
        },
        _updatePaging:function(){
            var pagingElm = document.getElementById(this._pagingWrapId),
                boxElm = document.getElementById(this.box_id);
            if(pagingElm){
                pagingElm.parentNode.removeChild(pagingElm);
            }

            this._updatePagingButtonSetting();

            //加载分页按钮
            if(this._pagingType === 'button'){
                pagingElm = document.createElement('div');
                pagingElm.id = this._pagingWrapId;

                this._pagingEnableItemNum = 0;

                for(var prop in this._pagingButtonSetting){
                    if(this._pagingButtonSetting[prop].enable){
                        spanElm = document.createElement('span');
                        evm.addClass(spanElm,this._pagingButtonSetting[prop].class);
                        evm.html(spanElm,this._pagingButtonSetting[prop].text);
                        if(!this._pagingButtonSetting[prop].show){
                            evm.css(spanElm,'visibility:hidden');
                        }
                        pagingElm.appendChild(spanElm);

                        if(this._pagingButtonSetting[prop].show) {
                            this._pagingEnableItemNum += 1;
                        }
                    }
                }
                boxElm.appendChild(pagingElm);
            }
        },
        _onFocus:function(){
            var listElm = document.getElementById(this._listWrapId),
                item = this._items[this._activeIndex],
                pagingActiveElm;
            if(this._activeName === 'list'){
                if(item.picFocus && item.picBlur){
                    evm.css(listElm.childNodes[this._activeIndex],'background-image:url('+ item.picFocus +')');
                }
                else{
                    evm.addClass(listElm.childNodes[this._activeIndex],'active');
                }

                //开启自动滚动文字
                this._stopAutoScroll(listElm.childNodes[this._activeIndex]);
                this._startAutoScroll(listElm.childNodes[this._activeIndex]);
            }
            else if(this._activeName === 'paging'){
                pagingActiveElm = this._findVisiblePagingButton(this._pagingActiveIndex);
                if(pagingActiveElm){
                    evm.addClass(pagingActiveElm,'active');
                }
            }
        },
        _onBlur:function(){
            var listElm = document.getElementById(this._listWrapId),
                item = this._items[this._activeIndex],
                pagingActiveElm;

            if(this._activeName === 'list'){
                if(item.picFocus && item.picBlur){
                    evm.css(listElm.childNodes[this._activeIndex],'background-image:url('+ item.picBlur +')');
                }
                else{
                    evm.removeClass(listElm.childNodes[this._activeIndex],'active');
                }

                //取消自动滚动文字
                this._stopAutoScroll(listElm.childNodes[this._activeIndex]);
            }
            else if(this._activeName === 'paging'){
                pagingActiveElm = this._findVisiblePagingButton(this._pagingActiveIndex);
                if(pagingActiveElm){
                    evm.removeClass(pagingActiveElm,'active');
                }
            }
        },
        _pagePrev:function(){
            this._activeIndex = this._pageSize - 1;
            this._pageIndex--;
            this._create();
        },
        _pageNext:function(){
            this._activeIndex = 0;
            this._pageIndex++;
            this._create();
        },
        /**
         * 根据分页属性配置按钮配置项
         * @private
         */
        _updatePagingButtonSetting:function(){
            var pageIndex = this._pageIndex,
                pageTotalNum = this._pageTotalNum;

            if(pageIndex === 1 && pageIndex === pageTotalNum){
                this._pagingButtonSetting.prev.show = false;
                this._pagingButtonSetting.next.show = false;
            }
            else if(pageIndex === 1 && pageIndex !== pageTotalNum){
                this._pagingButtonSetting.prev.show = false;
                this._pagingButtonSetting.next.show = true;
            }
            else if(pageIndex > 1 && pageIndex === pageTotalNum){
                this._pagingButtonSetting.prev.show = true;
                this._pagingButtonSetting.next.show = false;
            }
            else if(pageIndex > 1 && pageIndex !== pageTotalNum){
                this._pagingButtonSetting.prev.show = true;
                this._pagingButtonSetting.next.show = true;
            }
        },
        _findVisiblePagingButton:function(index){
            var pagingElm = document.getElementById(this._pagingWrapId),
                child = pagingElm && pagingElm.firstChild,
                index = index || 0,
                counter = 0;

            while(child && counter <= index){
                if(child.style.visibility !== 'hidden'){
                    if(index === counter){
                        return child;
                    }
                    else{
                        counter++;
                    }
                }

                child = child.nextSibling;
            }
        },
        _onSelect:function(){
            var pagingActiveElm;

            if(this._activeName === 'paging'){
                pagingActiveElm = this._findVisiblePagingButton(this._pagingActiveIndex);
                if(evm.hasClass(pagingActiveElm,'prev')){
                    this._pagePrev();
                    this._onFocus();
                }else if(evm.hasClass(pagingActiveElm,'next')){
                    this._pageNext();
                    this._onFocus();
                }
            }
            else if(this._activeName === 'list'){
                this.onSelect(this._activeIndex,this._items,this._allRemoteData);
            }
        },
        _startAutoScroll:function(itemWrapElm){
            this._autoScroll(itemWrapElm,0);
        },
        _stopAutoScroll:function(itemWrapElm){
            this._autoScroll(itemWrapElm,1);
        },
        _autoScroll:function(itemWrapElm,type){
            var me = this,
                itemElm = itemWrapElm,
                child = itemElm.firstChild,
                itemElmWidth,
                childWidth,
                interval = 30;

            if(!itemElm.getAttribute('data-autoscroll')){
                return;
            }

            itemElmWidth = itemElm.offsetWidth;
            childWidth = child.offsetWidth;

            function childScroll(){
                var left = child.offsetLeft;

                left = left - 1;

                if(left <= itemElmWidth - childWidth){
                    child.style.left = 0 + 'px';
                }
                else{
                    child.style.left = left + 'px';
                }
                if((left - 1) <= (itemElmWidth - childWidth)){
                    interval = 1000;
                }
                else{
                    interval = 20;
                }
                me._timeId = setTimeout(childScroll,interval);
            }

            if(type === 1){
                clearTimeout(me._timeId);
                child.style.left = 0 + 'px';
                return;
            }
            me._timeId = setTimeout(childScroll,interval);
        }
    });
})();