/**
 * @file 等大小网格列表
 * @import /swatapps/ape/evm.js
 * @param options
 * @constructor
 */
function EqualGridList(options){
    //父元素id属性
    this._boxId = options.boxId;
    //构建class属性
    this._prefix = 'equalGridList';
    //网格数据
    this._data = null;
    //网格焦点索引位置
    this._activeIndex = 0;
    //配置自动分页或手动点击上下页按钮分页
    this._autoPaging = options.autoPaging != null ? options.autoPaging : true;
    //分页属性
    this._pageIndex = 1;
    this._pageSize = 1;
    this._pageTotalNum = 1;
    //每页数据
    this._items = null;

    //网格大小
    this._gridWidth = options.gridWidth;
    this._gridHeight = options.gridHeight;
    this._baseLeft = options.baseLeft || 0;
    this._baseTop = options.baseTop || 0;
    //网格水平和垂直间距
    this._gapH = options.gapH || 0;
    this._gapV = options.gapV || 0;
    //网格列数和行数
    this._cols = options.cols || 1;
    this._rows = options.rows || 1;

    //配置高亮背景层
    this._activeLayerElm;
    this._activeLayerWidth = options.activeLayerWidth;
    this._activeLayerHeight = options.activeLayerHeight;
    this._activeLayerDx = options.activeLayerDx;
    this._activeLayerDy = options.activeLayerDy;

    this.onWidgetBlur = options.onWidgetBlur || function(){};
    this.doSelect = options.doSelect || function(){};
}
EqualGridList.prototype = {
    action:function(dir){
        var ai = this._activeIndex;
        switch(dir){
            case 'UP':
                if(ai - this._cols >= 0){
                    this._onBlur();
                    this._activeIndex -= this._cols;
                    this._onFocus();
                }
                else{
                    this._onBlur();
                    this.onWidgetBlur(dir);
                }
                break;
            case 'RIGHT':
                if(this._autoPaging){
                    if(ai % this._cols + 1 < this._cols && (ai + 1) < this._items.length){
                        this._onBlur();
                        this._activeIndex++;
                        this._onFocus();
                    }
                    else{
                        if(this._pageIndex === this._pageTotalNum && (ai + 1) < this._items.length){
                            this._onBlur();
                            this._activeIndex++;
                            this._onFocus();
                        }
                        else{
                            this._next(dir);
                        }
                    }
                }
                else{
                    if((ai + 1) < this._items.length){
                        this._onBlur();
                        this._activeIndex++;
                        this._onFocus();
                    }
                    else{
                        this._onBlur();
                        this.onWidgetBlur(dir);
                    }
                }
                break;
            case 'DOWN':
                if(ai + this._cols < this._items.length){
                    this._onBlur();
                    this._activeIndex += this._cols;
                    this._onFocus();
                }
                else{
                    this._onBlur();
                    this.onWidgetBlur(dir);
                }
                break;
            case 'LEFT':
                if(this._autoPaging){
                    if(ai % this._cols - 1 >= 0){
                        this._onBlur();
                        this._activeIndex--;
                        this._onFocus();
                    }
                    else{
                        if(this._pageIndex === 1 && (ai - 1) >= 0){
                            this._onBlur();
                            this._activeIndex--;
                            this._onFocus();
                        }
                        else{
                            this._prev(dir);
                        }
                    }
                }
                else{
                    if((ai - 1) >= 0){
                        this._onBlur();
                        this._activeIndex--;
                        this._onFocus();
                    }else{
                        this._onBlur();
                        this.onWidgetBlur(dir);
                    }
                }
                break;
            case 'SELECT':
                this._doSelect();
                break;
            default:
                this._onFocus();
                break;
        }
    },
    init:function(data){
        data && (this._data = data);
        if(!data || data.length < 0){
            throw new Error('未获取列表数据!');
        }

        this._pageSize = this._cols*this._rows;
        this._pageTotalNum = Math.ceil(this._data.length/this._pageSize);

        this._fixPageIndex();
        this._updateItems();
        this._fixActiveIndex();
        this._create();
    },
    getActiveIndex:function(){
        return this._activeIndex;
    },
    setActiveIndex:function(nIndex){
        this._activeIndex = nIndex;
    },
    _fixActiveIndex:function(){
        this._activeIndex = Math.max(0,Math.min(this._items.length-1,this._activeIndex));
    },
    getPageIndex:function(){
        return this._pageIndex;
    },
    setPageIndex:function(nIndex){
        this._pageIndex = nIndex;
    },
    getPageSize:function(){
        return this._pageSize;
    },
    prev:function(){
        var pi = this._pageIndex;
        if(pi - 1 >= 1){
            this._onBlur();
            this._pageIndex--;
            this._updateItems();
            this._create();
            this._activeIndex = this._items.length - 1;
        }
    },
    next:function(){
        var pi = this._pageIndex;
        if(pi + 1 <= this._pageTotalNum){
            this._onBlur();
            this._pageIndex++;
            this._updateItems();
            this._create();
            this._activeIndex = 0;
        }
    },
    _fixPageIndex:function(){
        this._pageIndex = Math.max(1,Math.min(this._pageTotalNum,this._pageIndex));
    },
    _create:function(){
        var boxElm = document.getElementById(this._boxId),
            oPos,
            liElm,
            tempElm;

        while(boxElm.firstChild){
            boxElm.removeChild(boxElm.firstChild);
        }
        for(var i = 0;i < this._items.length;i++){
            liElm = document.createElement('li');
            liElm.className = this._prefix + '-' + 'item';
            liElm.style.position = 'absolute';
            liElm.style.width = this._gridWidth + 'px';
            liElm.style.height = this._gridHeight + 'px';

            //增加图片
            if(this._items[i].picBlur){
                var imgElm = document.createElement('img');
                imgElm.src = this._items[i].picBlur;
                liElm.appendChild(imgElm);
            }

            if(this._items[i].text){
                //增加文字背景层
                var overlayElm = document.createElement('div');
                overlayElm.className = 'overlay';
                liElm.appendChild(overlayElm);

                //增加文字结点
                var textElm = document.createElement('div');
                textElm.className = this._prefix + '-' + 'text';

                var text = this._chMask2Half(this._items[i].text);
                var textLen;
                if(text){
                    var spanElm = document.createElement('span');
                    if((textLen = this._characterCountByCh(text)) > this._maxTextNum){
                        textElm.style.position = 'relative';
                        textElm.setAttribute('data-autoscroll',true);
                        evm.addClass(textElm,'autoscroll');

                        spanElm.style.position = 'absolute';
                        spanElm.style.width = textLen + 'em';
                        spanElm.innerHTML = text;
                    }else{
                        spanElm.innerHTML = text;
                    }
                    textElm.appendChild(spanElm);
                }
                liElm.appendChild(textElm);
            }            

            oPos = this._getPos(i);
            liElm.style.left = oPos.left + 'px';
            liElm.style.top = oPos.top + 'px';

            boxElm.appendChild(liElm);
        }

        //生成高亮背景层
        if(this._activeLayerWidth && this._activeLayerHeight && !this._activeLayerElm){
            this._activeLayerElm = document.createElement('div');
            this._activeLayerElm.style.display = 'none';
            this._activeLayerElm.style.position = 'absolute';
            this._activeLayerElm.className = this._boxId + '-' + this._prefix + '-activeLayer';
            this._activeLayerElm.style.width = this._activeLayerWidth + 'px';
            this._activeLayerElm.style.height = this._activeLayerHeight + 'px';
            document.getElementsByTagName('body')[0].appendChild(this._activeLayerElm);
        }
    },
    _updateItems:function(){
        this._items = this._data.slice((this._pageIndex-1)*this._pageSize,this._pageIndex*this._pageSize);
    },
    _getPos:function(nIndex){
        var oPos = {};

        oPos.left = this._baseLeft + (nIndex % this._cols)*(this._gridWidth+this._gapH);
        oPos.top = this._baseTop + parseInt(nIndex / this._cols)*(this._gridHeight+this._gapV);
        return oPos;
    },
    _onFocus:function(){
        var boxElm = document.getElementById(this._boxId),
            liElm = boxElm.childNodes[this._activeIndex],
            oPos = this._getPos(this._activeIndex);

        evm.addClass(liElm,'focus');

        //设置高亮背景
        if(this._activeLayerElm){
            this._activeLayerElm.style.left = oPos.left - this._activeLayerDx + 'px';
            this._activeLayerElm.style.top = oPos.top - this._activeLayerDy + 'px';
            this._activeLayerElm.style.display = 'block';
        }
    },
    _onBlur:function(){
        var boxElm = document.getElementById(this._boxId),
            liElm = boxElm.childNodes[this._activeIndex],
            oPos = this._getPos(this._activeIndex);

        evm.removeClass(liElm,'focus');

        //设置高亮背景
        if(this._activeLayerElm){
            this._activeLayerElm.style.display = 'none';
        }
    },
    onFocus:function(){
        this._onFocus();
    },
    onBlur:function(){
        this._onBlur();
    },
    _prev:function(dir){
        var pi = this._pageIndex;
        if(pi - 1 >= 1){
            this._onBlur();
            this._pageIndex--;
            this._updateItems();
            this._create();
            this._activeIndex = this._items.length - 1;
            this._onFocus();
        }
        else{
            this._onBlur();
            this.onWidgetBlur(dir);
        }
    },
    _next:function(dir){
        var pi = this._pageIndex;
        if(pi + 1 <= this._pageTotalNum){
            this._onBlur();
            this._pageIndex++;
            this._updateItems();
            this._create();
            this._activeIndex = 0;
            this._onFocus();
        }
        else{
            this._onBlur();
            this.onWidgetBlur(dir);
        }
    },
    _updateBtnsStatus:function(){
        var ptl = this._pageTotalNum,
            pi = this._pageIndex,
            prevElm = document.getElementById(this._prevId),
            nextElm = document.getElementById(this._nextId);

        prevElm.style.display = 'none';
        nextElm.style.display = 'none';

        if(pi - 1 > 0){
            prevElm.style.display = 'block';
        }
        if(pi + 1 <= ptl){
            nextElm.style.display = 'block';
        }
    },
    _doSelect:function(){
        // var item = this._items[this._activeIndex],
            // id = item.id，
            url = this._items[this._activeIndex]['url'];
            this.doSelect(url);
            
        // this.doSelect(id,item);
    },
    //中文标点符号转为半角符号
    _chMask2Half:function(str){
        return str.replace(/[，！。]/g,function(match){
            switch(match){
                case '，':
                    return ',';
                case '！':
                    return '!';
                case '。':
                    return '.';
            }
        });
    },
    //按照中文字符统计字符串长度
    _characterCountByCh:function(str){
        var length = str.length;
        var realLength = 0
        var temLength = 0;
        for (var i = 0; i < length; i++) {
            var charCode = str.charCodeAt(i);
            if (charCode >= 0 && charCode <= 128) {
                temLength += 1;
            } else {
                realLength += 1;
            }
            if (temLength == 2) {
                realLength += 1;
                temLength = 0;
            }
        }
        return realLength;
    },
    _startAutoScroll:function(itemWrapElm){
        this._autoScroll(itemWrapElm,0);
    },
    _stopAutoScroll:function(itemWrapElm){
        this._autoScroll(itemWrapElm,1);
    },
    _autoScroll:function(itemWrapElm,type){
        var me = this,
            itemElm = itemWrapElm,
            child = itemElm.firstChild,
            itemElmWidth,
            childWidth,
            interval = 30;

        if(!itemElm.getAttribute('data-autoscroll')){
            return;
        }

        itemElmWidth = itemElm.offsetWidth;
        childWidth = child.offsetWidth;

        function childScroll(){
            var left = child.offsetLeft;

            left = left - 1;

            if(left <= itemElmWidth - childWidth){
                child.style.left = 0 + 'px';
            }
            else{
                child.style.left = left + 'px';
            }
            if((left - 1) <= (itemElmWidth - childWidth)){
                interval = 1000;
            }
            else{
                interval = 20;
            }
            me._timeId = setTimeout(childScroll,interval);
        }

        if(type === 1){
            clearTimeout(me._timeId);
            child.style.left = 0 + 'px';
            return;
        }
        me._timeId = setTimeout(childScroll,interval);
    },
};
