/**
 *  @file 横向列表不含分页
 *  @import /swatapps/ape/evm.js
 *  @des 组件接口参考杨涛云组件定义规范
 */
(function(){
    evm.ext.comp.HListNoPaging = function(options){
        //强制使用new
        if(this === evm.ext.comp || this === window){
            return new arguments.callee(options);
        }
        //必须传参
        if(evm.isUndefined(options) && !(evm.isObject(options))){
            throw new Error('evm.ext.comp.HListNoPaging实例化失败：没有传入参数');
        }
        function handler(options){
            // 触发事件的默认方法
            try{
                //console.log('VListBox 事件触发,默认处理方法');
            }catch(E){}
        }

        evm.ext.comp.BaseComponent.call(this,options);

        //====配置属性定义;=========================
        this.config = options;
        //自定义属性
        //容器元素的id属性
        this.box_id = this.config.boxID || this.comp_name;
        this._allRemoteData = this.config.allRemoteData;
        //列表数据 {Array}
        /**
         *  {
         *      picBlur:'',
         *      picFocus:'',
         *      id:'',
         *      text:'',
         *      redirectUrl:''
         *  }
         */
        this.data = this.config.data;
        //容器中列表项的最大个数
        this.max_list_num = this.config.maxLists || 6;
        //焦点元素索引
        this._activeIndex = 0;

        //====自定义事件的回调方法,在创建对象时传入对应的回调方法;=========================
        this.onBlur = this.config.onBlur || handler;
        this.onChange = this.config.onChange || handler;
        this.onChangeOut = this.config.onChangeOut || handler;
        this.onFocus = this.config.onFocus || handler;
        this.onFilter = this.config.onFilter || handler;
        this.onSelect = this.config.onSelect || handler;
        this.onEvent  = this.config.onEvent || handler;
    };

    evm.extend(evm.ext.comp.HListNoPaging.prototype,{
        action:function(dir,keyEvent){
            switch(dir){
                case 'UP':
                    this._onBlur();
                    this.onBlur(dir);
                    break;
                case 'DOWN':
                    this._onBlur();
                    this.onBlur(dir);
                    break;
                case 'RIGHT':
                    this._onBlur();
                    if(this._activeIndex + 1 < this.data.length){
                        this._activeIndex++;
                        this.onFocus(dir,this._activeIndex,this._allRemoteData);
                    }
                    this._onFocus();
                    break;
                case 'LEFT':
                    this._onBlur();
                    if(this._activeIndex - 1 >= 0){
                        this._activeIndex--;
                        this.onFocus(dir,this._activeIndex,this._allRemoteData);
                    }
                    this._onFocus();
                    break;
                case 'SELECT':
                    this.onSelect(dir,this._activeIndex,this._allRemoteData,this.data);
                    break;
                default:
                    this._onFocus();
                    this.onFocus(dir,this._activeIndex,this._allRemoteData);
                    break;
            }
            return this;
        },
        init:function(remoteData){
            remoteData && (this.data = remoteData);
            if(this.data && this.data.length > 0){
                this.data = this.data.slice(0,this.max_list_num);
                this._create();
            }
            return this;
        },
        setActiveIndex:function(index){
            var realIndex = Math.min(this.data.length,Math.max(0,index));
            this._activeIndex = realIndex;
            return this;
        },
        setActiveIndexLast:function(){
            this._activeIndex = this.data.length - 1;
            return this;
        },
        getActiveIndex:function(){
            return this._activeIndex;
        },
        setAllRemoteData:function(data){
            this._allRemoteData = data;
        },
        getAllRemoteData:function(){
            return this._allRemoteData;
        },
        _create:function(){
            var me = this,
                boxElm = document.getElementById(this.box_id),
                prefix = 'vListNoPaging-item',
                listData = me.data,
                itemElm,
                item;
            for(var i = 0;i < listData.length;i++){
                item = listData[i];
                itemElm = document.createElement('div');
                evm.addClass(itemElm,prefix);
                itemElm.id = this.box_id + '-' + prefix + '-' + i;
                //列表背景有提供图片
                if(item.picBlur && item.picFocus){
                    evm.css(itemElm,'background-image:url('+ item.picBlur +')');
                    this.data[i].withPic = true;
                }
                else{
                    evm.html(itemElm,item.text);
                }
                boxElm.appendChild(itemElm);
            }
        },
        _onFocus:function(){
            var item = this.data[this._activeIndex],
                prefix = this.box_id + '-' +'vListNoPaging-item-',
                itemElem = document.getElementById(prefix+this._activeIndex);

            if(item.withPic){
                evm.css(itemElem,'background-image:url('+ item.picFocus +')');
            }

            evm.addClass(itemElem,'active');
        },
        _onBlur:function(){
            var item = this.data[this._activeIndex],
                prefix = this.box_id + '-' +'vListNoPaging-item-',
                itemElem = document.getElementById(prefix+this._activeIndex);

            if(item.withPic){
                evm.css(itemElem,'background-image:url('+ item.picBlur +')');
            }

            evm.removeClass(itemElem,'active');
        }
    });
})();