/**
 * @file 文字垂直列表
 * @import /swatapps/ape/evm.js
 */

function VTextList(options){
    //焦点位置
    this._activeIndex = 0;

    //配置自动分页或手动点击上下页按钮分页
    this._autoPaging = options.autoPaging != null ? options.autoPaging : true;

    //用于构建class属性
    this._prefix = 'vTextList';
    //父元素Id
    this._boxId = options.boxId;
    //父元素
    this._boxElm = null;
    //分页属性
    this._pageSize = options.pageSize || 1;
    this._pageIndex = 1;
    this._pageTotalNum = 1;

    //全部列表数据 {Array}
    this._data = options.data || null;
    //当前页列表数据
    this._items = null;

    //列表最大显示文字个数
    this._maxTextNum = options.maxTextNum || 8;

    //配置方法
    this.onWidgetBlur = options.onWidgetBlur || function(){};
    this.onWidgetFocus = options.onWidgetFocus || function(){};
    this.doSelect = options.doSelect || function(){};
    this.onChange = options.onChange || function(){};
    this.onWidgetInit = options.onWidgetInit || function(){};
}
VTextList.prototype = {
    init:function(listData){
        this._boxElm = document.getElementById(this._boxId);
        if(!this._boxElm){
            throw new Error('父元素Id未配置');
        }

        listData && (this._data = listData);

        if(listData instanceof Array){
            this._pageTotalNum = Math.ceil(this._data.length/this._pageSize);

            this._fixPageIndex();
            this._updateItems();
            this._fixActiveIndex();
            this._create();
            this.onWidgetInit(this._items[this._activeIndex]);
        }
        else{
            throw TypeError();
        }
        return this;
    },
    //组件内控制按键动作方法
    action:function(dir){
        switch(dir){
            case 'UP':
                this._moveV(-1,dir);
                break;
            case 'RIGHT':
                this._moveH(1,dir);
                break;
            case 'DOWN':
                this._moveV(1,dir);
                break;
            case 'LEFT':
                this._moveH(-1,dir);
                break;
            case 'SELECT':
                this._doSelect();
                break;
            default:
                this._onFocus();
                break;
        }
    },
    getPageIndex:function(){
        return this._pageIndex;
    },
    //设置焦点位置，不做边界判断
    setActiveIndex:function(nIndex){
        this._activeIndex = nIndex;
    },
    _fixActiveIndex:function(){
        this._activeIndex = Math.max(0,Math.min(this._items.length-1,this._activeIndex));
    },
    getActiveIndex:function(){
        return this._activeIndex;
    },
    //设置分页位置，不做边界判断
    setPageIndex:function(nIndex){
        this._pageIndex = nIndex;
    },
    _fixPageIndex:function(){
        this._pageIndex = Math.max(1,Math.min(this._pageTotalNum,this._pageIndex));
    },
    //生成DOM节点
    _create:function(){
        var listWrapElm,
            divElm,
            spanElm,
            text,
            textLen;

        //处理列表
        listWrapElm = document.getElementById(this._boxId+ '-' +this._prefix+'-'+'listWrap');
        if(listWrapElm){
            this._removeChildren(listWrapElm);
        }
        else{
            listWrapElm = document.createElement('div');
            listWrapElm.id = this._boxId + '-' + this._prefix + '-listWrap';
            this._boxElm.appendChild(listWrapElm);
        }

        for(var i =0;i < this._items.length;i++){
            divElm = document.createElement('div');
            divElm.className = this._prefix+'item';
            divElm.style.position = 'relative';

            text = this._chMask2Half(this._items[i].text);
            if(text){
                spanElm = document.createElement('span');
                if((textLen = this._characterCountByCh(text)) > this._maxTextNum){
                    divElm.setAttribute('data-autoscroll',true);
                    evm.addClass(divElm,'autoscroll');
                    spanElm.style.width = textLen + 'em';
                    //spanElm.innerText = text;
                    spanElm.innerHTML = text;
                }else{
                    //spanElm.innerText = text;
                    spanElm.innerHTML = text;
                }
                divElm.appendChild(spanElm);
            }
            listWrapElm.appendChild(divElm)
        }
    },
    //更新当前页列表数据
    _updateItems:function(){
        this._items = (this._data && this._data.slice((this._pageIndex-1)*this._pageSize,this._pageIndex*this._pageSize)) || [];
    },
    _startAutoScroll:function(itemWrapElm){
        this._autoScroll(itemWrapElm,0);
    },
    _stopAutoScroll:function(itemWrapElm){
        this._autoScroll(itemWrapElm,1);
    },
    _autoScroll:function(itemWrapElm,type){
        var me = this,
            itemElm = itemWrapElm,
            child = itemElm.firstChild,
            itemElmWidth,
            childWidth,
            interval = 30;

        if(!itemElm.getAttribute('data-autoscroll')){
            return;
        }

        itemElmWidth = itemElm.offsetWidth;
        childWidth = child.offsetWidth;

        function childScroll(){
            var left = child.offsetLeft;

            left = left - 1;

            if(left <= itemElmWidth - childWidth){
                child.style.left = 0 + 'px';
            }
            else{
                child.style.left = left + 'px';
            }
            if((left - 1) <= (itemElmWidth - childWidth)){
                interval = 1000;
            }
            else{
                interval = 20;
            }
            me._timeId = setTimeout(childScroll,interval);
        }

        if(type === 1){
            clearTimeout(me._timeId);
            child.style.left = 0 + 'px';
            return;
        }
        me._timeId = setTimeout(childScroll,interval);
    },
    //中文标点符号转为半角符号
    _chMask2Half:function(str){
        return str.replace(/[，！。]/g,function(match){
            switch(match){
                case '，':
                    return ',';
                case '！':
                    return '!';
                case '。':
                    return '.';
            }
        });
    },
    //按照中文字符统计字符串长度
    _characterCountByCh:function(str){
        var length = str.length;
        var realLength = 0
        var temLength = 0;
        for (var i = 0; i < length; i++) {
            var charCode = str.charCodeAt(i);
            if (charCode >= 0 && charCode <= 128) {
                temLength += 1;
            } else {
                realLength += 1;
            }
            if (temLength == 2) {
                realLength += 1;
                temLength = 0;
            }
        }
        return realLength;
    },
    //删除子节点
    _removeChildren:function(parent){
        while(parent.firstChild){
            parent.removeChild(parent.firstChild);
        }
    },
    _onFocus:function(){
        var listWrapElm;

        listWrapElm = document.getElementById(this._boxId+ '-' +this._prefix+'-'+'listWrap');
        evm.addClass(listWrapElm.childNodes[this._activeIndex],'focus');

        this._stopAutoScroll(listWrapElm.childNodes[this._activeIndex]);
        this._startAutoScroll(listWrapElm.childNodes[this._activeIndex]);
    },
    _onBlur:function(){
        var listWrapElm;

        listWrapElm = document.getElementById(this._boxId + '-' + this._prefix+'-'+'listWrap');
        evm.removeClass(listWrapElm.childNodes[this._activeIndex],'focus');

        this._stopAutoScroll(listWrapElm.childNodes[this._activeIndex]);
    },
    _moveV:function(nStep,dir){
        var lai;

        lai = this._activeIndex;
        lai += nStep;

        if(lai >= 0 && (lai < this._items.length)){
            this._onBlur();
            this._activeIndex = lai;
            this._onFocus();
            this._onChange();
        }
        else if(dir === 'UP'){
            this._autoPaging && this._prev(dir);
        }
        else if(dir === 'DOWN'){
            this._autoPaging && this._next(dir);
        }
        
    },
    _moveH:function(nStep,dir){
        this._onBlur();
        this.onWidgetBlur(dir);
    },
    _doSelect:function(){
    },
    _onChange:function(){
        var item = this._items[this._activeIndex];
        this.onChange(item,this._activeIndex);
    },
    _prev:function(dir){
        if(this._pageIndex >= 2){
            this._onBlur();
            this._pageIndex--;
            this._fetchListData();
            this._create();

            this._activeIndex = this._items.length - 1;
            this._onFocus();
            this._onChange();
        }
        else{
            this._onBlur();
            this.onWidgetBlur(dir);
        }
    },
    _next:function(dir){
        if(this._pageIndex < this._pageTotalNum){
            this._onBlur();
            this._pageIndex++;
            this._fetchListData();
            this._create();

            this._activeIndex = 0;
            this._onFocus();
            this._onChange();
        }
        else{
            this._onBlur();
            this.onWidgetBlur(dir);
        }
    },
    prev:function(){
        this._prev();
    },
    next:function(){
        this._next();
    },
    //为异步加载分页数据准备
    _fetchListData:function(){
        this._updateItems();
    },
    _doSelect:function(){
        var item = this._items[this._activeIndex];
        this.doSelect.call(this,item);
    }
};
